#include "altqfile.h"

#include <QDir>
#include <QProcess>

AltQFile::AltQFile(QObject *parent) : QFile(parent)
{
}

AltQFile::AltQFile(const QString & name) : QFile(name)
{
}

AltQFile::AltQFile(const QString & name, QObject *parent) : QFile(name, parent)
{
}

bool AltQFile::copy(const QString & newName)
{
    bool result = false;
    bool useCmd = false;

    if (QDir::separator() == '/') {
        useCmd = true;
    } else {
        useCmd = false;
    }

    if (useCmd) {

        QProcess process;

        QStringList args;
        args << "-p";
        args << fileName();
        args << newName;

        process.start("cp", args);
        process.waitForFinished(-1);

        if (process.exitCode() == 0) {
            result = true;
        } else {
            result = false;
        }

    } else {
        result = QFile::copy(newName);
    }

    return result;
}

bool AltQFile::exists(const QString & name)
{
    QString n = name.trimmed();
    QChar sep = QDir::separator();

    if (QDir::isAbsolutePath(n)){
        return QFile::exists(n);
    }

    bool result = false;

    QStringList paths;
    QString envPath(qgetenv("PATH"));

    if (sep == '/') {
        paths = envPath.split(":");
    } else {
        envPath.remove(QRegExp("^\\w+="));
        paths = envPath.split(";");
    }

    paths << "";

    foreach(QString path, paths){

        path = path.trimmed();

        if (!path.isEmpty() && !path.endsWith(sep)){
            path.append(sep);
        }

        QString f = path.append(n);

        if (QFile::exists(f)){
            result = true;
            break;
        }
    }

    return result;
}
