#include "anybox.h"
#include "myabcfile.h"
#include "global.h"
#include "myabcfolder.h"
#include "remark.h"

#include <QDir>
#include <QTextStream>

AnyBox::AnyBox(const QString &name, MyUnit* parent) : MyFolder(name, parent)
{
    m_dateFormat = "dd.MM.yyyy"; 
    m_useSubFile = false;

    QString abcFilter;
    for (int i = ABC_WIDTH; i > 0; i--){ abcFilter += ABC_CHAR; }

    m_count = 0;

    QDir sa(completeName());
    QFileInfoList a = sa.entryInfoList(QStringList(abcFilter),QDir::Dirs);
    if (!a.isEmpty()) {
        foreach(QFileInfo tmpA, a){
            MyAbcFolder* fa = new MyAbcFolder(tmpA.baseName(),this);
            QDir sb(tmpA.absoluteFilePath());
            QFileInfoList b = sb.entryInfoList(QStringList(abcFilter),QDir::Dirs);
            foreach(QFileInfo tmpB, b) {
                MyAbcFolder* fb = new MyAbcFolder(tmpB.baseName(),fa);
                MyAbcFile* abcFile = new MyAbcFile(fb);

                connect(abcFile, SIGNAL(markAbcFile(MyAbcFile*,int)), this, SLOT(markAbcFile(MyAbcFile*,int)));
                connect(abcFile, SIGNAL(unmarkAbcFile(MyAbcFile*,int)), this, SLOT(unmarkAbcFile(MyAbcFile*,int)));

                m_count += abcFile->countEntries();
            }
        }
    }    
}

void AnyBox::setExt(const QString &s)
{
    m_ext.clear();

    QStringList exts;
    exts = s.split(",");

    foreach(QString e, exts){
        QString et = e.trimmed();
        if (!et.isEmpty()){
            m_ext << et.toLower();
        }
    }
}

void AnyBox::setExtPlayList(const QString &s)
{
    m_extPlayList = s;
}

void AnyBox::setExecPreview(const QString &s)
{
    m_execPreview = s;
}

void AnyBox::setExecPresent(const QString &s)
{
    m_execPresent = s;
}

void AnyBox::setDateFormat(const QString &s)
{
    m_dateFormat = s;
}

QSet<QString> AnyBox::ext()
{
    return m_ext;
}

QString AnyBox::extPlayList()
{
    return m_extPlayList;
}

QString AnyBox::execPreview()
{
    return m_execPreview;
}

QString AnyBox::execPresent()
{
    return m_execPresent;
}

QString AnyBox::dateFormat()
{
    return m_dateFormat;
}

quint32 AnyBox::getCount()
{
    return m_count;
}

void AnyBox::addCount(quint32 c)
{
    m_count += c;
}

quint32 AnyBox::startValueCategory(MySourceFile *file)
{
    quint32 result = 0;

    QDate dateSrcFile;
    dateSrcFile = file->exifDate();

    switch (dateSrcFile.month()){
    case 3:
    case 4:
    case 5:
        result += 1 << 21;
        break;
    case 6:
    case 7:
    case 8:
        result += 2 << 21;
        break;
    case 9:
    case 10:
    case 11:
        result += 3 << 21;
        break;
    default:
        ;
    }

    QTime timeSrcFile;
    timeSrcFile = file->exifTime();

    QTime night_morning(6,0);
    QTime morning_day(12,0);
    QTime day_evening(18,0);
    QTime evening_night(0,0);

    if ((timeSrcFile >= evening_night) && (timeSrcFile < night_morning)){

    } else {
        if ((timeSrcFile >= night_morning) && (timeSrcFile < morning_day)){
            result += 1 << 19;
        } else {
            if ((timeSrcFile >= morning_day) && (timeSrcFile < day_evening)){
                result += 2 << 19;
            } else {
                result += 3 << 19;
            }
        }
    }

    return result;
}

void AnyBox::makeSubFile(QString s)
{

}

Category AnyBox::category()
{
    return m_category;
}

QString AnyBox::makePlayListFile(const QString &filename, QList<Entry*> entries)
{    
    QString sldFilename;
    sldFilename = unit(NAME_SLD_FOLDER)->completeName() + QDir::separator() + filename + "." + m_extPlayList;
    QFile sldFile(sldFilename);

    QTextStream st(&sldFile);

    if (sldFile.open(QIODevice::WriteOnly | QIODevice::Text)){

        int n = 0;

        foreach(Entry* entry, entries){

            if (n > MAX_NUM_OF_SHOW) {
                emit showReduce();
                break;
            }

            Remark r(entry->getField(REMARK));
            QString o(entry->getField(ORIGINALFILENAME));
            QString str = completeName() + QDir::separator() + r.re() + QDir::separator() + r.ma() +
                          QDir::separator() + r.rk() + "." + o.section(".",-1,-1);
            st << str << "\n";
            n++;
        }

        sldFile.close();
    }

    return sldFilename;
}

QString AnyBox::makePlayListFile(const QString &filename)
{
    QString sldFilename;
    sldFilename = unit(NAME_SLD_FOLDER)->completeName() + QDir::separator() + filename + "." + m_extPlayList;
    QFile sldFile(sldFilename);

    QTextStream st(&sldFile);

    if (sldFile.open(QIODevice::WriteOnly | QIODevice::Text)){

        int n = 0;

        foreach(Entry* entry, markedEntries()){

            if (n > MAX_NUM_OF_SHOW) {
                emit showReduce();
                break;
            }

            Remark r(entry->getField(REMARK));
            QString o(entry->getField(ORIGINALFILENAME));
            QString str = completeName() + QDir::separator() + r.re() + QDir::separator() + r.ma() +
                          QDir::separator() + r.rk() + "." + o.section(".",-1,-1);
            st << str << "\n";
            n++;
        }

        sldFile.close();
    }

    return sldFilename;
}

QString AnyBox::previewSelected(const QString &filename, QProcess *process)
{    
    if (markedEntries().isEmpty()) { return ""; }
    QString sldFilename;
    sldFilename = makePlayListFile(filename, markedEntries());
    //process->start(m_execPreview + " " + sldFilename);
    process->start(m_execPreview.arg(sldFilename));
    return sldFilename;
}

QString AnyBox::previewSelected(int subset, const QString &filename, QProcess *process)
{    
    if (markedEntries(subset).isEmpty()) { return ""; }
    QString sldFilename;
    sldFilename = makePlayListFile(filename, markedEntries(subset));
    //process->start(m_execPreview + " " + sldFilename);
    process->start(m_execPreview.arg(sldFilename));
    return sldFilename;
}

QString AnyBox::presentSelected(const QString &filename, QProcess *process)
{
    QString sldFilename;
    sldFilename = makePlayListFile(filename, markedEntries());
    //process->start(m_execPresent + " " + sldFilename);    
    process->start(m_execPresent.arg(sldFilename));
    return sldFilename;
}

QString AnyBox::previewSelected(QProcess *process)
{ 
    QTime curTime;
    curTime = QTime::currentTime();
    QString sldFilename(curTime.toString("hhmmsszzz"));
    return previewSelected(sldFilename, process);
}

QString AnyBox::previewSelected(int subset, QProcess *process)
{
    QTime curTime;
    curTime = QTime::currentTime();
    QString sldFilename(curTime.toString("hhmmsszzz"));    
    return previewSelected(subset, sldFilename, process);
}

QString AnyBox::presentSelected(QProcess *process)
{
    QTime curTime;
    curTime = QTime::currentTime();
    QString sldFilename(curTime.toString("hhmmsszzz"));
    return presentSelected(sldFilename, process);
}

QList<Entry*> AnyBox::markedEntries()
{
    QList<Entry*> result;
    foreach(MyAbcFile* abcFile, markedAbcFiles()){
        result.append(abcFile->markedEntries());
    }
    return result;
}

QList<Entry*> AnyBox::markedEntries(int s)
{
    QList<Entry*> result;
    foreach(MyAbcFile* abcFile, markedAbcFiles(s)){
        result.append(abcFile->markedEntries(s));
    }
    return result;
}

void AnyBox::storeMarkedEntries()
{
    foreach(MyAbcFile* abcFile, markedAbcFiles()){
        abcFile->store();
    }
}

int AnyBox::markedEntriesCount()
{
    int t = 0;
    foreach(MyAbcFile* abcFile, markedAbcFiles()){
        t = t + abcFile->markedEntriesCount();
    }
    return t;
}

int AnyBox::markedEntriesCount(int subset)
{
    int t = 0;

    foreach(MyAbcFile* abcFile, markedAbcFiles(subset)){
        t = t + abcFile->markedEntriesCount(subset);
    }

    return t;
}

QSet<MyAbcFile*> AnyBox::markedAbcFiles()
{
    QSet<MyAbcFile*> result;
    foreach(QSet<MyAbcFile*> subset, m_subsetsOfMarkedAbcFiles){
        result.unite(subset);
    }
    return result;
}

QSet<MyAbcFile*> AnyBox::markedAbcFiles(int subset)
{
    QSet<MyAbcFile*> result;

    if (subset < m_subsetsOfMarkedAbcFiles.count()){
        result = m_subsetsOfMarkedAbcFiles.at(subset);
    }

    return result;
}

void AnyBox::markAbcFile(MyAbcFile* abcFile, int subset)
{
    if (subset < m_subsetsOfMarkedAbcFiles.count()){
        m_subsetsOfMarkedAbcFiles[subset] << abcFile;
    } else {
        QSet<MyAbcFile*> s;
        s << abcFile;
        m_subsetsOfMarkedAbcFiles.append(s);
    }
}

void  AnyBox::unmarkAbcFile(MyAbcFile* abcFile, int subset)
{
    if (subset < m_subsetsOfMarkedAbcFiles.count()){
        m_subsetsOfMarkedAbcFiles[subset].remove(abcFile);

        if (m_subsetsOfMarkedAbcFiles[subset].isEmpty()){
            m_subsetsOfMarkedAbcFiles.removeAt(subset);
        }
    }        
}

void AnyBox::unmarkAllEntries()
{
    foreach(MyAbcFile* abcFile, markedAbcFiles()){ 
        abcFile->deleteSubsets();
    }
    m_subsetsOfMarkedAbcFiles.clear();
}

QSet<MyAbcFile*> AnyBox::markAllEntries(int subset)
{
    QSet<MyAbcFile*> abcFiles;
    int s = subset;
    foreach(MyAbcFile* abcFile, findChildren<MyAbcFile*>(QRegExp("^" + NAME_ABC_FILE + "_"))){
        abcFile->markAllEntries(s);
        abcFiles << abcFile;
    }

    return abcFiles;
}

void AnyBox::memorizeMarked()
{    
    m_subsetsOfMarkedAbcFilesMemory = m_subsetsOfMarkedAbcFiles;
    foreach(MyAbcFile* abcFile, markedAbcFiles()){
        abcFile->memorizeSubsets();
    }
}

void AnyBox::restoreMarked()
{
    unmarkAllEntries();
    m_subsetsOfMarkedAbcFiles = m_subsetsOfMarkedAbcFilesMemory;
    foreach(MyAbcFile* abcFile, markedAbcFiles()){
        abcFile->restoreSubsetsFromMemory();
    }
    m_subsetsOfMarkedAbcFilesMemory.clear();
}

void AnyBox::deleteSubset(int subset)
{
    if (subset < m_subsetsOfMarkedAbcFiles.count()){        
        foreach(MyAbcFile* abcFile, m_subsetsOfMarkedAbcFiles.at(subset)){
            abcFile->deleteSubset(subset);
        }
    }
}

void AnyBox::uniteSubsets()
{
    QSet<MyAbcFile*> sub;
    foreach(QSet<MyAbcFile*> s, m_subsetsOfMarkedAbcFiles){
        sub.unite(s);
    }
    m_subsetsOfMarkedAbcFiles.clear();
    m_subsetsOfMarkedAbcFiles << sub;
    foreach (MyAbcFile* abcFile, sub){
        abcFile->uniteSubsets();
    }
}

void AnyBox::setUseSubFile(bool use)
{
    m_useSubFile = use;
}

bool AnyBox::useSubFile()
{
    return m_useSubFile;
}

void AnyBox::setMyCategories(const QString &cat)
{      
    m_myCategories = cat;    
}
/*
QString AnyBox::myCategories()
{

}
*/
void AnyBox::setCategoryLabels()
{

}

void AnyBox::setBoxPixmap(const QPixmap pixmap)
{
    m_boxPixmap = pixmap;
}

void AnyBox::setWelcomeLabel(const QString label)
{
    m_welcomeLabel = label;
}

QPixmap AnyBox::boxPixmap()
{
    return m_boxPixmap;
}

QString AnyBox::welcomeLabel()
{
    return m_welcomeLabel;
}

