#include "arrangement.h"
#include "global.h"
#include "remark.h"
#include "myabcfolder.h"
#include "myabcfile.h"
#include "altqfile.h"

#include <QDateTime>
#include <QFile>
#include <QDir>
#include <QKeyEvent>
#include <QFileInfo>

Arrangement::Arrangement(AnyBox* b, QFrame* w) : QFrame(w)
{
    m_box = b;
    setupUi(w);

    boxPixmap->setPixmap(m_box->boxPixmap());

    src = static_cast<MySrcFolder*>(m_box->unit(NAME_SRC_FOLDER));
    sss = static_cast<MySssFile*>(m_box->unit(NAME_SSS_FILE));
    log = static_cast<MyLogFile*>(m_box->unit(NAME_LOG_FILE));

    m_box->memorizeMarked();

    QDateTime ms;
    qsrand(ms.currentMSecsSinceEpoch());

    for(char i = 'a'; i <= 'z'; i++){
        for(char j = 'a'; j <= 'z'; j++){
            abcNames << QString(i) + QString(j);
        }
    }

    capacity = abcNames.count();

    currentSession = sss->currentSessionNumber();
    sessionLabel->setText(currentSession);

    //.oO0
    annotationEdit->setFocus();

    label_O->hide();;

    label_B->hide(); variousLabel->hide();

    label_C->hide();
    duplicateProgress->hide();

    label_D->hide(); newLabel->hide();
    label_E->hide(); duplicateLabel->hide();

    label_F->hide();

    label_G->hide();
    arrangementProgress->hide();

    label_P->hide();
    label_Q->hide();
    resembleProgress->hide();

    label_H->hide();    
    label_I->hide(); totalLabel->hide();
    label_J->hide(); addLabel->hide();
    label_K->hide(); remainsLabel->hide();
    label_L->hide(); beforeLabel->hide();
    label_M->hide(); afterLabel->hide();
    label_N->hide(); timeLabel->hide();
    // 0Oo.

    annotationEdit->installEventFilter(this);
    connect(startButton, SIGNAL(clicked()), this, SLOT(pressStartButton()));    
}

bool Arrangement::eventFilter(QObject *target, QEvent *event)
{
    bool result;
    result = false;

    if (target == annotationEdit){
        if (event->type() == QEvent::KeyPress) {
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
            switch (keyEvent->key()){
            case Qt::Key_Enter:
            case Qt::Key_Return:
                startButton->click();
                result = true;
                break;
            default:;
            }
        }
    }
    return result;
}

void Arrangement::pressStartButton()
{
    QApplication::setOverrideCursor(Qt::WaitCursor);

    m_box->unmarkAllEntries();

    // .oO0
    startButton->setDisabled(true);
    annotationEdit->setReadOnly(true);
    label_O->show();
    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
    src->refresh();
    label_O->hide();
    // 0Oo.

    log->openLog(QString("ARRANGEMENT"));

    *log << "Total sessions: " + sss->countSessions();
    *log << "Current session number: " + currentSession;


    QMap <QString,QString> tmb; // tmb[контрольная сумма source-файла] = имя source-файла
    QMap <QString,QDate> tmc;   // tmc[имя source-файла] = дата dd.mm.yyyy
    QMap <QString,QTime> tmt;   // tmt[имя source-файла] = время hh.mm

    int countSourceFiles = 0;
    QSet <MySourceFile*> correctSourceFiles;

    QTime elapsedTime;
    elapsedTime.start();

    foreach (MySourceFile* tmp, src->getInConformityExt(m_box->ext())){
        QString fileName;
        fileName = tmp->objectName();
        QString fullFileName;
        fullFileName = tmp->completeName();
        QFile currentFile(fullFileName);
        if (currentFile.open(QIODevice::ReadOnly)) {
            QString key = tmp->checkSum();
            if (!tmb.contains(key)){
                tmb[key] = fileName;
                tmc[fileName] = tmp->exifDate();
                tmt[fileName] = tmp->exifTime();
                correctSourceFiles.insert(tmp);
            }
            currentFile.close();
        }
    }
    countSourceFiles = correctSourceFiles.count();

    // .oO0
    label_B->show();
    variousLabel->setText(QString::number(countSourceFiles));
    variousLabel->show();
    // 0Oo.


    QStringList freeRemarks;
    if (countSourceFiles > 0){

        // .oO0
        label_C->show();
        duplicateProgress->setMinimum(0);
        if (m_box->getCount() > 0) {
            duplicateProgress->setMaximum(m_box->getCount());
            duplicateProgress->setValue(0);
        } else {
            duplicateProgress->setValue(duplicateProgress->maximum());
        }

        duplicateProgress->show();
        label_D->show();
        newLabel->setText(QString::number(correctSourceFiles.count()));
        newLabel->show();        
        label_E->show();
        duplicateLabel->setText(QString::number(countSourceFiles-correctSourceFiles.count()));
        duplicateLabel->show();
        label_F->show();
        // 0Oo.

        QMap <QString,QString> tma;
        QMap <QString,QDate> tmd;

        int i = 1;
        int c = 1;
        foreach (MyAbcFile* abcFile, m_box->findChildren<MyAbcFile*>(QRegExp("^" + NAME_ABC_FILE + "_"))){
            QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
            if (abcFile->initOk()){
                tma.clear();
                tmd.clear();
                QString rema = abcFile->rema();
                QList<Entry> lines = abcFile->getAll();
                foreach (QString tmp, abcSubtract(abcFile->getUsedName())){
                    freeRemarks << rema + tmp;
                }
                foreach(Entry abcLine, lines){
                    tma[abcLine.getField(CHECKSUM)] = abcLine.getField(REMARK);
                    tmd[abcLine.getField(REMARK)] = QDate::fromString(abcLine.getField(DATEOFCREATION),m_box->dateFormat());
                    // .oO0
                    duplicateProgress->setValue(c++);
                    // 0Oo.
                }
                foreach (MySourceFile *pSrcFile, correctSourceFiles){
                    QString srcFileCheckSum(pSrcFile->checkSum());                    
                    QString name(pSrcFile->name());
                    QString dt(pSrcFile->objectName());
                    if (tmd.contains(dt)){
                        tmc[name] = tmd[dt];
                        pSrcFile->setExifDate(tmd[dt]);
                    }
                    if (tma.contains(srcFileCheckSum)){
                        *log << QString::number(i).rightJustified(5) +
                                " Duplicate file " + name + " and " + tma[srcFileCheckSum] +
                                " ...";
                        i++;
                        tmc.remove(name);
                        tmt.remove(name);
                        tmb.remove(srcFileCheckSum);
                        correctSourceFiles.remove(pSrcFile);
                        // .oO0
                        newLabel->setText(QString::number(correctSourceFiles.count()));
                        duplicateLabel->setText(QString::number(countSourceFiles-correctSourceFiles.count()));                        
                        // 0Oo.
                    }
                }
            } else {                
                *log << "Can not use entry file : " + abcFile->completeName();
            }
        }
    }

    int countNewFiles = correctSourceFiles.count();
    *log << "Found " + QString::number(countSourceFiles) + " files: " +
            QString::number(countNewFiles) + " new files and " +
            QString::number(countSourceFiles-countNewFiles) + " duplicate files";

    int needRemarks =  countNewFiles - freeRemarks.count();

    bool over;
    if (needRemarks > 0){
        over = false;
    } else {
        QStringList tmp;
        for (int i = countNewFiles; i; i--){            
            tmp << freeRemarks.takeAt(qrand() % freeRemarks.count());
        }
        freeRemarks = tmp;
        over = true;
    }

    bool boxIsFull = false;
    QList<MyFolder*> folders = m_box->findChildren<MyFolder*>(QRegExp("[a-z][a-z]"));

    while ((children().count() <= capacity) && !over){
        if (folders.isEmpty()){
            QStringList tmp = abcSubtract(children());
            if (tmp.isEmpty()){
                boxIsFull = true;
                break;
            } else {
                MyAbcFolder *p = new MyAbcFolder(abcRnd(tmp), m_box);
                folders.append(p);
            }
        }
        MyFolder *pFolder = folders.takeFirst();
        QObjectList subFolders = pFolder->children();
        while((subFolders.count() < capacity) && !over){
            MyAbcFolder *pSubFolder = new MyAbcFolder(abcRnd(abcSubtract(subFolders)), pFolder);
            subFolders.append(pSubFolder);
            MyAbcFile *pAbc = new MyAbcFile (pSubFolder);
            if (pAbc->initOk()) {
                connect(pAbc, SIGNAL(markAbcFile(MyAbcFile*,int)), m_box, SLOT(markAbcFile(MyAbcFile*,int)));
                connect(pAbc, SIGNAL(unmarkAbcFile(MyAbcFile*,int)), m_box, SLOT(unmarkAbcFile(MyAbcFile*,int)));
                QString rema = pAbc->rema();
                if (needRemarks > capacity){
                    foreach (QString tmp, abcNames){
                        freeRemarks << rema + tmp;
                    }
                    needRemarks = countNewFiles - freeRemarks.count();
                } else {
                    QStringList lst = abcNames;
                    while (needRemarks){                        
                        freeRemarks << rema + lst.takeAt(qrand() % lst.count());
                        needRemarks--;
                    }
                    over = true;
                }
            } else {
                *log << "Can not use entry file " + pAbc->completeName();
            }
        }
    }

    // .oO0
    if (countNewFiles > 0){
        label_G->show();
        arrangementProgress->setMinimum(0);
        arrangementProgress->setMaximum(countNewFiles);
        arrangementProgress->show();
    }
    int c = 1;
    // 0Oo.

    quint32 tryNumber = 0;
    quint32 tryNumberEdit = 0;
    QSet<MyAbcFile*> usedAbcFiles;
    QMap<QString, QString> resembleReplace;

    foreach(MySourceFile *pSrcFile, correctSourceFiles)
    {
        quint32 category;
        category = m_box->startValueCategory(pSrcFile);

        // .oO0
        arrangementProgress->setValue(c++);
        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
        // 0Oo.

        if (freeRemarks.isEmpty()){ break; }
        Remark a(freeRemarks.takeFirst());        
        //QFile srcFile(pSrcFile->completeName());
        AltQFile srcFile(pSrcFile->completeName());

        QFileInfo srcFileInfo(srcFile);
        Remark o(srcFileInfo.completeBaseName()); // имя редактируемой копии или абсолютно новое имя

        MyAbcFile* abcOriginal; // поиск ремарки с именем "o"
        abcOriginal = m_box->findChild<MyAbcFile*>(NAME_ABC_FILE + "_" + o.rema());

        bool editFile;      // распознавание текущего файла: копия или абсолютно новый
        editFile = false;   // первоначально - файл абсолютно новый, ни на что не похожий...

        // текущий файл - это редактируемая копия существующего?
        if (abcOriginal && abcOriginal->entryExist(o.rk())){
            editFile = true;    // ДЮ - ПЕДЮЙРХПСЕЛЮЪ ЙНОХЪ
            // заполняем соответствие:
            // что заменить в поле resemble - на что заменить (имя новой ремарки)
            resembleReplace.insert(abcOriginal->getEntry(o.rk())->getField(RESEMBLE), a.remark());
        }

        QString newName = m_box->completeName() + QDir::separator() +
                          a.re() + QDir::separator() + a.ma() + QDir::separator() + a.rk() +
                          "." + pSrcFile->ext();

        MyFolder *ta = m_box->findChild<MyFolder*>(a.re());
        MyFolder *tb = ta->findChild<MyFolder*>(a.ma());
        MyAbcFile *tc = tb->findChild<MyAbcFile*>(NAME_ABC_FILE + "_" + a.re() + a.ma());

        *log << QString::number(tryNumber+1).rightJustified(5) +
                " Copy " + pSrcFile->completeName() +
                " to " + newName + " (" + a.remark() + ")";

        if (srcFile.copy(newName)) {            

            QFile copiedFile(newName);
            copiedFile.setPermissions(QFile::ReadOwner|QFile::ReadUser|QFile::ReadGroup|QFile::ReadOther);

            Entry tmpEntry;

            if (editFile) {

                Entry originalEntry(*abcOriginal->getEntry(o.rk()));

                tmpEntry = originalEntry;
                tmpEntry.setField(REMARK,a.remark());
                tmpEntry.setField(RESEMBLE,a.remark());
                tmpEntry.setField(CHECKSUM,pSrcFile->checkSum());
                tmpEntry.setField(ORIGINALFILENAME,pSrcFile->name());

                tryNumberEdit++;

            } else {

                tmpEntry.setEntry( sss->currentSessionNumber() + "~"
                                     + a.remark() + "~"
                                     + a.remark() + "~"
                                     + pSrcFile->checkSum() + "~"
                                     + QString("0") + "~"
                                     + QString::number(category) + "~"
                                     + "unknown" + "~"
                                     + pSrcFile->name() + "~"
                                     + "unknown,unknown" + "~"
                                     + tmc[pSrcFile->objectName()].toString(m_box->dateFormat()));

                m_box->makeSubFile(tmpEntry.getField(REMARK) + "\n" +
                                   tmpEntry.getField(TITLE) + "\n" +
                                   tmpEntry.getField(DATEOFCREATION));
            }

            tc->update(tmpEntry);
            tryNumber++;
            srcFile.remove();
            delete(pSrcFile);
            usedAbcFiles.insert(tc);

            tc->markEntry(&tmpEntry, 0);

        } else {
            *log << "(!) Try is not success...";
        }
    }

    if (!resembleReplace.isEmpty()) { // есть ли необходимость в обновлении полей resemble?

        //.oO0
        label_Q->show();
        label_P->show();
        resembleProgress->setValue(0);
        resembleProgress->setMaximum(m_box->getCount());
        resembleProgress->show();
        //0Oo.


        int c = 1;
        // перебор всех записей и замена поля resemble в соответствии с resembleReplace
        foreach(MyAbcFile* abcFile, m_box->findChildren<MyAbcFile*>(QRegExp("^" + NAME_ABC_FILE + "_"))){
            QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
            foreach(Entry ent, abcFile->getAll()){
                QString rs;
                rs = ent.getField(RESEMBLE);
                // если поле resemble текущей записи требует замены...
                if (resembleReplace.contains(rs)){
                    ent.setField(RESEMBLE,resembleReplace.value(rs));
                    abcFile->update(ent);
                    usedAbcFiles.insert(abcFile);
                }
                //.oO0
                resembleProgress->setValue(c++);
                //0Oo.

            }
        }
    }

    foreach (MyAbcFile* tmp, usedAbcFiles.values()){
        tmp->store();
    }

    if (tryNumber > tryNumberEdit) {                           // добавить сессию только в том случае, если среди
        sss->add(m_box->objectName(), annotationEdit->text()); // размещаемых файлов присутствуют абсолютно новые файлы,
    }                                                          // а не только редактированные копии уже существующих...

    if (boxIsFull){
        *log << "(!) Box is full...";
    }

    QTime tm(0,0,0,0);

    // .oO0
    label_H->show();    

    label_I->show();
    totalLabel->setText(QString::number(countNewFiles));
    totalLabel->show();

    label_J->show();
    addLabel->setText(QString::number(tryNumber));
    addLabel->show();

    label_K->show();
    remainsLabel->setText(QString::number(countNewFiles-tryNumber));
    remainsLabel->show();

    label_L->show();
    beforeLabel->setText(QString::number(m_box->getCount()));
    beforeLabel->show();

    label_M->show();
    afterLabel->setText(QString::number(m_box->getCount() + tryNumber));
    afterLabel->show();

    label_N->show();
    timeLabel->setText(tm.addMSecs(elapsedTime.elapsed()).toString("hh:mm:ss"));
    timeLabel->show();
    // 0Oo.

    m_box->addCount(tryNumber);

    if (!tryNumber){
        m_box->restoreMarked();
    }

    *log << "New files added to the box: " + QString::number(tryNumber) +
            " (Total: " + QString::number(countNewFiles) +
            ", remains: " + QString::number(countNewFiles-tryNumber) + ")";

    log->closeLog();

    emit needUpdateStatus();

    QApplication::restoreOverrideCursor();
}

QStringList Arrangement::abcSubtract(QStringList abcLst)
{
    QSet<QString> a,b;

    b = QSet<QString>::fromList(abcNames);
    a = b.subtract(QSet<QString>::fromList(abcLst));
    return a.toList();
}


QStringList Arrangement::abcSubtract(QObjectList p)
{
    QSet<QString> a,b;
    QStringList s;

    b = QSet<QString>::fromList(abcNames);
    foreach(QObject *tmp, p) {
        s.append(tmp->objectName());
    }
    a = b.subtract(QSet<QString>::fromList(s));

    return a.toList();
}

QString Arrangement::abcRnd(QStringList abcLst)
{    
    int n;
    n = qrand() % abcLst.count();
    return abcLst.at(n);
}

void Arrangement::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
    }
    QFrame::changeEvent(event);
}
