#include "boxinfo.h"
#include "mysssfile.h"
#include "global.h"
#include "mytmpfile.h"

#include <QDir>
#include <QDate>
#include <QTableWidgetItem>
#include <QKeyEvent>
#include <QFileInfo>

BoxInfo::BoxInfo(AnyBox* b, QFrame* w) : QFrame(w)
{
    m_box = b;
    setupUi(w);

    boxPixmap->setPixmap(m_box->boxPixmap());

    QApplication::setOverrideCursor(Qt::WaitCursor);

    //.oO0
    mode->setHidden(true);
    //0Oo.

    MySssFile* s;    
    s = static_cast<MySssFile*>(m_box->unit(NAME_SSS_FILE));

    uint c;
    c = 0;

    uint l;
    l = 0;
    QSet<uint> ss;
    QMap<uint, QDate> sa;
    QMap<uint, QDate> sb;
    QMap<uint, uint> sc;

    uint wc;
    wc = 0;
    uint wk;
    wk = 0;
    uint wt;
    wt = 0;

    QDate begin;
    QDate end;


    bool f;
    f = true;

    foreach (MyAbcFile* abcFile, m_box->findChildren<MyAbcFile*>(QRegExp("^" + NAME_ABC_FILE + "_"))){
        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
        foreach (Entry tmp,  abcFile->getAll()){

            c++;

            uint entrySession;
            entrySession = tmp.getField(SESSION).toUInt();
            ss.insert(entrySession);
            if (entrySession > l) {
                l = entrySession;
            }

            QDate entryDate;
            entryDate = QDate::fromString(tmp.getField(DATEOFCREATION), m_box->dateFormat());
            if (f) {
                begin = entryDate;
                end = begin;
                f = false;
            } else {
                if (entryDate < begin) {
                    begin = entryDate;
                }
                if (entryDate > end) {
                    end = entryDate;
                }
            }

            if (!sa.contains(entrySession)){
                sa.insert(entrySession,entryDate);
                sb.insert(entrySession,entryDate);
                sc.insert(entrySession, 1);
            } else {
                if (entryDate < sa.value(entrySession)) {
                    sa[entrySession] = entryDate;
                }
                if (entryDate > sb.value(entrySession)) {
                    sb[entrySession] = entryDate;
                }
                sc[entrySession]++;
            }

            QString r;
            r = tmp.getField(REMARK);
            QString e;
            e = tmp.getField(RESEMBLE);
            quint32 cat;
            cat = tmp.getField(CATEGORY).toUInt();

            uint entryCategory;
            entryCategory = tmp.getField(CATEGORY).toUInt();                        
            if ((r == e ) && (m_box->category().isEmpty(cat))){
                wc++;
            }

            QString entryKeyword;
            entryKeyword = tmp.getField(KEYWORD);
            if ((r == e) && (entryKeyword.isEmpty() || (entryKeyword.indexOf("unknown",0,Qt::CaseInsensitive) > -1))){
                wk++;
            }

            QString entryTitle;
            entryTitle = tmp.getField(TITLE);
            if ((r == e) && (entryTitle.isEmpty() || (entryTitle.indexOf("unknown",0,Qt::CaseInsensitive) > -1))){
                wt++;
            }
        }
    }

    QDir sldFolder(m_box->unit(NAME_SLD_FOLDER)->completeName());

    int albCount = 0;

    foreach(QFileInfo tmp, sldFolder.entryInfoList(QDir::Files, QDir::Time)){
        if (tmp.suffix() == m_box->extPlayList()) { albCount++; }
    }

    MyTmpFile *tmpFile = static_cast<MyTmpFile*>(m_box->unit(NAME_TMP_FILE));
    tmpFile->loadTemplates(m_box->objectName());

    //.oO0

    boxName->setText(m_box->objectName());

    if (ss.count() != 0) {
        sessionNum->setText(QString::number(ss.count()));
        sessionLast->setText(QString::number(l));
        QString a;
        a = s->sessionDateCreation(l);
        if (a.isEmpty()) {
            label_EA->hide();
            dateLast->hide();
            label_EB->hide();
        } else {
            dateLast->setText(a);
        }
        remarkNum->setText(QString::number(c));
        dateBegin->setText(begin.toString(m_box->dateFormat()));
        dateEnd->setText(end.toString(m_box->dateFormat()));
        templateNum->setText(QString::number(tmpFile->templatesCount(m_box->objectName())));
        albumNum->setText(QString::number(albCount));

        outCategory->setText(QString::number(wc));
        pOutCategory->setText(QString::number(100-100*qreal(wc)/c,'f',2));
        outKeyword->setText(QString::number(wk));
        pOutKeyword->setText(QString::number(100-100*qreal(wk)/c,'f',2));
        outTitle->setText(QString::number(wt));
        pOutTitle->setText(QString::number(100-100*qreal(wt)/c,'f',2));


        sessionTable->setColumnWidth(NO, 40);
        sessionTable->setColumnWidth(CREATION_DATE, 150);
        sessionTable->setColumnWidth(COUNT, 50);
        sessionTable->setColumnWidth(TIME_LENGTH, 180);

        QList<uint> ssl;
        ssl = ss.values();
        qSort(ssl.begin(), ssl.end());

        int r;
        r = 0;

        QTableWidgetItem* i;

        foreach(uint tmpSession, ssl){
            sessionTable->resizeRowToContents(r);
            sessionTable->setRowCount(r+1);            
            i = new QTableWidgetItem(QString::number(tmpSession));
            i->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
            sessionTable->setItem(r,NO,i);

            i = new QTableWidgetItem(s->sessionDateCreation(tmpSession));
            i->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
            sessionTable->setItem(r,CREATION_DATE,i);

            i = new QTableWidgetItem(QString::number(sc.value(tmpSession)));
            i->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
            sessionTable->setItem(r,COUNT,i);

            QString tl;
            if (sa.value(tmpSession) == sb.value(tmpSession)){
                tl = sa.value(tmpSession).toString(m_box->dateFormat());
            } else {
                tl = sa.value(tmpSession).toString(m_box->dateFormat()) + " - " +
                     sb.value(tmpSession).toString(m_box->dateFormat());
            }

            i = new QTableWidgetItem(tl);
            i->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
            sessionTable->setItem(r,TIME_LENGTH,i);

            i = new QTableWidgetItem(s->sessionAnnotation(tmpSession));
            i->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
            sessionTable->setItem(r,ANNOTATION,i);

            r++;
        }

        sessionTable->setCurrentCell(0,NO);
        sessionTable->installEventFilter(this);

    } else {
        sessionNum->setText("-");
        sessionLast->setText("-");
        label_EA->hide();
        dateLast->hide();
        label_EB->hide();
        remarkNum->setText("-");
        dateBegin->hide();
        dateEnd->hide();
        outCategory->setText("-");
        label_IA->hide();
        pOutCategory->hide();
        label_IB->hide();
        outKeyword->setText("-");
        label_JA->hide();
        pOutKeyword->hide();
        label_JB->hide();
        outTitle->setText("-");
        label_KA->hide();
        pOutTitle->hide();
        label_KB->hide();
        templateNum->setText("-");
        albumNum->setText("-");
    }

    label_A->setHidden(true);
    spring->setHidden(true);
    mode->setVisible(true);
    sessionTable->setFocus();
    //0Oo.

    QApplication::restoreOverrideCursor();

    retranslateUi(this);
}

bool BoxInfo::eventFilter(QObject *target, QEvent *event)
{
    bool result;
    result = false;

    if (target == sessionTable){
        if (event->type() == QEvent::KeyPress) {
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
            switch (keyEvent->key()){
            case Qt::Key_Up:
                if (sessionTable->currentRow() == 0){
                    result = true;
                }
                break;
            case Qt::Key_Down:
                if (sessionTable->currentRow() == sessionTable->rowCount() - 1){
                    result = true;
                }
                break;
            default:;
            }
        }
    }
    return result;
}

void BoxInfo::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
    }
    QFrame::changeEvent(event);
}
