#ifndef CATEGORY_H
#define CATEGORY_H

#include <QtGlobal>
#include <QStringList>

/*
  Категория
  Категория описывает целое положительное 28-битное число. Число разделено на поля A,B ... G
  В категории хранятся имена полей и имена отдельных битов этого числа, а также, флаг use
  (использовать/не использовать поле) и флаг testable (тестировать/не тестировать поле  на
  "пустое" значение.

  |    A    |    B    |C| D| E|  F |G|
  |xxxxxxxxx|xxxxxxxxx|x|xx|xx|xxxx|x|
  |1..     ..10..                ..28|

  A - 9 бит
  B - 9 бит
  C - 1 бит
  D - 2 бита
  E - 2 бита
  F - 4 бита
  G - 1 бит

  Статическими методами subCategoryValue_...(...)  можно получить числовые значения отдельных полей
  Методами listSubCategoryValues_...(...) можно получить список имен битов полей
  Проверить категорию на "пустое" значение можно методом isEmpty(...)
*/

class Category
{

public:

    struct content{

        content(QString n, QString d){
            name = n;
            desc = d;
        }

        QString name;
        QString desc;
    };

    struct subCategory {
        QString title;        
        QList<content> contents;
        bool use;
        bool testable;
    };

    Category();

    void setSubCategories_A(const QString &, const QString &, bool, bool);
    void setSubCategories_C(const QString &, const QString &, bool, bool);
    void setSubCategories_D(const QString &, const QString &, bool, bool);
    void setSubCategories_E(const QString &, const QString &, bool, bool);
    void setSubCategories_F(const QString &, const QString &, bool, bool);
    void setSubCategories_G(const QString &, const QString &, bool, bool);

    subCategory subCategory_A();
    subCategory subCategory_B();
    subCategory subCategory_C();
    subCategory subCategory_D();
    subCategory subCategory_E();
    subCategory subCategory_F();
    subCategory subCategory_G();

    QStringList listSubCategoryValues_A();
    QStringList listSubCategoryValues_B();
    QStringList listSubCategoryValues_C();
    QStringList listSubCategoryValues_D();
    QStringList listSubCategoryValues_E();
    QStringList listSubCategoryValues_F();
    QStringList listSubCategoryValues_G();

    static quint32 subCategoryValue_A(quint32);
    static quint32 subCategoryValue_B(quint32);
    static quint32 subCategoryValue_C(quint32);
    static quint32 subCategoryValue_D(quint32);
    static quint32 subCategoryValue_E(quint32);
    static quint32 subCategoryValue_F(quint32);
    static quint32 subCategoryValue_G(quint32);

    bool isEmpty(quint32);

private:

    subCategory m_A;
    subCategory m_B;
    subCategory m_C;
    subCategory m_D;
    subCategory m_E;
    subCategory m_F;
    subCategory m_G;

};

#endif // CATEGORY_H
