#include "describedialog.h"
#include "myabcfile.h"
#include "remark.h"
#include "global.h"

#include <QStringList>
#include <QDate>
#include <QRegExp>
#include <QKeyEvent>
#include <QDir>
#include <QFileInfo>
#include <QStack>
#include <QTextStream>

DescribeDialog::DescribeDialog(AnyBox* b, QFrame* w) : QFrame(w)
{    
    m_box = b;
    setupUi(w);

    boxPixmap->setPixmap(m_box->boxPixmap());

    m_currentEntry = 0;

    QTime curTime;
    curTime = QTime::currentTime();
    m_tmpFilename = curTime.toString("hhmmsszzz");

    // .oO0        
    label_B->setHidden(true);
    mode->setHidden(true);
    // 0Oo.

    modifySymbol = "*";

    setInfoString();   

    wrd = static_cast<MyWrdFile*>(m_box->unit(NAME_WRD_FILE));
    hash = wrd->getWords();

    hardcopyLineEdit->setValidator(new QRegExpValidator(QRegExp("\\d*"),this));

    process = new QProcess(this);

    m_correctedList.clear();
    m_uncorrectedList = m_box->markedEntries();

    if (m_uncorrectedList.isEmpty()) {
        //.oO0
        label_A->setHidden(true);
        label_B->setVisible(true);
        //0Oo.
        return;
    }    

    // .oO0
    label_A->setHidden(true);
    mode->setVisible(true);
    // 0Oo.

    m_corrIndex = -1;
    m_uncorrIndex = 0;
    m_currentEntry = m_uncorrectedList.at(m_uncorrIndex);   

    demonstration(m_currentEntry);

    setModePrevCorrButton();
    setModeNextCorrButton();
    setModePrevUncorrButton();
    setModeNextUncorrButton();

    which_A->setFocus();

    describeTab->installEventFilter(this);
    keywordLineEdit->installEventFilter(this);
    synonymLineEdit->installEventFilter(this);
    keywordListWidget->installEventFilter(this);
    synonymListWidget->installEventFilter(this);
    listWidget->installEventFilter(this);
    titleLineEdit->installEventFilter(this);
    authorLineEdit->installEventFilter(this);    
    hardcopyLineEdit->installEventFilter(this);

    //Category    
    connect(categoryClearButton,SIGNAL(clicked()),this,SLOT(pressCategoryClearButton()));
    connect(categoryOriginalButton,SIGNAL(clicked()),this,SLOT(pressCategoryOriginalButton()));
    //Keyword
    connect(keywordLineEdit,SIGNAL(textChanged(QString)),
            this,SLOT(likeKeywords(QString)));
    connect(keywordListWidget,SIGNAL(itemClicked(QListWidgetItem*)),
            this,SLOT(choiceKeyword(QListWidgetItem*)));
    connect(keywordAddButton,SIGNAL(clicked()),
            this,SLOT(pressAddKeywordButton()));
    connect(synonymLineEdit,SIGNAL(textChanged(QString)),
            this,SLOT(likeSynonyms(QString)));
    connect(synonymAddButton,SIGNAL(clicked()),
            this,SLOT(pressAddSynonymButton()));
    connect(synonymListWidget,SIGNAL(itemSelectionChanged()),
            this,SLOT(synonymSelect()));
    connect(removeSynonymButton,SIGNAL(clicked()),
            this,SLOT(pressRemoveSynonymButton()));
    connect(chooseButton,SIGNAL(clicked()),
            this,SLOT(pressChooseButton()));           
    connect(listWidget,SIGNAL(itemSelectionChanged()),
            this,SLOT(keywordSelect()));            
    connect(removeListButton,SIGNAL(clicked()),
            this,SLOT(pressRemoveListButton()));
    connect(clearListButton,SIGNAL(clicked()),
            this,SLOT(pressClearKeywordButton()));
    connect(originalListButton,SIGNAL(clicked()),
            this,SLOT(pressOriginalKeywordButton()));
    //Title&Author    
    connect(titleClearButton,SIGNAL(clicked()),this,SLOT(pressTitleClearButton()));
    connect(titleOriginalButton,SIGNAL(clicked()),this,SLOT(pressTitleOriginalButton()));
    //Date    
    connect(dateTodayButton,SIGNAL(clicked()),this,SLOT(pressDateTodayButton()));
    connect(dateOriginalButton,SIGNAL(clicked()),this,SLOT(pressDateOriginalButton()));
    //Print
    connect(hardcopyClearButton,SIGNAL(clicked()),this,SLOT(pressHardcopyClearButton()));
    connect(hardcopyOriginalButton,SIGNAL(clicked()),this,SLOT(pressHardcopyOriginalButton()));            
    //
    connect(applyButton,SIGNAL(clicked()),this,SLOT(pressApplyButton()));
    connect(saveButton,SIGNAL(clicked()),this,SLOT(pressSaveButton()));
    //
    connect(prevCorrButton,SIGNAL(clicked()),this,SLOT(pressPrevCorrButton()));
    connect(nextCorrButton,SIGNAL(clicked()),this,SLOT(pressNextCorrButton()));
    connect(prevUncorrButton,SIGNAL(clicked()),this,SLOT(pressPrevUncorrButton()));
    connect(nextUncorrButton,SIGNAL(clicked()),this,SLOT(pressNextUncorrButton()));

    retranslateThis();
}

DescribeDialog::~DescribeDialog()
{
    wrd->storeWords();
    if (process->state() == QProcess::Running){
        process->terminate();
        process->waitForFinished(-1);                
    }

    if (m_tmpFile.exists()){
        m_tmpFile.remove();
    }

    m_box->storeMarkedEntries();
}

bool DescribeDialog::eventFilter(QObject *target, QEvent *event)
{    
    if (target == describeTab ||
        target == keywordLineEdit ||
        target == synonymLineEdit ||
        target == keywordListWidget ||
        target == synonymListWidget ||
        target == listWidget ||
        target == titleLineEdit ||
        target == authorLineEdit ||
        target == hardcopyLineEdit){
        if (event->type() == QEvent::KeyPress) {            
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);            
            if (target == describeTab) {
                int c = describeTab->count();
                int i = describeTab->currentIndex();
                switch (keyEvent->key()){
                case Qt::Key_PageUp:
                    if (i != (c-1)) { describeTab->setCurrentIndex(i+1); }
                    break;
                case Qt::Key_PageDown:
                    if (i > 0) { describeTab->setCurrentIndex(i-1); }
                    break;
                case Qt::Key_Enter:
                    focusNextChild();
                    break;
                case Qt::Key_Return:
                    focusNextChild();
                    break;
                default:;
                }
                return true;
            }
            if (target == keywordLineEdit &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter)) {
                if (keyEvent->modifiers() & Qt::ShiftModifier) {
                    chooseButton->click();
                } else {
                    keywordAddButton->click();
                }
                return true;
            }
            if (target == synonymLineEdit &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter)) {
                if (keyEvent->modifiers() & Qt::ShiftModifier) {
                    synonymAddButton->click();
                    keywordLineEdit->setFocus();
                } else {
                    synonymAddButton->click();
                }
                return true;
            }
            if (target == keywordListWidget &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter)) {
                choiceKeyword(keywordListWidget->currentItem());
                return true;
            }
            if (target == synonymListWidget &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter)) {
                if (synonymListWidget->currentItem()->isSelected()) {
                    focusNextChild();
                }
                return true;
            }
            if (target == listWidget &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter)) {
                if (listWidget->currentItem()->isSelected()) {
                    focusNextChild();
                }
                return true;
            }
            if (target == titleLineEdit &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter)) {                
                focusNextChild();
                return true;
            }
            if (target == authorLineEdit &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter)) {
                describeTab->setFocus();
                return true;
            }            
            if (target == hardcopyLineEdit &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter)) {
                describeTab->setFocus();
                return true;
            }
        }
    }
    return QFrame::eventFilter(target, event);
}

void DescribeDialog::setTextBrowser(Entry* entry)
{
    //Category
    original.category = entry->getField(CATEGORY).toUInt();

    //Keyword
    QStringList li;
    QStringList lo;    
    li = entry->getField(KEYWORD).split(",");
    lo.clear();
    foreach (QString tmp, li){
        if (hash->contains(tmp)){
            lo << tmp;
        }
    }
    original.keywordList = lo.join(",");

    //Title
    QString ta, tb;
    QStringList l;
    l = entry->getField(TITLE).split(",");
    switch (l.count()){
    case 0:
        ta = "unknown";
        tb = "unknown";
        break;
    case 1:
        ta = l.takeFirst().simplified();
        if (ta.isEmpty()) {
            ta = "unknown";
        }
        tb = "unknown";
        break;
    default:
        ta = l.takeFirst().simplified();
        if (ta.isEmpty()) {
            ta = "unknown";
        }
        tb = l.join(" ").simplified();
        if (tb.isEmpty()) {
            tb = "unknown";
        }

    }
    original.title = ta + "," + tb;
    //Date
    QDate a;   
    a = QDate::fromString(entry->getField(DATEOFCREATION),m_box->dateFormat());
    if (!a.isValid()){
        a = QDate::currentDate();
    }
    original.date = a;
    //Print 
    original.print = entry->getField(PRINT).toUInt();

    current = original;

    //Category
    originalCategoryText = setCategory(original.category);
    //Keyword
    originalKeywordListText = setKeyword(original.keywordList);
    //Title
    originalTitleText = setTitle(original.title);
    //Date
    setDate(original.date);
    //Print
    setPrint(original.print);

    QString info;
    info = template_info.arg(entry->getField(REMARK),
                             originalTitleText,
                             original.date.toString(m_box->dateFormat()),
                             originalCategoryText,
                             originalKeywordListText,
                             QString::number(original.print),
                             entry->getField(SESSION),
                             entry->getField(ORIGINALFILENAME),
                             entry->getField(CHECKSUM));
    originalBrowser->setHtml(info);    
}

void DescribeDialog::demonstration(Entry *entry)
{
    QList<Entry*> lst;
    lst << entry;    
    QString sldFilename = m_box->makePlayListFile(m_tmpFilename, lst);
    //process->start(m_box->execPreview() + " " + sldFilename);
    process->start(m_box->execPreview().arg(sldFilename));
    m_tmpFile.setFileName(sldFilename);
}

void DescribeDialog::setModeApplyButton()
{
    if (categoryOriginalButton->isEnabled()
        || originalListButton->isEnabled()
        || titleOriginalButton->isEnabled()
        || dateOriginalButton->isEnabled()
        || hardcopyOriginalButton->isEnabled()) {
        applyButton->setEnabled(true);
        saveButton->setEnabled(true);
    } else {
        applyButton->setDisabled(true);
        saveButton->setDisabled(true);
    }
}

void DescribeDialog::pressApplyButton()
{
    original = current;

    if (m_uncorrectedList.contains(m_currentEntry)){
        m_correctedList << m_uncorrectedList.takeAt(m_uncorrIndex);
        if (m_uncorrIndex == m_uncorrectedList.count()){
            m_uncorrIndex--;
        }
        m_corrIndex++;
        setInfoString();
        setModePrevUncorrButton();
        setModeNextUncorrButton();
        setModePrevCorrButton();
        setModeNextCorrButton();
    }

    m_currentEntry->setField(CATEGORY, QString::number(original.category));
    m_currentEntry->setField(KEYWORD, original.keywordList);
    m_currentEntry->setField(TITLE, original.title);
    m_currentEntry->setField(DATEOFCREATION, original.date.toString(m_box->dateFormat()));
    m_currentEntry->setField(PRINT, QString::number(original.print));

    wrd->storeWords();

    setTextBrowser(m_currentEntry);

    if (original.keywordList.isEmpty()){
        m_currentEntry->setField(KEYWORD,"unknown");
    }

    if (m_box->useSubFile()){
        m_box->makeSubFile(m_currentEntry->getField(REMARK) + "\n" +
                           m_currentEntry->getField(TITLE) + "\n" +
                           m_currentEntry->getField(DATEOFCREATION));
    }
}

void DescribeDialog::pressSaveButton()
{
    pressApplyButton();
    m_box->storeMarkedEntries();
}

// Вкладка Category
//----------------------------------------------------------------

QString DescribeDialog::setCategory(quint32 a)
{
    QString toString;

    quint32 partWhich = Category::subCategoryValue_A(a);
    quint32 partWhichMain = Category::subCategoryValue_B(a);
    quint32 partWhere = Category::subCategoryValue_C(a);
    quint32 partDaytime = Category::subCategoryValue_D(a);
    quint32 partSeason = Category::subCategoryValue_E(a);
    quint32 partPicture = Category::subCategoryValue_F(a);
    quint32 partMethod = Category::subCategoryValue_G(a);

    QStack<QCheckBox*> s;
    QVector<QRadioButton*> q;

    s.push(which_I); s.push(which_H); s.push(which_G);
    s.push(which_F); s.push(which_E); s.push(which_D);
    s.push(which_C); s.push(which_B); s.push(which_A);
    while(!s.isEmpty()){
        QCheckBox* w;
        w = s.pop();

        //.oO0
        disconnect(w,SIGNAL(stateChanged(int)),this,SLOT(updateCurrentCategory()));
        //0Oo.

        if (partWhich & 1) {
            QString f;
            if (!toString.isEmpty()) { toString.append(", "); }
            f = w->text();
            if (partWhichMain & 1) {
                w->setCheckState(Qt::Checked);
                toString.append(f.replace(0,1,f.left(1).toUpper()));
            } else {
                w->setCheckState(Qt::PartiallyChecked);
                toString.append(f);
            }
        } else {
             w->setCheckState(Qt::Unchecked);
        }
        partWhich >>= 1;
        partWhichMain >>= 1;

        //.oO0
        connect(w,SIGNAL(stateChanged(int)),this,SLOT(updateCurrentCategory()));
        //0Oo.
    }

    toString.append("; ");

    //.oO0
    disconnect(where_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(where_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    if (partWhere){
        where_B->setChecked(true);
        toString.append(where_B->text());
    } else {
        where_A->setChecked(true);
        toString.append(where_A->text());
    }

    //.oO0
    connect(where_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(where_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    toString.append("; ");

    //.oO0
    disconnect(daytime_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(daytime_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(daytime_C,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(daytime_D,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    q << daytime_A << daytime_B << daytime_C << daytime_D;
    q.at(partDaytime)->setChecked(true);
    toString.append(q.at(partDaytime)->text());
    q.clear();

    //.oO0
    connect(daytime_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(daytime_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(daytime_C,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(daytime_D,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    toString.append("; ");

    //.oO0
    disconnect(season_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(season_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(season_C,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(season_D,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    q << season_A << season_B << season_C << season_D;
    q.at(partSeason)->setChecked(true);
    toString.append(q.at(partSeason)->text());
    q.clear();

    //.oO0
    connect(season_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(season_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(season_C,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(season_D,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    toString.append("; ");

    //.oO0
    disconnect(picture_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_C,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_D,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_E,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_F,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_G,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_H,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_I,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_J,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_K,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_L,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_M,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_N,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_O,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(picture_P,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    q << picture_A << picture_B << picture_C << picture_D;
    q << picture_E << picture_F << picture_G << picture_H;
    q << picture_I << picture_J << picture_K << picture_L;
    q << picture_M << picture_N << picture_O << picture_P;
    q.at(partPicture)->setChecked(true);
    toString.append(q.at(partPicture)->text());
    q.clear();

    //.oO0
    connect(picture_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_C,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_D,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_E,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_F,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_G,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_H,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_I,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_J,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_K,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_L,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_M,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_N,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_O,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(picture_P,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    toString.append("; ");

    //.oO0
    disconnect(method_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    disconnect(method_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    if (partMethod){
        method_B->setChecked(true);
        toString.append(method_B->text());
    } else {
        method_A->setChecked(true);
        toString.append(method_A->text());
    }

    //.oO0
    connect(method_A,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    connect(method_B,SIGNAL(clicked()),this,SLOT(updateCurrentCategory()));
    //0Oo.

    updateCurrentCategory();

    return toString;
}

void DescribeDialog::updateCurrentCategory()
{    
    QStack<QCheckBox*> s;
    QStack<QRadioButton*> q;

    quint32 partWhich = 0;
    quint32 partWhichMain = 0;
    quint32 partWhere = 0;
    quint32 partDaytime = 0;
    quint32 partSeason = 0;
    quint32 partPicture = 0;
    quint32 partMethod = 0;

    s.push(which_A); s.push(which_B); s.push(which_C);
    s.push(which_D); s.push(which_E); s.push(which_F);
    s.push(which_G); s.push(which_H); s.push(which_I);
    while(!s.isEmpty()){
        partWhichMain <<= 1 ;
        partWhich <<= 1 ;
        switch(s.pop()->checkState()){
            case Qt::Checked: partWhichMain += 1;
            case Qt::PartiallyChecked: partWhich += 1;
            case Qt::Unchecked:;
        }
    }

    if (where_B->isChecked()) { partWhere = 1; } else { partWhere = 0; }

    q.push(daytime_D); q.push(daytime_C); q.push(daytime_B); q.push(daytime_A);
    while(!q.isEmpty()){
        if (q.pop()->isChecked()) { break; }
        partDaytime++;
    }

    q.push(season_D); q.push(season_C); q.push(season_B); q.push(season_A);
    while(!q.isEmpty()){
        if (q.pop()->isChecked()) { break; }
        partSeason++;
    }

    q.push(picture_P); q.push(picture_O); q.push(picture_N); q.push(picture_M);
    q.push(picture_L); q.push(picture_K); q.push(picture_J); q.push(picture_I);
    q.push(picture_H); q.push(picture_G); q.push(picture_F); q.push(picture_E);
    q.push(picture_D); q.push(picture_C); q.push(picture_B); q.push(picture_A);
    while(!q.isEmpty()){
        if (q.pop()->isChecked()) { break; }
        partPicture++;
    }

    if (method_B->isChecked()) { partMethod = 1; } else { partMethod = 0; }

    current.category = (partMethod << 27) + (partPicture << 23) + (partSeason << 21)
                     + (partDaytime << 19) + (partWhere << 18) + (partWhichMain << 9) + partWhich;
    //.oO0
    setModeCategoryClearButton();
    setModeCategoryOriginalButton();
    setModeApplyButton();
    //0Oo.
}

void DescribeDialog::setModeCategoryClearButton()
{
    if (current.category == 0){
        categoryClearButton->setDisabled(true);
    } else {
        categoryClearButton->setEnabled(true);
    }
}

void DescribeDialog::pressCategoryClearButton()
{
    setCategory(0);
    which_A->setFocus();
}

void DescribeDialog::setModeCategoryOriginalButton()
{
    if (current.category == original.category){
        categoryOriginalButton->setDisabled(true);
        describeTab->setTabText(describeTab->indexOf(tab_A),tabHdrCategory);
    } else {
        categoryOriginalButton->setEnabled(true);
        describeTab->setTabText(describeTab->indexOf(tab_A),tabHdrCategory + " " + modifySymbol);
    }
}

void DescribeDialog::pressCategoryOriginalButton()
{
    setCategory(original.category);
    which_A->setFocus();
}

// Вкладка Keyword
//----------------------------------------------------------------
QString DescribeDialog::setKeyword(QString a)
{
    listWidget->clear();

    if (a.isEmpty()) {
        //.oO0
        setModeKeywordChooseButton();
        setModeKeywordClearButton();
        setModeKeywordOriginalButton();
        setModeKeywordChooseButton();
        setModeApplyButton();
        //0Oo.
        return "-";
    }

    QString toString;

    QStringList l;
    l = a.split(",");
    foreach (QString tmp, l){        
        QString n,m;
        splitUp(hash->value(tmp),n,m);
        QListWidgetItem* i = new QListWidgetItem(listWidget);
        i->setText(n);

        if (!toString.isEmpty()) { toString.append(", "); }
        toString.append(n);

        i->setToolTip(m);
        listWidget->addItem(i);
    }
    //.oO0
    setModeKeywordChooseButton();
    setModeKeywordClearButton();
    setModeKeywordOriginalButton();
    setModeKeywordChooseButton();    
    setModeApplyButton();
    setModeListWidget();
    //0Oo.

    return toString;
}

void DescribeDialog::splitUp(const QString &s, QString &n, QString &m)
{
    QStringList a;
    a = s.split("~");
    a.takeFirst();
    a.takeLast();
    n = a.takeFirst();
    if (!a.isEmpty()) {
        m = a.join(", ");
    } else {
        m = " - ";
    }
}

void DescribeDialog::setModeKeywordChooseButton()
{
    if ((current.keywordList.indexOf(currentKey) == -1) && (!currentKey.isEmpty())){
        chooseButton->setEnabled(true);
    } else {
        chooseButton->setEnabled(false);
    }
}

void DescribeDialog::pressChooseButton()
{
    if (current.keywordList.isEmpty()){
        current.keywordList = currentKey;
    } else {
        current.keywordList.append("," + currentKey);
    }
    QString n,m;
    splitUp(hash->value(currentKey),n,m);
    QListWidgetItem* i = new QListWidgetItem(listWidget);
    i->setText(n);
    i->setToolTip(m);
    listWidget->addItem(i);
    keywordLineEdit->clear();
    keywordLineEdit->setFocus();
    setModeKeywordClearButton();
    setModeKeywordOriginalButton();
    setModeListWidget();
    setModeApplyButton();
}

void DescribeDialog::setModeKeywordOriginalButton()
{
    QStringList a,b;
    a = current.keywordList.split(","); a.sort();
    b = original.keywordList.split(","); b.sort();
    if (a.join(",") == b.join(",")){
        originalListButton->setEnabled(false);
        describeTab->setTabText(describeTab->indexOf(tab_B),tabHdrKeyword);
    } else {
        originalListButton->setEnabled(true);
        describeTab->setTabText(describeTab->indexOf(tab_B),tabHdrKeyword + " " + modifySymbol);
    }
}

void DescribeDialog::pressOriginalKeywordButton()
{
    current.keywordList = original.keywordList;    
    setKeyword(current.keywordList);
    setModeListWidget();
    keywordLineEdit->setFocus();
}

void DescribeDialog::setModeKeywordClearButton()
{
    if (current.keywordList.isEmpty()){
        clearListButton->setEnabled(false);
    } else {
        clearListButton->setEnabled(true);
    }
}

void DescribeDialog::pressClearKeywordButton()
{
    current.keywordList = "";
    setKeyword(current.keywordList);
    setModeListWidget();
    keywordLineEdit->setFocus();
}

void DescribeDialog::pressAddKeywordButton()
{
    QString s = keywordLineEdit->text();
    wrd->addNewWord(s);    
    likeKeywords(s);
    synonymLineEdit->setFocus();
}

void DescribeDialog::pressAddSynonymButton()
{
    QString s = synonymLineEdit->text();
    wrd->addSynonym(currentKey, s);
    synonymLineEdit->clear();
    synonymListWidget->addItem(s);
    if (current.keywordList.indexOf(currentKey) > -1) {
        QString n,m;
        splitUp(hash->value(currentKey),n,m);
        listWidget->findItems(n, Qt::MatchFixedString).first()->setToolTip(m);
    }
    setModeSynonymListWidget();
    synonymLineEdit->setFocus();
}

void DescribeDialog::pressRemoveListButton()
{
    QList<QListWidgetItem*> l;
    QListWidgetItem* i;
    QStringList s;
    int a,b;
    s = current.keywordList.split(",");
    l = listWidget->selectedItems();
    while (!l.isEmpty()){
        i = l.takeFirst();
        a = listWidget->row(i);
        foreach(QString tmp, s){
            if (hash->contains(tmp)){
                QString n,m;
                splitUp(hash->value(tmp),n,m);
                b = listWidget->row(listWidget->findItems(n,Qt::MatchFixedString).first());
                if (a == b) {
                    s.removeOne(tmp);
                    break;
                }
            }
        }
        listWidget->takeItem(a);
    }
    current.keywordList = s.join(",");
    setModeKeywordClearButton();
    setModeKeywordOriginalButton();
    setModeKeywordChooseButton();
    setModeListWidget();
    setModeApplyButton();
    keywordLineEdit->setFocus();
}

void DescribeDialog::pressRemoveSynonymButton()
{
    QList<QListWidgetItem*> l;
    l = synonymListWidget->selectedItems();
    while (!l.isEmpty()){
        synonymListWidget->takeItem(synonymListWidget->row(l.takeFirst()));
    }
    bool hit;
    if (current.keywordList.indexOf(currentKey) > -1) {
        hit = true;
    } else {
        hit = false;
    }

    QListWidgetItem* p;
    QString n,m;
    splitUp(hash->value(currentKey),n,m);
    if (hit) {
       p = listWidget->findItems(n, Qt::MatchFixedString).first();
    }

    QStringList w;
    w.append(keywordLineEdit->text());
    int c = synonymListWidget->count();
    for (int i = 0; i < c; i++){
        w.append(synonymListWidget->item(i)->text());
    }
    wrd->setSynonym(currentKey,w);

    if (hit) {
        splitUp(hash->value(currentKey),n,m);
        p->setText(n);
        p->setToolTip(m);
    }
    setModeSynonymListWidget();
    synonymLineEdit->setFocus();
}

void DescribeDialog::keywordSelect()
{
    QList<QListWidgetItem *> l;
    l = listWidget->selectedItems();
    if (l.isEmpty()){
        removeListButton->setEnabled(false);
    } else {
        removeListButton->setEnabled(true);
    }
}

void DescribeDialog::synonymSelect()
{
    QList<QListWidgetItem *> l;
    l = synonymListWidget->selectedItems();
    if (l.isEmpty()){
        removeSynonymButton->setEnabled(false);
    } else {
        removeSynonymButton->setEnabled(true);
    }
}

void DescribeDialog::choiceKeyword(QListWidgetItem* i)
{
    keywordListWidget->setCurrentRow(-1);
    keywordLineEdit->setText(i->text());
    keywordLineEdit->setFocus();
}

void DescribeDialog::likeKeywords(QString s)
{
    keywordListWidget->clear();
    keywordListWidget->setEnabled(false);
    synonymListWidget->clear();

    bool hit = false;
    currentKey = "";

    if (!s.isEmpty()) {
        QStringList likeWords;
        keywordAddButton->setEnabled(true);
        foreach (QString tmp, hash->keys()){
            int pos_start, pos_end;
            QString w,v;
            v = hash->value(tmp);
            pos_start = v.indexOf("~" + s,0,Qt::CaseInsensitive);
            while (pos_start > -1) {
                keywordListWidget->setEnabled(true);
                pos_end = v.indexOf("~", pos_start+1,Qt::CaseInsensitive);
                w = v.mid(pos_start+1,pos_end - pos_start - 1);
                likeWords << w;
                if (!hit) {                    
                    if (QString::compare(w,s,Qt::CaseInsensitive) == 0){
                        currentKey = tmp;
                        QString q(v);
                        q.remove(pos_start+1,pos_end - pos_start);
                        QStringList l;
                        l = q.split("~");
                        l.takeFirst();
                        l.takeLast();
                        synonymListWidget->addItems(l);
                        keywordAddButton->setEnabled(false);
                        synonymLineEdit->setEnabled(true);
                        synonymAddButton->setEnabled(false);
                        //synonymListWidget->setEnabled(true);
                        hit = true;
                    } else {
                        synonymLineEdit->setEnabled(false);
                        synonymLineEdit->clear();
                        synonymAddButton->setEnabled(false);
                        //synonymListWidget->setEnabled(false);
                        keywordAddButton->setEnabled(true);
                    }
                }
                pos_start = v.indexOf("~" + s,pos_end,Qt::CaseInsensitive);
            }
        }
        keywordListWidget->addItems(likeWords);
    } else {
        synonymLineEdit->setEnabled(false);
        synonymAddButton->setEnabled(false);
        //synonymListWidget->setEnabled(false);
        keywordAddButton->setEnabled(false);
        keywordLineEdit->clear();
        synonymListWidget->clear();
    }
    setModeKeywordChooseButton();
    setModeSynonymListWidget();
}

void DescribeDialog::likeSynonyms(QString s)
{
    if (!s.isEmpty()) {
        QRegExp exp("~" + s + "~", Qt::CaseInsensitive);
        QString v;
        bool f =true;
        foreach (QString tmp, hash->keys()){
            v = hash->value(tmp);
            if (exp.indexIn(v) != -1){
                f =false;
                break;
            }
        }
        if (f) {
            synonymAddButton->setEnabled(true);
        } else {
            synonymAddButton->setEnabled(false);
        }
    } else {
        synonymAddButton->setEnabled(false);
    }
}

void DescribeDialog::setModeListWidget()
{
    if (listWidget->count() == 0){
        listWidget->setDisabled(true);
    } else {
        listWidget->setEnabled(true);
    }
}

void DescribeDialog::setModeSynonymListWidget()
{
    if (synonymListWidget->count() == 0){
        synonymListWidget->setDisabled(true);
    } else {
        synonymListWidget->setEnabled(true);
    }
}


//  Вкладка Title&Author
//----------------------------------------------------------------
QString DescribeDialog::setTitle(QString s)
{
    QString toString;

    //.oO0
    disconnect(titleLineEdit,SIGNAL(textChanged(QString)),this,SLOT(updateCurrentTitle()));
    disconnect(authorLineEdit,SIGNAL(textChanged(QString)),this,SLOT(updateCurrentTitle()));
    //0Oo.

    QStringList l;
    QString t,a;
    l = s.split(",");
    t = l.takeFirst().simplified();
    a = l.takeFirst().simplified();
    if (t == "unknown") {
        titleLineEdit->setText("");
    } else {
        titleLineEdit->setText(t);
    }
    if (a == "unknown") {
        authorLineEdit->setText("");
    } else {
        authorLineEdit->setText(a);
    }

    toString = t + ", " + a;

    //.oO0
    connect(titleLineEdit,SIGNAL(textChanged(QString)),this,SLOT(updateCurrentTitle()));
    connect(authorLineEdit,SIGNAL(textChanged(QString)),this,SLOT(updateCurrentTitle()));
    //0Oo.

    updateCurrentTitle();

    return toString;
}

void DescribeDialog::updateCurrentTitle()
{
    QString t,a;
    t = titleLineEdit->text().simplified();
    a = authorLineEdit->text().simplified();
    if (t.isEmpty()) { t = "unknown"; }
    if (a.isEmpty()) { a = "unknown"; }
    current.title = t + "," + a;

    //.oO0
    setModeTitleClearButton();
    setModeTitleOriginalButton();
    setModeApplyButton();
    //0Oo.
}

void DescribeDialog::setModeTitleClearButton()
{
    if (current.title == "unknown,unknown"){
        titleClearButton->setDisabled(true);
    } else {
        titleClearButton->setEnabled(true);
    }
}

void DescribeDialog::pressTitleClearButton()
{
    setTitle("unknown,unknown");
    titleLineEdit->setFocus();
}

void DescribeDialog::setModeTitleOriginalButton()
{
    if (current.title == original.title) {
        titleOriginalButton->setDisabled(true);
        describeTab->setTabText(describeTab->indexOf(tab_C),tabHdrTitle);
    } else {
        titleOriginalButton->setEnabled(true);
        describeTab->setTabText(describeTab->indexOf(tab_C),tabHdrTitle + " " + modifySymbol);
    }
}

void DescribeDialog::pressTitleOriginalButton()
{
    setTitle(original.title);
    titleLineEdit->setFocus();
}

// Вкладка Date
//----------------------------------------------------------------
void DescribeDialog::setDate(QDate d)
{
    //.oO0
    disconnect(dateCalendar,SIGNAL(activated(QDate)),this,SLOT(updateSelectedDate(QDate)));
    //0Oo.

    dateCalendar->setSelectedDate(d);

    //.oO0
    connect(dateCalendar,SIGNAL(activated(QDate)),this,SLOT(updateSelectedDate(QDate)));
    //0Oo.

    updateSelectedDate(d);
}

void DescribeDialog::updateSelectedDate(QDate a)
{        
    //selectedDate->setText(a.toString(anybox->dateFormat()));
    selectedDate->setText(a.toString(m_box->dateFormat()));
    current.date = a;
    //.oO0
    setModeDateTodayButton();
    setModeDateOriginalButton();
    setModeApplyButton();
    describeTab->setFocus();
    //0Oo.    
}

void DescribeDialog::setModeDateTodayButton()
{
    if (current.date == QDate::currentDate()) {
        dateTodayButton->setDisabled(true);
    } else {
        dateTodayButton->setEnabled(true);
    }
}

void DescribeDialog::pressDateTodayButton()
{
    QDate c;
    c = QDate::currentDate();
    dateCalendar->setSelectedDate(c);
    updateSelectedDate(c);
    dateCalendar->setFocus();
}

void DescribeDialog::setModeDateOriginalButton()
{
    if (current.date == original.date) {
        dateOriginalButton->setDisabled(true);
        describeTab->setTabText(describeTab->indexOf(tab_D),tabHdrDate);
    } else {
        dateOriginalButton->setEnabled(true);
        describeTab->setTabText(describeTab->indexOf(tab_D),tabHdrDate + " " + modifySymbol);
    }
}

void DescribeDialog::pressDateOriginalButton()
{
    dateCalendar->setSelectedDate(original.date);
    updateSelectedDate(original.date);
    dateCalendar->setFocus();
}

// Вкладка Hardcopy
//----------------------------------------------------------------
void DescribeDialog::setPrint(uint p)
{
    //.oO0
    disconnect(hardcopyLineEdit,SIGNAL(textChanged(QString)),this,SLOT(updateCurrentHardcopy()));
    //0Oo.

    hardcopyLineEdit->setText(QString::number(p));

    //.oO0
    connect(hardcopyLineEdit,SIGNAL(textChanged(QString)),this,SLOT(updateCurrentHardcopy()));
    //0Oo.

    updateCurrentHardcopy();
}

void DescribeDialog::updateCurrentHardcopy()
{
    QString s;
    s = hardcopyLineEdit->text();
    if (s.isEmpty()){
        current.print = 0;
    } else {
        current.print = s.toUInt();
    }
    //.oO0
    setModeHardcopyClearButton();
    setModeHardcopyOriginalButton();
    setModeApplyButton();
    //0Oo.
}

void DescribeDialog::setModeHardcopyClearButton()
{
    if (current.print == 0) {
        hardcopyClearButton->setDisabled(true);
    } else {
        hardcopyClearButton->setEnabled(true);
    }    
}

void DescribeDialog::pressHardcopyClearButton()
{
    setPrint(0);
    hardcopyLineEdit->setFocus();
}

void DescribeDialog::setModeHardcopyOriginalButton()
{
    if (current.print == original.print) {
        hardcopyOriginalButton->setDisabled(true);
        describeTab->setTabText(describeTab->indexOf(tab_E),tabHdrPrint);
    } else {
        hardcopyOriginalButton->setEnabled(true);
        describeTab->setTabText(describeTab->indexOf(tab_E),tabHdrPrint + " " + modifySymbol);
    }
}

void DescribeDialog::pressHardcopyOriginalButton()
{
    setPrint(original.print);
    hardcopyLineEdit->setFocus();
}

void DescribeDialog::setModePrevCorrButton()
{       
    if (m_correctedList.count() == 1){
        prevCorrButton->setEnabled(true);
        return;
    }

    if (m_corrIndex < 1){
        prevCorrButton->setDisabled(true);
    } else {
        prevCorrButton->setEnabled(true);
    }
}

void DescribeDialog::pressPrevCorrButton()
{
    process->terminate();
    process->waitForFinished(-1);

    if (m_correctedList.count() > 1) {
        m_corrIndex--;
    }
    m_currentEntry = m_correctedList.at(m_corrIndex);
    setTextBrowser(m_currentEntry);
    demonstration(m_currentEntry);
    setModePrevCorrButton();
    setModeNextCorrButton();
}

void DescribeDialog::setModeNextCorrButton()
{
    int t = m_correctedList.count();

    if (t == 1){
        nextCorrButton->setEnabled(true);
        return;
    }

    if (m_corrIndex >= t - 1){
        nextCorrButton->setDisabled(true);
    } else {
        nextCorrButton->setEnabled(true);
    }
}

void DescribeDialog::pressNextCorrButton()
{    
    process->terminate();
    process->waitForFinished(-1);

    if (m_correctedList.count() > 1) {
        m_corrIndex++;
    }
    m_currentEntry = m_correctedList.at(m_corrIndex);
    setTextBrowser(m_currentEntry);
    demonstration(m_currentEntry);
    setModePrevCorrButton();
    setModeNextCorrButton();
}

void DescribeDialog::setModePrevUncorrButton()
{ 
    if (m_uncorrectedList.count() == 1){
        prevUncorrButton->setEnabled(true);
        return;
    }

    if (m_uncorrIndex < 1){
        prevUncorrButton->setDisabled(true);
    } else {
        prevUncorrButton->setEnabled(true);
    }
}

void DescribeDialog::pressPrevUncorrButton()
{
    process->terminate();
    process->waitForFinished(-1);

    if (m_uncorrectedList.count() > 1) {
        m_uncorrIndex--;
    }
    m_currentEntry = m_uncorrectedList.at(m_uncorrIndex);
    setTextBrowser(m_currentEntry);
    demonstration(m_currentEntry);
    setModePrevUncorrButton();
    setModeNextUncorrButton();
}

void DescribeDialog::setModeNextUncorrButton()
{
    int t = m_uncorrectedList.count();

    if (t == 1){
        nextUncorrButton->setEnabled(true);
        return;
    }

    if (m_uncorrIndex >= t - 1){
        nextUncorrButton->setDisabled(true);
    } else {
        nextUncorrButton->setEnabled(true);
    }
}

void DescribeDialog::pressNextUncorrButton()
{
    process->terminate();
    process->waitForFinished(-1);

    if (m_uncorrectedList.count() > 1) {
        m_uncorrIndex++;
    }
    m_currentEntry = m_uncorrectedList.at(m_uncorrIndex);
    setTextBrowser(m_currentEntry);
    demonstration(m_currentEntry);
    setModePrevUncorrButton();
    setModeNextUncorrButton();
}

void DescribeDialog::setInfoString()
{
    uint t;
    uint c;
    uint u;

    t = m_box->markedEntriesCount();
    c = m_correctedList.count();
    u = t - c;

    total->setText(QString::number(t));
    corrected->setText(QString::number(c));
    uncorrected->setText(QString::number(u));

    qreal pct;
    pct = qreal(c) * 100 / t;

    correctedPct->setText(QString::number(pct,'f',1));
    uncorrectedPct->setText(QString::number(100 - pct,'f',1));
}

void DescribeDialog::setTabVisible(int t)
{
    int c;
    c = describeTab->count();

    if ((t > -1) && (t < c)) {
        for(int i = 0; i < c; i++){
            if (i != t) {
                describeTab->setTabEnabled(i, false);
            } else {
                describeTab->setCurrentIndex(i);
            }
        }

    }
}

void DescribeDialog::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
        retranslateThis();
    }
    QFrame::changeEvent(event);
}

void DescribeDialog::retranslateThis()
{
    m_box->setCategoryLabels();
    Category cat;
    cat = m_box->category();
    Category::subCategory subCat;

    subCat = cat.subCategory_A();
    if (subCat.use) {
        which->setTitle(subCat.title);
        which_A->setText(subCat.contents.at(0).name);
        which_A->setToolTip(subCat.contents.at(0).desc);
        which_B->setText(subCat.contents.at(1).name);
        which_B->setToolTip(subCat.contents.at(1).desc);
        which_C->setText(subCat.contents.at(2).name);
        which_C->setToolTip(subCat.contents.at(2).desc);
        which_D->setText(subCat.contents.at(3).name);
        which_D->setToolTip(subCat.contents.at(3).desc);
        which_E->setText(subCat.contents.at(4).name);
        which_E->setToolTip(subCat.contents.at(4).desc);
        which_F->setText(subCat.contents.at(5).name);
        which_F->setToolTip(subCat.contents.at(5).desc);
        which_G->setText(subCat.contents.at(6).name);
        which_G->setToolTip(subCat.contents.at(6).desc);
        which_H->setText(subCat.contents.at(7).name);
        which_H->setToolTip(subCat.contents.at(7).desc);
        which_I->setText(subCat.contents.at(8).name);
        which_I->setToolTip(subCat.contents.at(8).desc);
    } else {
        which->hide();
    }

    subCat = cat.subCategory_C();
    if (subCat.use) {
        where->setTitle(subCat.title);
        where_A->setText(subCat.contents.at(0).name);
        where_A->setToolTip(subCat.contents.at(0).desc);
        where_B->setText(subCat.contents.at(1).name);
        where_B->setToolTip(subCat.contents.at(1).desc);
    } else {
        where->hide();
    }

    subCat = cat.subCategory_G();
    if (subCat.use) {
        method->setTitle(subCat.title);
        method_A->setText(subCat.contents.at(0).name);
        method_A->setToolTip(subCat.contents.at(0).desc);
        method_B->setText(subCat.contents.at(1).name);
        method_B->setToolTip(subCat.contents.at(1).desc);
    } else {
        method->hide();
    }

    subCat = cat.subCategory_D();
    if (subCat.use) {
        daytime->setTitle(subCat.title);
        daytime_A->setText(subCat.contents.at(0).name);
        daytime_A->setToolTip(subCat.contents.at(0).desc);
        daytime_B->setText(subCat.contents.at(1).name);
        daytime_B->setToolTip(subCat.contents.at(1).desc);
        daytime_C->setText(subCat.contents.at(2).name);
        daytime_C->setToolTip(subCat.contents.at(2).desc);
        daytime_D->setText(subCat.contents.at(3).name);
        daytime_D->setToolTip(subCat.contents.at(3).desc);
    } else {
        daytime->hide();
    }

    subCat = cat.subCategory_E();
    if (subCat.use) {
        season->setTitle(subCat.title);
        season_A->setText(subCat.contents.at(0).name);
        season_A->setToolTip(subCat.contents.at(0).desc);
        season_B->setText(subCat.contents.at(1).name);
        season_B->setToolTip(subCat.contents.at(1).desc);
        season_C->setText(subCat.contents.at(2).name);
        season_C->setToolTip(subCat.contents.at(2).desc);
        season_D->setText(subCat.contents.at(3).name);
        season_D->setToolTip(subCat.contents.at(3).desc);
    } else {
        season->hide();
    }

    subCat = cat.subCategory_F();
    if (subCat.use) {
        picture->setTitle(subCat.title);
        picture_A->setText(subCat.contents.at(0).name);
        picture_A->setToolTip(subCat.contents.at(0).desc);
        picture_B->setText(subCat.contents.at(1).name);
        picture_B->setToolTip(subCat.contents.at(1).desc);
        picture_C->setText(subCat.contents.at(2).name);
        picture_C->setToolTip(subCat.contents.at(2).desc);
        picture_D->setText(subCat.contents.at(3).name);
        picture_D->setToolTip(subCat.contents.at(3).desc);
        picture_E->setText(subCat.contents.at(4).name);
        picture_E->setToolTip(subCat.contents.at(4).desc);
        picture_F->setText(subCat.contents.at(5).name);
        picture_F->setToolTip(subCat.contents.at(5).desc);
        picture_G->setText(subCat.contents.at(6).name);
        picture_G->setToolTip(subCat.contents.at(6).desc);
        picture_H->setText(subCat.contents.at(7).name);
        picture_H->setToolTip(subCat.contents.at(7).desc);
        picture_I->setText(subCat.contents.at(8).name);
        picture_I->setToolTip(subCat.contents.at(8).desc);
        picture_J->setText(subCat.contents.at(9).name);
        picture_J->setToolTip(subCat.contents.at(9).desc);
        picture_K->setText(subCat.contents.at(10).name);
        picture_K->setToolTip(subCat.contents.at(10).desc);
        picture_L->setText(subCat.contents.at(11).name);
        picture_L->setToolTip(subCat.contents.at(11).desc);
        picture_M->setText(subCat.contents.at(12).name);
        picture_M->setToolTip(subCat.contents.at(12).desc);
        picture_N->setText(subCat.contents.at(13).name);
        picture_N->setToolTip(subCat.contents.at(13).desc);
        picture_O->setText(subCat.contents.at(14).name);
        picture_O->setToolTip(subCat.contents.at(14).desc);
        picture_P->setText(subCat.contents.at(15).name);
        picture_P->setToolTip(subCat.contents.at(15).desc);
    } else {
        picture->hide();
    }

    template_info = tr("<table>"
                    "<tr> <td width = 120> <b>Remark</b> </td> <td>%1</td> </tr> "
                    "</table>"
                    "<hr />"
                    "<table>"
                    "<tr> <td width = 120> <b>Caption</b> </td> <td>%2</td> </tr>"
                    "<tr> <td> <b>Date</b>      </td> <td>%3</td> </tr>"
                    "<tr> <td> <b>Category</b>  </td> <td>%4</td> </tr>"
                    "<tr> <td> <b>Keyword</b>   </td> <td>%5</td> </tr>"
                    "<tr> <td> <b>Hardcopy</b>  </td> <td>%6</td> </tr>"
                    "</table>"
                    "<hr />"
                    "<table>"
                    "<tr> <td width = 120> <b>Session</b> </td> <td>%7</td> </tr>"
                    "<tr> <td> <b>Original name</b>       </td> <td>%8</td> </tr>"
                    "<tr> <td> <b>Checksum</b>            </td> <td>%9</td> </tr>"
                    "</table>");

    tabHdrCategory = describeTab->tabText(describeTab->indexOf(tab_A));
    tabHdrKeyword = describeTab->tabText(describeTab->indexOf(tab_B));
    tabHdrTitle = describeTab->tabText(describeTab->indexOf(tab_C));
    tabHdrDate = describeTab->tabText(describeTab->indexOf(tab_D));
    tabHdrPrint = describeTab->tabText(describeTab->indexOf(tab_E));

    if (m_currentEntry){
        setTextBrowser(m_currentEntry);
    }
}
