#include "displaydialog.h"
#include "mytmpfile.h"
#include "myabcfile.h"

#include <QDir>
#include <QDate>
#include <QKeyEvent>
#include <QTableWidgetItem>
#include <QTextStream>
#include <QProcess>

DisplayDialog::DisplayDialog(AnyBox* b, QFrame* w) : QFrame(w)
{
    m_box = b;
    setupUi(w);

    boxPixmap->setPixmap(m_box->boxPixmap());

    //.oO0
    label_B->setHidden(true);
    mode->setHidden(true);
    //0Oo.

    m_boxPath = m_box->completeName();
    m_lenBoxPath = m_boxPath.length();

    QDir sldFolder(m_box->unit(NAME_SLD_FOLDER)->completeName());

    MyTmpFile* tmpFile;
    tmpFile = static_cast<MyTmpFile*>(b->unit(NAME_TMP_FILE));
    tmpFile->loadTemplates(m_box->objectName());

    QString n;

    int r;
    r = 0;

    QTableWidgetItem* i;

    foreach(QFileInfo tmp, sldFolder.entryInfoList(QDir::Files, QDir::Time)){
        n = tmp.completeBaseName();

        if (tmp.suffix() != m_box->extPlayList()){ continue; }

        name.append(n);        
        QString dsc;        
        dsc = tmpFile->describes()->value(n);
        description.append(dsc);
        fileInfo.append(tmp);        

        displayTable->setColumnWidth(NAME, 120);
        displayTable->setColumnWidth(CAPACITY, 60);
        displayTable->setColumnWidth(TIME_LENGTH, 180);

        displayTable->setRowCount(r+1);
        i = new QTableWidgetItem(n);
        displayTable->setItem(r,NAME,i);
        i = new QTableWidgetItem("-");
        i->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        displayTable->setItem(r,CAPACITY,i);
        i = new QTableWidgetItem("-");
        i->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
        displayTable->setItem(r,TIME_LENGTH,i);
        i = new QTableWidgetItem(dsc);
        displayTable->setItem(r,DESCRIPTION,i);

        r++;
    }

    //.oO0
    label_A->setHidden(true);
    //0Oo.

    if (name.isEmpty()){

        //.oO0
        label_B->setVisible(true);
        //0Oo.

    } else {

        //.oO0
        //spaceFrame->setHidden(true);        
        displayTable->resizeRowsToContents();
        displayTable->setCurrentCell(0,NAME);
        displayTable->setFocus();
        total->setText(QString::number(name.count()));
        spring->setHidden(true);
        mode->setVisible(true);
        //0Oo.

        cr = 0;
        startTimer(500);

        displayTable->installEventFilter(this);

        connect(startButton, SIGNAL(clicked()), this, SLOT(pressStartButton()));
    }
}

bool DisplayDialog::eventFilter(QObject *target, QEvent *event)
{
    bool result;
    result = false;

    if (target == displayTable){
        if (event->type() == QEvent::KeyPress) {
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
            switch (keyEvent->key()){
            case Qt::Key_Up:
                if (displayTable->currentRow() == 0){
                    result = true;
                }
                break;
            case Qt::Key_Down:
                if (displayTable->currentRow() == displayTable->rowCount() - 1){
                    result = true;
                }
                break;
            case Qt::Key_Enter:
            case Qt::Key_Return:
            case Qt::Key_Space:
                startButton->click();
                result = true;
                break;
            default:;
            }
        }
    }
    return result;
}

void DisplayDialog::timerEvent(QTimerEvent *event)
 {
    fillItemCapacityAndTime(cr);
    cr++;
    if (cr < fileInfo.count()){
    } else {
        killTimer(event->timerId());
    }

 }

void DisplayDialog::fillItemCapacityAndTime(int r)
{
    QFile file(fileInfo.at(r).absoluteFilePath());
    QTextStream st(&file);

    uint c = 0;
    QString l;

    if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {

        QDate min;
        QDate max;
        bool first = true;

        while (!st.atEnd()){
           l = st.readLine();
           int f;

           f = l.indexOf(m_boxPath,0,Qt::CaseInsensitive);
           if (f > -1){
               c++;

               QString re;
               QString ma;
               QString rk;
               MyAbcFile* abcFile;

               re = l.mid(f + m_lenBoxPath + 1, ABC_WIDTH);
               ma = l.mid(f + m_lenBoxPath + 1 + ABC_WIDTH + 1, ABC_WIDTH);
               rk = l.mid(f + m_lenBoxPath + 1 + ABC_WIDTH + 1 + ABC_WIDTH + 1, ABC_WIDTH);

               abcFile = m_box->findChild<MyAbcFile*>(NAME_ABC_FILE + "_" + re + ma);

               if (abcFile) {

                   QDate remarkDate;
                   remarkDate = QDate::fromString(abcFile->getEntryPart(rk, DATEOFCREATION), m_box->dateFormat());

                   if (remarkDate.isValid()){
                       if (first){
                           min = remarkDate;
                           max = min;
                           first = false;
                       } else {
                           if (remarkDate < min){
                               min = remarkDate;
                           }
                           if (remarkDate > max){
                               max = remarkDate;
                           }
                       }
                   }
               }
           }
        }
        capacity.append(c);
        if (min == max) {            
            timeLength.append(min.toString(m_box->dateFormat()));
        } else {
            timeLength.append(min.toString(m_box->dateFormat()) + " - " + max.toString(m_box->dateFormat()));
        }
        file.close();        

        QTableWidgetItem* j;

        j = displayTable->item(r,CAPACITY);
        j->setText(QString::number(capacity.at(r)));
        j = displayTable->item(r,TIME_LENGTH);
        j->setText(timeLength.at(r));
    }
}

void DisplayDialog::pressStartButton()
{   
    QString filename;

    filename.append(m_box->unit(NAME_SLD_FOLDER)->completeName());

    filename.append(QDir::separator());
    filename.append(displayTable->item(displayTable->currentRow(),NAME)->text());
    filename.append(".");
    filename.append(m_box->extPlayList());
    QProcess::startDetached(m_box->execPresent().arg(filename));
}

void DisplayDialog::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
    }
    QFrame::changeEvent(event);
}

