#include "firstdialog.h"

#include "global.h"
#include "mysrcfolder.h"
#include "basefolder.h"
#include "altqfile.h"

#include <QDir>
#include <QListWidgetItem>
#include <QMessageBox>
#include <QKeyEvent>
#include <QTextStream>

FirstDialog::FirstDialog(AnyBox* b, QFrame* w) : QFrame(w)
{
    m_box = b;    
    setupUi(w);

    label_A->setText(this->objectName());

    boxPixmap->setPixmap(m_box->boxPixmap());

    QTime curTime;
    curTime = QTime::currentTime();
    QString p;
    p = static_cast<BaseFolder*>(m_box->unit(NAME_SLD_FOLDER))->completeName();
    m_sldFile.setFileName(p + QDir::separator() +  curTime.toString("hhmmsszzz") + "." + m_box->extPlayList());

    process = new QProcess(this);

    countDetectFiles = 0;

    QString firstDir = static_cast<BaseFolder*>(m_box->baseUnit())->firstDir();
    foreach(QString tmp, firstDir.split(",")) {
        device.append(tmp.simplified());
    }

    foreach(QString tmp, m_box->ext().values()){
        extension.append("*." + tmp);
    }

    // .oO0
    detectFrame->setHidden(true);
    label_B->setHidden(true);
    detectFiles->setHidden(true);    

    selectFrame->setHidden(true);
    label_D->setHidden(true);
    selectFiles->setHidden(true);
    label_K->setHidden(true);
    selectTotalSize->setHidden(true);
    label_L->setHidden(true);

    label_F->setHidden(true);
    copyProgress->setHidden(true);

    resultFrame->setHidden(true);

    detectList->setFocus();
    // 0Oo.

    detectList->installEventFilter(this);
    selectList->installEventFilter(this);

    detecting();

    connect(detectNothingButton,SIGNAL(clicked()),this,SLOT(detectNothing()));
    connect(detectSubstractButton,SIGNAL(clicked()),this,SLOT(detectSubstract()));
    connect(detectAddButton,SIGNAL(clicked()),this,SLOT(detectAdd()));
    connect(detectAllButton,SIGNAL(clicked()),this,SLOT(detectAll()));
    connect(detectAgreeButton,SIGNAL(clicked()),this,SLOT(detectAgree()));

    connect(detectList,SIGNAL(itemClicked(QListWidgetItem*)),
            this,SLOT(enableDetectAdd(QListWidgetItem*)));
    connect(detectList,SIGNAL(itemClicked(QListWidgetItem*)),
            this,SLOT(enableDetectSubstract(QListWidgetItem*)));
    connect(detectList,SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
            this,SLOT(enableDetectAdd(QListWidgetItem*)));
    connect(detectList,SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
            this,SLOT(enableDetectSubstract(QListWidgetItem*)));

    connect(selectNothingButton,SIGNAL(clicked()),this,SLOT(selectNothing()));
    connect(selectSubstractButton,SIGNAL(clicked()),this,SLOT(selectSubstract()));
    connect(selectAddButton,SIGNAL(clicked()),this,SLOT(selectAdd()));
    connect(selectAllButton,SIGNAL(clicked()),this,SLOT(selectAll()));
    connect(selectAgreeButton,SIGNAL(clicked()),this,SLOT(selectAgree()));

    connect(selectList,SIGNAL(itemSelectionChanged()),this,SLOT(enableSelectAdd()));
    connect(selectList,SIGNAL(itemSelectionChanged()),this,SLOT(enableSelectSubstract()));

    retranslateUi(this);
}

FirstDialog::~FirstDialog()
{
    if (process->state() == QProcess::Running){
        process->terminate();
        process->waitForFinished(-1);        
    }

    if (m_sldFile.exists()){
        m_sldFile.remove();
    }
}

bool FirstDialog::eventFilter(QObject *target, QEvent *event)
{
    bool result;
    result = false;

    if (target == detectList ||
        target == selectList){
        if (event->type() == QEvent::KeyPress) {
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
            if (target == detectList) {
                switch (keyEvent->key()){
                case Qt::Key_Up:
                    if (detectList->currentRow() == 0){
                        result = true;
                    }
                    break;
                case Qt::Key_Down:
                    if (detectList->currentRow() == detectList->count() - 1){
                        result = true;
                    }
                    break;
                default:;
                }
            }
            if (target == selectList) {
                switch (keyEvent->key()){
                case Qt::Key_Up:
                    if (selectList->currentRow() == 0){
                        result = true;
                    }
                    break;
                case Qt::Key_Down:
                    if (selectList->currentRow() == selectList->count() - 1){
                        result = true;
                    }
                    break;
                default:;
                }
            }
        }
    }
    return result;
}

void FirstDialog::detecting()
{           
    foreach(QString tmp, device){        

        QString devicePath;
        QString deviceDescription;
        QStringList p;

        p = tmp.split("|");

        switch (p.count()){
        case 2:
            deviceDescription = p.value(1).simplified();
        case 1:
            devicePath = p.at(0).simplified();
        default:;
        }

        if (devicePath.isEmpty()) { continue; }

        QDir dir(devicePath);
        set_D a;
        a.device = devicePath;
        QListWidgetItem *i;
        if (dir.exists()){            
            a.count = dir.entryList(extension,QDir::Files).count();
            if (a.count > 0) {                
                a.mode = off;
                a.text = a.device + "\t" + QString::number(a.count) + " " + tr("files");
                i = new QListWidgetItem(QIcon(":img/checkMarkEmpty.png"), a.text, detectList);

            } else {
                a.mode = no;
                a.text = a.device + "\t(!) " + tr("device is empty...");
                i = new QListWidgetItem(QIcon(":img/checkMarkDash.png"), a.text, detectList);
            }
        } else {            
            a.mode = no;
            a.count = 0;
            a.text = a.device + "\t(!) " + tr("device not detected...");
            i = new QListWidgetItem(QIcon(":img/checkMarkDash.png"), a.text, detectList);
        }
        i->setToolTip(deviceDescription);
        devices.append(a);
    }
    // .oO0
    label_A->setText(tr("Detect removable devices... complete"));
    detectFrame->setVisible(true);
    label_B->setVisible(true);
    detectFiles->setText(QString::number(countDetectFiles));
    detectFiles->setVisible(true);    
    if (detectList->count() > 0){
        detectList->setCurrentRow(0);
        enableDetectAdd(detectList->currentItem());
        enableDetectSubstract(detectList->currentItem());
    }
    enableDetectAgree();
    enableDetectAll();
    enableDetectNothing();
    // 0Oo.
}

void FirstDialog::detectAll()
{
    for (int r = 0; r < detectList->count(); r++)
    {
        if (devices[r].mode == off){
            devices[r].mode = on;
            detectList->item(r)->setIcon(QIcon(":img/checkMark.png"));
            countDetectFiles += devices[r].count;
        }
    }
    detectAllButton->setEnabled(false);
    detectNothingButton->setEnabled(true);
    detectAgreeButton->setEnabled(true);
    enableDetectAdd(detectList->currentItem());
    enableDetectSubstract(detectList->currentItem());
    detectFiles->setText(QString::number(countDetectFiles));
}

void FirstDialog::detectNothing()
{
    for (int r = 0; r < detectList->count(); r++)
    {
        if (devices[r].mode == on){
            devices[r].mode = off;
            detectList->item(r)->setIcon(QIcon(":img/checkMarkEmpty.png"));
            countDetectFiles -= devices[r].count;
        }
    }
    detectAllButton->setEnabled(true);
    detectNothingButton->setEnabled(false);
    detectAgreeButton->setEnabled(false);
    enableDetectAdd(detectList->currentItem());
    enableDetectSubstract(detectList->currentItem());
    detectFiles->setText(QString::number(countDetectFiles));
}

void FirstDialog::enableDetectAgree()
{
    bool f;
    f = false;
    foreach(set_D tmp, devices){
        if (tmp.mode == on){
            f = true;
            break;
        }
    }
    if (f) {
        detectAgreeButton->setEnabled(true);
    } else {
        detectAgreeButton->setEnabled(false);
    }
}

void FirstDialog::enableDetectAll()
{
    bool f;
    f = false;
    for (int r = 0; r < detectList->count(); r++)
    {
        if (devices[r].mode == off){
           f = true;
           break;
        }
    }
    if (f) {
        detectAllButton->setEnabled(true);
    } else {
        detectAllButton->setEnabled(false);
    }
}

void FirstDialog::enableDetectNothing()
{
    bool f;
    f = false;
    for (int r = 0; r < detectList->count(); r++)
    {
        if (devices[r].mode == on){
            f = true;
           break;
        }
    }
    if (f) {
        detectNothingButton->setEnabled(true);
    } else {
        detectNothingButton->setEnabled(false);
    }
}

void FirstDialog::detectAdd()
{
    int r = detectList->currentRow();
    if (devices[r].mode != no) {
        devices[r].mode = on;
        countDetectFiles +=devices[r].count;
    }
    detectList->item(r)->setIcon(QIcon(":img/checkMark.png"));

    enableDetectAll();
    enableDetectNothing();
    enableDetectAgree();
    detectAddButton->setEnabled(false);
    detectSubstractButton->setEnabled(true);
    detectFiles->setText(QString::number(countDetectFiles));
}

void FirstDialog::detectSubstract()
{
    int r = detectList->currentRow();
    if (devices[r].mode != no) {
        devices[r].mode = off;
        countDetectFiles -= devices[r].count;
    }
    detectList->item(r)->setIcon(QIcon(":img/checkMarkEmpty.png"));

    enableDetectAll();
    enableDetectNothing();
    enableDetectAgree();
    detectAddButton->setEnabled(true);
    detectSubstractButton->setEnabled(false);
    detectFiles->setText(QString::number(countDetectFiles));
}


void FirstDialog::enableDetectAdd(QListWidgetItem *i)
{
    if (devices[detectList->row(i)].mode == off) {
        detectAddButton->setEnabled(true);
    } else {
        detectAddButton->setEnabled(false);
    }
}

void FirstDialog::enableDetectSubstract(QListWidgetItem *i)
{
    if (devices[detectList->row(i)].mode == on) {
        detectSubstractButton->setEnabled(true);
    } else {
        detectSubstractButton->setEnabled(false);
    }
}

void FirstDialog::detectAgree()
{
    // .oO0
    detectFrame->setHidden(true);
    selectFrame->setVisible(true);
    selectList->setFocus();
    // 0Oo.

    QStringList lst;
    countSelectFiles = 0;
    totalSize = 0;
    foreach (set_D tmp, devices){
        if (tmp.mode != on) { continue; }
        QDir dir(tmp.device);
        set_F a;
        QString absPath;
        foreach (QFileInfo f, dir.entryInfoList(extension,QDir::Files,QDir::DirsFirst | QDir::Name | QDir::Type)){
            absPath = QDir::toNativeSeparators(f.absoluteFilePath());
            a.text = absPath + " (" + QString::number(qreal(f.size())/1048576,'f',1) + "Mb)";
            a.name = absPath;
            a.mode = off;
            a.size = f.size();
            new QListWidgetItem(QIcon(":img/checkMarkEmpty.png"), a.text, selectList);
            files.append(a);
            lst << absPath;
        }
    }    

    QTextStream st(&m_sldFile);

    int n;
    int lstCount;
    lstCount = lst.count();
    if (lstCount > MAX_NUM_OF_SHOW){
        n = MAX_NUM_OF_SHOW;
        QMessageBox::information(this,tr("Warning"),tr("Selected list is very big. Displaying is restrict..."));
    } else {
        n = lstCount;
    }

    if (m_sldFile.open(QIODevice::WriteOnly | QIODevice::Text)){
        for(int i = 0; i < n; i++){
            st << lst.at(i) << "\n";
        }
        m_sldFile.close();
    }

    process->start(m_box->execPreview().arg(QDir::toNativeSeparators(m_sldFile.fileName())));

    // .oO0
    label_D->setVisible(true);
    selectFiles->setText(QString::number(countSelectFiles));
    selectFiles->setVisible(true);    
    label_K->setVisible(true);
    selectTotalSize->setText(QString::number(qreal(totalSize)/1048576,'f',1));
    selectTotalSize->setVisible(true);
    label_L->setVisible(true);
    selectNothingButton->setEnabled(false);
    selectSubstractButton->setEnabled(false);
    selectAddButton->setEnabled(false);
    selectAllButton->setEnabled(true);
    selectAgreeButton->setEnabled(false);
    // 0Oo.
}

void FirstDialog::selectNothing()
{
    for (int r = 0; r < selectList->count(); r++)
    {
        if (files[r].mode == on){
            files[r].mode = off;
            selectList->item(r)->setIcon(QIcon(":img/checkMarkEmpty.png"));
            countSelectFiles--;
            totalSize -= files[r].size;
        }
    }
    selectAllButton->setEnabled(true);
    selectNothingButton->setEnabled(false);
    selectAgreeButton->setEnabled(false);
    enableSelectAdd();
    selectSubstractButton->setEnabled(false);
    selectFiles->setText(QString::number(countSelectFiles));
    selectTotalSize->setText(QString::number(qreal(totalSize)/1048576,'f',1));
}

void FirstDialog::selectSubstract()
{
    int r;
    foreach(QListWidgetItem *tmp, selectList->selectedItems()){
        tmp->setSelected(false);
        r = selectList->row(tmp);
        if (files[r].mode == on) {
            files[r].mode = off;
            countSelectFiles--;
            totalSize -= files[r].size;
            selectList->item(r)->setIcon(QIcon(":img/checkMarkEmpty.png"));
        }
    }
    enableSelectAll();
    enableSelectNothing();
    enableSelectAgree();
    selectAddButton->setEnabled(false);
    selectSubstractButton->setEnabled(false);
    selectFiles->setText(QString::number(countSelectFiles));
    selectTotalSize->setText(QString::number(qreal(totalSize)/1048576,'f',1));
}

void FirstDialog::selectAdd()
{
    int r;
    foreach(QListWidgetItem *tmp, selectList->selectedItems()){
        tmp->setSelected(false);
        r = selectList->row(tmp);
        if (files[r].mode == off) {
            files[r].mode = on;
            countSelectFiles++;
            totalSize += files[r].size;
            selectList->item(r)->setIcon(QIcon(":img/checkMark.png"));
        }
    }
    enableSelectAll();
    enableSelectNothing();
    enableSelectAgree();
    selectAddButton->setEnabled(false);
    selectSubstractButton->setEnabled(false);
    selectFiles->setText(QString::number(countSelectFiles));
    selectTotalSize->setText(QString::number(qreal(totalSize)/1048576,'f',1));
}

void FirstDialog::selectAll()
{
    for (int r = 0; r < selectList->count(); r++)
    {
        if (files[r].mode == off){
            files[r].mode = on;
            selectList->item(r)->setIcon(QIcon(":img/checkMark.png"));
            countSelectFiles++;
            totalSize += files[r].size;
        }
    }
    selectAllButton->setEnabled(false);
    selectNothingButton->setEnabled(true);
    selectAgreeButton->setEnabled(true);
    selectAddButton->setEnabled(false);    
    enableSelectSubstract();
    selectFiles->setText(QString::number(countSelectFiles));
    selectTotalSize->setText(QString::number(qreal(totalSize)/1048576,'f',1));
}

void FirstDialog::selectAgree()
{
    QApplication::setOverrideCursor(Qt::WaitCursor);

    // .oO0
    selectFrame->setHidden(true);
    label_F->setVisible(true);
    copyProgress->setMinimum(0);
    copyProgress->setMaximum(countSelectFiles);
    copyProgress->setVisible(true);
    // 0Oo.

    int c = 0;
    int b = 0;
    AltQFile f;
    QFileInfo i;

    QString srcDir;
    srcDir = static_cast<MySrcFolder*>(m_box->unit(NAME_SRC_FOLDER))->completeName() + QDir::separator();

    QString newName;
    foreach (set_F tmp, files){
        if (tmp.mode == on){            
            f.setFileName(tmp.name);
            i.setFile(f);
            c++;

            // .oO0
            copyProgress->setValue(c);
            QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
            // 0Oo.

            newName = srcDir + i.fileName();
            if (!f.copy(newName)) { b++; }
        }
    }

    // .oO0
    detectFilesResult->setText(QString::number(countDetectFiles));
    selectFilesResult->setText(QString::number(countSelectFiles));
    copyFilesResult->setText(QString::number(countSelectFiles - b));
    resultFrame->setVisible(true);
    // 0Oo.

    QApplication::restoreOverrideCursor();
}

void FirstDialog::enableSelectNothing()
{
    bool f;
    f = false;
    for (int r = 0; r < selectList->count(); r++)
    {
        if (files[r].mode == on){
           f = true;
           break;
        }
    }
    if (f) {
        selectNothingButton->setEnabled(true);
    } else {
        selectNothingButton->setEnabled(false);
    }
}

void FirstDialog::enableSelectSubstract()
{
    bool f;
    f = false;
    int r;
    foreach(QListWidgetItem *tmp, selectList->selectedItems()){
        r = selectList->row(tmp);
        if (files[r].mode == on) {
            f = true;
            break;
        }
    }
    if (f) {
        selectSubstractButton->setEnabled(true);
    } else {
        selectSubstractButton->setEnabled(false);
    }
}

void FirstDialog::enableSelectAdd()
{
    bool f;
    f = false;
    int r;
    foreach(QListWidgetItem *tmp, selectList->selectedItems()){
        r = selectList->row(tmp);
        if (files[r].mode == off) {
            f = true;
            break;
        }
    }
    if (f) {
        selectAddButton->setEnabled(true);
    } else {
        selectAddButton->setEnabled(false);
    }
}

void FirstDialog::enableSelectAll()
{
    bool f;
    f = false;
    for (int r = 0; r < selectList->count(); r++)
    {
        if (files[r].mode == off){
           f = true;
           break;
        }
    }
    if (f) {
        selectAllButton->setEnabled(true);
    } else {
        selectAllButton->setEnabled(false);
    }
}

void FirstDialog::enableSelectAgree()
{
    bool f;
    f = false;
    foreach(set_F tmp, files){
        if (tmp.mode == on){
            f = true;
            break;
        }
    }
    if (f) {
        selectAgreeButton->setEnabled(true);
    } else {
        selectAgreeButton->setEnabled(false);
    }
}

void FirstDialog::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
        retranslateThis();
    }
    QFrame::changeEvent(event);
}

void FirstDialog::retranslateThis()
{
    label_A->setText(tr("Detect removable devices... complete"));
    for(int r = 0; r < detectList->count(); r++){
        QString s;
        s = detectList->item(r)->text();
        QRegExp exp("\\s");
        QString d;
        d = s.mid(0,exp.indexIn(s));
        QDir dir(d);
        int c;
        QString str;

        if (dir.exists()){
            c = dir.entryList(extension,QDir::Files).count();
            if (c > 0) {
                str = d + "\t" + QString::number(c) + " " + tr("files");
            } else {
                str = d + "\t(!) " + tr("device is empty...");
            }
        } else {
            str = d + "\t(!) " + tr("device not detected...");
        }

        detectList->item(r)->setText(str);
    }
}
