#include <QApplication>
#include <QTimer>
#include <QDir>
#include <QTextCodec>

#include <QTranslator>

#include "global.h"
#include "mysettings.h"
#include "mysetdialog.h"
#include "mysplash.h"
#include "mainwindow.h"

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    QMap<QString,QVariant> sets;

    sets[KEY_GENERAL + KEY_BOX_DIR] = BOX_DIR_DEFAULT;
    sets[KEY_GENERAL + KEY_CPY_DIR] = CPY_DIR_DEFAULT;
    sets[KEY_GENERAL + KEY_BURN_CD] = CMD_BURN_CD_DEFAULT;
    sets[KEY_GENERAL + KEY_FIRST_DIR] = FIRST_DIR_DEFAULT;
    sets[KEY_GENERAL + KEY_EXTRACT_META_APP] = EXTRACT_META_APP;
    sets[KEY_GENERAL + KEY_LANGUAGE] = LANGUAGE;

    sets[KEY_PHOTO + KEY_EXT] = PHOTO_EXT;
    sets[KEY_BOOK + KEY_EXT] = BOOK_EXT;
    sets[KEY_CLIP + KEY_EXT] = CLIP_EXT;
    sets[KEY_MUSIC + KEY_EXT] = MUSIC_EXT;

    sets[KEY_PHOTO + KEY_PREVIEW_APP] = PHOTO_PREVIEW_APP;
    sets[KEY_PHOTO + KEY_PRESENT_APP] = PHOTO_PRESENT_APP;    

    sets[KEY_BOOK + KEY_PREVIEW_APP] = BOOK_PREVIEW_APP;
    sets[KEY_BOOK + KEY_PRESENT_APP] = BOOK_PRESENT_APP;

    sets[KEY_CLIP + KEY_PREVIEW_APP] = CLIP_PREVIEW_APP;
    sets[KEY_CLIP + KEY_PRESENT_APP] = CLIP_PRESENT_APP;   

    sets[KEY_MUSIC + KEY_PREVIEW_APP] = MUSIC_PREVIEW_APP;
    sets[KEY_MUSIC + KEY_PRESENT_APP] = MUSIC_PRESENT_APP;    

    sets[KEY_PHOTO + KEY_CATEGORIES] = PHOTO_CATEGORIES;
    sets[KEY_BOOK + KEY_CATEGORIES] = BOOK_CATEGORIES;
    sets[KEY_CLIP + KEY_CATEGORIES] = CLIP_CATEGORIES;
    sets[KEY_MUSIC + KEY_CATEGORIES] = MUSIC_CATEGORIES;

    MySettings *myset = 0;
    MainWindow *win = 0;

    QString sep = QDir::separator();

    bool initOk = false;

    QString message;

    MySplash *splash = new MySplash(":img/splash_420x290.png");
    splash->show();

    // Load settings
    //---------------
    message = QObject::tr("Read the settings...");
    splash->showMessage(message);

    if (argc == 2) {
        QString fileName(argv[1]);
        fileName=fileName.trimmed();
        if (fileName.endsWith(sep)){
            fileName.append("mybox.ini");
        }
        myset = new MySettings(fileName, sets);
    } else {
        myset = new MySettings("mysoft", "mybox", sets);
    }

    myset->sync();

    // Load language
    //---------------
    message = QObject::tr("Loading languages...");
    splash->showMessage(message);
    QString currentLanguage = myset->value(KEY_GENERAL + KEY_LANGUAGE).toString();

    QTranslator appTranslator;
    QTranslator qtTranslator;

    appTranslator.load(currentLanguage,":/lng/");
    qtTranslator.load("qt_"+ currentLanguage, ":/lng/");

    qApp->installTranslator(&appTranslator);
    qApp->installTranslator(&qtTranslator);

    // Test settings
    //---------------
    QSettings::Status setStatus = myset->status();

    switch (setStatus) {
    case QSettings::AccessError:
        message = QObject::tr("Cannot access the file") + "\n'" + myset->fileName() + "'";
        break;
    case QSettings::FormatError:
        message = QObject::tr("Error in the configuration file") + "\n'" + myset->fileName() + "'";
        break;
    case QSettings::NoError:
        do {
            message = QObject::tr("Loading...");
            splash->showMessage(message);

            QString strMyBoxDir = myset->value(KEY_GENERAL + KEY_BOX_DIR).toString();

            QDir myBoxDir(strMyBoxDir);

            QString nameMyBoxDir = myBoxDir.dirName();

            QString pathMyBoxDir;
            pathMyBoxDir = strMyBoxDir.left(strMyBoxDir.lastIndexOf(nameMyBoxDir));

            if (!nameMyBoxDir.isEmpty() && myBoxDir.exists()) {

                // Load mainWindow
                //-----------------
                win = new MainWindow(myset);
                win->setAppTranslator(&appTranslator);
                win->setQtTranslator(&qtTranslator);
                initOk = win->initStatus(message);

                if (initOk) {
                    break;
                } else {
                    splash->showMessage(message);
                    delete win;
                    win = 0;
                }
            }

            MySetDialog dialog(myset);

            if (dialog.exec()){
                dialog.saveSettings();
            } else {
                message = QObject::tr("Creating MyBox cancelled...");
                break;
            }

        } while(!initOk);

    default:;
    }

    splash->showMessage(message);
    splash->activateUserAction();

    if (initOk){
        QObject::connect(splash, SIGNAL(destroyed()), win, SLOT(show()));
        QTimer::singleShot(3000, splash, SLOT(close()));
    } else {
        QObject::connect(splash, SIGNAL(destroyed()), &app, SLOT(quit()));
    }

    int exitStatus = app.exec();

    if (win) { delete win; }
    if (myset) { delete myset; }

    return exitStatus;
}
