#include "mainwindow.h"
#include "global.h"

#include <QMessageBox>
#include <QDir>
#include <QDesktopWidget>

MainWindow::MainWindow(MySettings *myset)
{
    m_set = myset;

    m_myBox = new MyBox(m_set);

    int minW=700;
    int minH=730;

    QDesktopWidget scr;
    QWidget *wScr;

    wScr = scr.screen();

    setMinimumSize(minW, minH);
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

    move((wScr->width() - frameGeometry().width())/2,
         (wScr->height() - frameGeometry().height())/2);

    m_currentBox = static_cast<AnyBox*>(m_myBox->baseFolder()->unit(NAME_PHOTO));

    m_centralFrame = new QFrame();
    setCentralWidget(m_centralFrame);
    new BoxSelect(m_currentBox, m_centralFrame);

    createActions();
    boxesActionGroup = new QActionGroup(this);
    boxesActionGroup->addAction(photoBoxAction);
    boxesActionGroup->addAction(bookBoxAction);
    boxesActionGroup->addAction(videoBoxAction);
    boxesActionGroup->addAction(musicBoxAction);
    photoBoxAction->setChecked(true);

    createMenus();
    createToolBars();
    createStatusBar();

    setWindowIcon(QIcon(":img/mybox_logo2.ico"));
    this->setWindowTitle("It's MyBox");

    retranslateUi();
}

bool MainWindow::initStatus(QString &message)
{
    return m_myBox->initStatus(message);
}

void MainWindow::closeEvent(QCloseEvent *event)
{    
    int r = QMessageBox::question(this, tr("Quit from MyBox") + "...", tr("Are you sure") + "?",
                                 QMessageBox::No | QMessageBox::Yes);

    if (r == QMessageBox::No) {
        event->ignore();
    }
}

void MainWindow::createActions()
{        
    settingsAction = new QAction(this);
    settingsAction->setIcon(QIcon(":img/spanner.png"));        
    connect(settingsAction, SIGNAL(triggered()), this, SLOT(settingsActing()));

    aboutAction = new QAction(this);
    aboutAction->setIcon(QIcon(":img/about.png"));
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(aboutActing()));

    quitAction = new QAction(this);
    quitAction->setIcon(QIcon(":img/quit.png"));
    connect(quitAction, SIGNAL(triggered()), this, SLOT(close()));

    photoBoxAction = new QAction(this);
    photoBoxAction->setIcon(QIcon(":img/photo_32x32.png"));
    connect(photoBoxAction, SIGNAL(triggered()), this, SLOT(photoBoxActing()));
    photoBoxAction->setCheckable(true);

    bookBoxAction = new QAction(this);
    bookBoxAction->setIcon(QIcon(":img/book_32x32.png"));
    connect(bookBoxAction, SIGNAL(triggered()), this, SLOT(bookBoxActing()));
    bookBoxAction->setCheckable(true);

    videoBoxAction = new QAction(this);
    videoBoxAction->setIcon(QIcon(":img/video_32x32.png"));
    connect(videoBoxAction, SIGNAL(triggered()), this, SLOT(videoBoxActing()));
    videoBoxAction->setCheckable(true);

    musicBoxAction = new QAction(this);
    musicBoxAction->setIcon(QIcon(":img/music_32x32.png"));
    connect(musicBoxAction, SIGNAL(triggered()), this, SLOT(musicBoxActing()));
    musicBoxAction->setCheckable(true);

    firstStepsAction = new QAction(this);
    firstStepsAction->setIcon(QIcon(":img/rejection_32x32.png"));
    connect(firstStepsAction, SIGNAL(triggered()), this, SLOT(firstStepsActing()));

    arrangementAction = new QAction(this);
    arrangementAction->setIcon(QIcon(":img/arrangement_32x32.png"));
    connect(arrangementAction, SIGNAL(triggered()), this, SLOT(arrangementActing()));

    selectDialogAction = new QAction(this);
    selectDialogAction->setIcon(QIcon(":img/select_32x32.png"));
    connect(selectDialogAction, SIGNAL(triggered()), this, SLOT(selectDialogActing()));

    selectTemplateAction = new QAction(this);
    selectTemplateAction->setIcon(QIcon(":img/template_32x32.png"));
    connect(selectTemplateAction, SIGNAL(triggered()), this, SLOT(selectTemplateActing()));

    selectOutCategoryAction = new QAction(this);
    selectOutCategoryAction->setIcon(QIcon(":img/select_category_32x32.png"));
    connect(selectOutCategoryAction, SIGNAL(triggered()), this, SLOT(selectOutCategoryActing()));

    selectOutKeywordAction = new QAction(this);
    selectOutKeywordAction->setIcon(QIcon(":img/select_keyword_32x32.png"));
    connect(selectOutKeywordAction, SIGNAL(triggered()), this, SLOT(selectOutKeywordActing()));

    selectOutTitleAction = new QAction(this);
    selectOutTitleAction->setIcon(QIcon(":img/select_title_32x32.png"));
    connect(selectOutTitleAction, SIGNAL(triggered()), this, SLOT(selectOutTitleActing()));

    displayAction = new QAction(this);
    displayAction->setIcon(QIcon(":img/display_32x32.png"));
    connect(displayAction, SIGNAL(triggered()), this, SLOT(displayActing()));

    addSimilarAction = new QAction(this);
    addSimilarAction->setIcon(QIcon(":img/similar_32x32.png"));
    connect(addSimilarAction, SIGNAL(triggered()), this, SLOT(addSimilarActing()));

    restoringForEditingAction = new QAction(this);
    restoringForEditingAction->setIcon(QIcon(":img/edit_32x32.png"));
    connect(restoringForEditingAction, SIGNAL(triggered()), this, SLOT(restoringForEditingActing()));

    restoringForPrintingAction = new QAction(this);
    restoringForPrintingAction->setIcon(QIcon(":img/prn_32x32.png"));
    connect(restoringForPrintingAction, SIGNAL(triggered()), this, SLOT(restoringForPrintingActing()));

    restoringForFollowingUsageAction = new QAction(this);
    restoringForFollowingUsageAction->setIcon(QIcon(":img/usb_32x32.png"));
    connect(restoringForFollowingUsageAction, SIGNAL(triggered()), this, SLOT(restoringForFollowingUsageActing()));

    fillAllAction = new QAction(this);
    fillAllAction->setIcon(QIcon(":img/fill_all_32x32.png"));
    connect(fillAllAction, SIGNAL(triggered()), this, SLOT(fillAllActing()));

    fillOnlyCategoryAction = new QAction(this);
    fillOnlyCategoryAction->setIcon(QIcon(":img/fill_category_32x32.png"));
    connect(fillOnlyCategoryAction, SIGNAL(triggered()), this, SLOT(fillOnlyCategoryActing()));

    fillOnlyKeywordAction = new QAction(this);
    fillOnlyKeywordAction->setIcon(QIcon(":img/fill_keyword_32x32.png"));
    connect(fillOnlyKeywordAction, SIGNAL(triggered()), this, SLOT(fillOnlyKeywordActing()));

    fillOnlyTitleAction = new QAction(this);
    fillOnlyTitleAction->setIcon(QIcon(":img/fill_title_32x32.png"));
    connect(fillOnlyTitleAction, SIGNAL(triggered()), this, SLOT(fillOnlyTitleActing()));

    checkAction = new QAction(this);
    checkAction->setIcon(QIcon(":img/check_32x32.png"));
    connect(checkAction, SIGNAL(triggered()), this, SLOT(checkActing()));

    infoAction = new QAction(this);
    infoAction->setIcon(QIcon(":img/info_32x32.png"));
    connect(infoAction, SIGNAL(triggered()), this, SLOT(infoActing()));
}

void MainWindow::createMenus()
{        
    // Language menu
    createLanguageMenu();

    // Base menu
    baseMenu = new QMenu(this);
    menuBar()->addMenu(baseMenu);
    baseMenu->addAction(settingsAction);    
    baseMenu->addAction(aboutAction);
    baseMenu->addSeparator();
    baseMenu->addMenu(languageMenu);
    baseMenu->addSeparator();
    baseMenu->addAction(quitAction);

    // Boxes menu
    boxesMenu = new QMenu(this);
    menuBar()->addMenu(boxesMenu);
    boxesMenu->addAction(photoBoxAction);
    boxesMenu->addAction(bookBoxAction);
    boxesMenu->addAction(videoBoxAction);
    boxesMenu->addAction(musicBoxAction);

    // Source menu
    sourceMenu = new QMenu(this);
    menuBar()->addMenu(sourceMenu);
    sourceMenu->addAction(firstStepsAction);
    sourceMenu->addAction(arrangementAction);

    // Select menu
    selectMenu = new QMenu(this);
    menuBar()->addMenu(selectMenu);
    selectMenu->addAction(selectDialogAction);
    selectMenu->addAction(selectTemplateAction);
    selectMenu->addSeparator();
    selectMenu->addAction(selectOutCategoryAction);
    selectMenu->addAction(selectOutKeywordAction);
    selectMenu->addAction(selectOutTitleAction);

    // Restoring menu
    restoringMenu = new QMenu(this);
    restoringMenu->setIcon(QIcon(":img/restore_32x32.png"));
    restoringMenu->addAction(restoringForEditingAction);
    restoringMenu->addAction(restoringForPrintingAction);
    restoringMenu->addAction(restoringForFollowingUsageAction);

    // Fill menu
    fillMenu = new QMenu(this);
    fillMenu->setIcon(QIcon(":img/fill_32x32.png"));
    fillMenu->addAction(fillAllAction);
    fillMenu->addAction(fillOnlyCategoryAction);
    fillMenu->addAction(fillOnlyKeywordAction);
    fillMenu->addAction(fillOnlyTitleAction);

    // Action on selected menu
    actionOnSelectedMenu = new QMenu(this);
    menuBar()->addMenu(actionOnSelectedMenu);
    actionOnSelectedMenu->addAction(displayAction);
    actionOnSelectedMenu->addAction(addSimilarAction);
    actionOnSelectedMenu->addSeparator();
    actionOnSelectedMenu->addMenu(restoringMenu);
    actionOnSelectedMenu->addMenu(fillMenu);

    // Verify menu
    verifyMenu = new QMenu(this);
    menuBar()->addMenu(verifyMenu);
    verifyMenu->addAction(checkAction);

    // Info menu
    infoMenu = new QMenu(this);
    menuBar()->addMenu(infoMenu);
    infoMenu->addAction(infoAction);
}

void MainWindow::createToolBars()
{
    // Boxes toolbar    
    boxesToolBar = new QToolBar(this);
    boxesToolBar->addAction(photoBoxAction);
    boxesToolBar->addAction(bookBoxAction);
    boxesToolBar->addAction(videoBoxAction);
    boxesToolBar->addAction(musicBoxAction);

    // Write toolbar    
    writeToolBar = new QToolBar(this);
    writeToolBar->addAction(firstStepsAction);
    writeToolBar->addAction(arrangementAction);
    writeToolBar->addAction(fillAllAction);

    // Read toolbar    
    readToolBar = new QToolBar(this);
    readToolBar->addAction(selectDialogAction);
    readToolBar->addAction(selectTemplateAction);
    readToolBar->addAction(displayAction);

    // Restoring toolbar    
    restoringToolBar = new QToolBar(this);
    restoringToolBar->addAction(restoringForEditingAction);
    restoringToolBar->addAction(restoringForPrintingAction);
    restoringToolBar->addAction(restoringForFollowingUsageAction);

    // Misc toolbar    
    miscToolBar = new QToolBar(this);
    miscToolBar->addAction(checkAction);
    miscToolBar->addAction(infoAction);

    // Base toolbar    
    baseToolBar = new QToolBar(this);
    baseToolBar->addAction(settingsAction);
    baseToolBar->addAction(aboutAction);
    baseToolBar->addAction(quitAction);

    // Compose toolbars
    addToolBar(boxesToolBar);
    addToolBar(writeToolBar);
    addToolBar(readToolBar);
    addToolBar(restoringToolBar);
    addToolBar(miscToolBar);
    addToolBar(baseToolBar);
}

void MainWindow::createStatusBar()
{
    m_statusLabel = new QLabel();
    m_statusLabel->setAlignment(Qt::AlignRight);
    m_statusLabel->setMargin(3);

    statusBar()->addWidget(m_statusLabel,1);

    updateStatusBar();
}

void MainWindow::settingsActing()
{    
    MySetDialog dialog(m_set);

    if (dialog.exec()){        
        dialog.saveSettings();
        m_myBox->updateSettings();
        if (dialog.needReboot()){
            QMessageBox::warning(this,
                                 tr("Warning: Change requires restart"),
                                 tr("The path to MyBox was changed. To work with the new directory, you must restart the program")
                                 );
        }
    }
}

void MainWindow::aboutActing()
{

    QString txt;

    txt.append("<h2>It's MyBox<sup> All standing</sup></h2>");
    txt.append(tr("Get under control..."
               "<ul><li>photos</li>"
               "<li>books</li>"
               "<li>video clips</li>"
               "<li>musics </li>"
               "</ul>")
               );
    txt.append(tr("... it is possible now."));

    QMessageBox msgBox;
    msgBox.setIconPixmap(QPixmap(":img/about_250x232.png"));
    msgBox.setWindowTitle(tr("About"));
    msgBox.setWindowIcon(QIcon(QPixmap(":img/mybox_logo2.ico")));
    msgBox.setText(txt);
    msgBox.exec();

}

void MainWindow::photoBoxActing()
{
    clearCentralWidget();

    m_currentBox = static_cast<AnyBox*>(m_myBox->baseFolder()->unit(NAME_PHOTO));

    new BoxSelect(m_currentBox, m_centralFrame);

    updateStatusBar();
}

void MainWindow::bookBoxActing()
{
    clearCentralWidget();

    m_currentBox = static_cast<AnyBox*>(m_myBox->baseFolder()->unit(NAME_BOOK));

    new BoxSelect(m_currentBox, m_centralFrame);

    updateStatusBar();
}

void MainWindow::videoBoxActing()
{
    clearCentralWidget();
    m_currentBox = static_cast<AnyBox*>(m_myBox->baseFolder()->unit(NAME_CLIP));

    new BoxSelect(m_currentBox, m_centralFrame);

    updateStatusBar();
}

void MainWindow::musicBoxActing()
{
    clearCentralWidget();
    m_currentBox = static_cast<AnyBox*>(m_myBox->baseFolder()->unit(NAME_MUSIC));

    new BoxSelect(m_currentBox, m_centralFrame);

    updateStatusBar();
}

void MainWindow::firstStepsActing()
{
    clearCentralWidget();

    new FirstDialog(m_currentBox, m_centralFrame);
}

void MainWindow::arrangementActing()
{
    clearCentralWidget();    

    Arrangement *d = new Arrangement(m_currentBox, m_centralFrame);
    connect(d, SIGNAL(needUpdateStatus()), this, SLOT(updateStatusBar()));    
}

void MainWindow::selectDialogActing()
{
    clearCentralWidget();

    SearchDialog *d = new SearchDialog(m_currentBox, m_centralFrame);
    connect(d, SIGNAL(needUpdateStatus()), this, SLOT(updateStatusBar()));
}

void MainWindow::selectTemplateActing()
{
    clearCentralWidget();

    TemplateDialog *d = new TemplateDialog(m_currentBox, m_centralFrame);
    connect(d, SIGNAL(needUpdateStatus()), this, SLOT(updateStatusBar()));    
}

void MainWindow::selectOutCategoryActing()
{
    clearCentralWidget();   

    WithoutCategoryDialog *d = new WithoutCategoryDialog(m_currentBox, m_centralFrame);
    connect(d, SIGNAL(needUpdateStatus()), this, SLOT(updateStatusBar()));    
}

void MainWindow::selectOutKeywordActing()
{
    clearCentralWidget();

    WithoutKeywordDialog *d = new WithoutKeywordDialog(m_currentBox, m_centralFrame);
    connect(d, SIGNAL(needUpdateStatus()), this, SLOT(updateStatusBar()));    
}

void MainWindow::selectOutTitleActing()
{
    clearCentralWidget();

    WithoutTitleDialog *d = new WithoutTitleDialog(m_currentBox, m_centralFrame);
    connect(d, SIGNAL(needUpdateStatus()), this, SLOT(updateStatusBar()));    
}

void MainWindow::displayActing()
{
    clearCentralWidget();

    new DisplayDialog(m_currentBox, m_centralFrame);
}

void MainWindow::addSimilarActing()
{
    clearCentralWidget();

    ResembleDialog *d = new ResembleDialog(m_currentBox, m_centralFrame);
    connect(d, SIGNAL(needUpdateStatus()), this, SLOT(updateStatusBar()));    
}

void MainWindow::restoringForEditingActing()
{
    clearCentralWidget();

    new RestoreEditDialog(m_currentBox, m_centralFrame);
}

void MainWindow::restoringForPrintingActing()
{
    clearCentralWidget();

    new RestorePrintDialog(m_currentBox, m_centralFrame);
}

void MainWindow::restoringForFollowingUsageActing()
{
    clearCentralWidget();

    new RestoreOtherDialog(m_currentBox, m_centralFrame);
}

void MainWindow::fillAllActing()
{
    clearCentralWidget();

    new DescribeDialog(m_currentBox, m_centralFrame);
}

void MainWindow::fillOnlyCategoryActing()
{
    clearCentralWidget();

    DescribeDialog* d;
    d = new DescribeDialog(m_currentBox, m_centralFrame);
    d->setTabVisible(0);    
}

void MainWindow::fillOnlyKeywordActing()
{
    clearCentralWidget();

    DescribeDialog* d;
    d = new DescribeDialog(m_currentBox, m_centralFrame);
    d->setTabVisible(1);    
}

void MainWindow::fillOnlyTitleActing()
{
    clearCentralWidget();

    DescribeDialog* d;
    d = new DescribeDialog(m_currentBox, m_centralFrame);
    d->setTabVisible(2);    
}

void MainWindow::checkActing()
{
    clearCentralWidget();

    new VerifyDialog(m_currentBox, m_centralFrame);
}

void MainWindow::infoActing()
{
    clearCentralWidget(); 

    new BoxInfo(m_currentBox, m_centralFrame);
}

void MainWindow::clearCentralWidget()
{
    delete m_centralFrame;
    m_centralFrame = new QFrame();
    setCentralWidget(m_centralFrame);
}

void MainWindow::updateStatusBar()
{
    m_statusLabel->setText(STATUS_TAMPLATE.arg(m_currentBox->objectName(),
                                               QString::number(m_currentBox->markedEntriesCount()),
                                               QString::number(m_currentBox->getCount())));
}

void MainWindow::retranslateUi()
{
    settingsAction->setText(tr("Settings"));
    settingsAction->setShortcut(tr("Ctrl+/"));
    settingsAction->setToolTip(tr("Setting the program parameters"));

    aboutAction->setText(tr("About"));
    //aboutAction->setShortcut(tr(""));
    aboutAction->setToolTip(tr("About this program"));

    quitAction->setText(tr("Quit"));
    quitAction->setShortcut(tr("Ctrl+Q"));
    quitAction->setToolTip(tr("Finish the work and close the program"));

    photoBoxAction->setText(tr("Images"));
    photoBoxAction->setShortcut(tr("F1"));
    photoBoxAction->setToolTip(tr("Start working with image collection"));

    bookBoxAction->setText(tr("Books"));
    bookBoxAction->setShortcut(tr("F2"));
    bookBoxAction->setToolTip(tr("Start working with book library"));

    videoBoxAction->setText(tr("Video"));
    videoBoxAction->setShortcut(tr("F3"));
    videoBoxAction->setToolTip(tr("Start working with a film library"));

    musicBoxAction->setText(tr("Music"));
    musicBoxAction->setShortcut(tr("F4"));
    musicBoxAction->setToolTip(tr("Start working with a music box"));

    firstStepsAction->setText(tr("First steps"));
    //firstStepsAction->setShortcut(tr(""));
    firstStepsAction->setToolTip(tr("Transfer files from media and their rejection"));

    arrangementAction->setText(tr("Arrangement"));
    //arrangementAction->setShortcut(tr(""));
    arrangementAction->setToolTip(tr("Placement of files in the box"));

    selectDialogAction->setText(tr("Dialog"));
    //selectDialogAction->setShortcuts(tr(""));
    selectDialogAction->setToolTip((tr("Select objects using the dialog box")));

    selectTemplateAction->setText(tr("Template"));
    //selectTemplateAction->setShortcut(tr(""));
    selectTemplateAction->setToolTip(tr("Select objects using the existing templates"));

    selectOutCategoryAction->setText(tr("Without category"));
    //selectOutCategoryAction->setShortcut(tr(""));
    selectOutCategoryAction->setToolTip(tr("Select the objects that have not been filled category"));

    selectOutKeywordAction->setText(tr("Without keyword"));
    //selectOutKeywordAction->setShortcut(tr(""));
    selectOutKeywordAction->setToolTip(tr("Select the objects that have no keywords"));

    selectOutTitleAction->setText(tr("Without caption"));
    //selectOutTitleAction->setShortcut(tr(""));
    selectOutTitleAction->setToolTip(tr("Select objects untitled"));

    displayAction->setText(tr("Display"));
    //displayAction->setShortcut(tr(""));
    displayAction->setToolTip(tr("Show selected objects"));

    addSimilarAction->setText(tr("Add similar"));
    //addSimilarAction->setShortcut(tr(""));
    addSimilarAction->setToolTip(tr("Create a group of similar objects"));

    restoringForEditingAction->setText(tr("For editing"));
    //restoringForEditingAction->setShortcut(tr(""));
    restoringForEditingAction->setToolTip(tr("Return the selected objects for editing"));

    restoringForPrintingAction->setText(tr("For printing"));
    //restoringForPrintingAction->setShortcut(tr(""));
    restoringForPrintingAction->setToolTip(tr("Return the selected objects to print them"));

    restoringForFollowingUsageAction->setText(tr("For following usage"));
    //restoringForFollowingUsageAction->setShortcut(tr(""));
    restoringForFollowingUsageAction->setToolTip(tr("Return the selected objects"));

    fillAllAction->setText(tr("All"));
    //fillAllAction->setShortcut(tr(""));
    fillAllAction->setToolTip(tr("Write a description of the object completely"));

    fillOnlyCategoryAction->setText(tr("Only category"));
    //fillOnlyCategoryAction->setShortcut(tr(""));
    fillOnlyCategoryAction->setToolTip(tr("Define the category of the object"));

    fillOnlyKeywordAction->setText(tr("Only keyword"));
    //fillOnlyKeywordAction->setShortcut(tr(""));
    fillOnlyKeywordAction->setToolTip(tr("Make a list of keywords for the object"));

    fillOnlyTitleAction->setText(tr("Only caption"));
    //fillOnlyTitleAction->setShortcut(tr(""));
    fillOnlyTitleAction->setToolTip(tr("Sign object"));

    checkAction->setText(tr("Start"));
    //checkAction->setShortcut(tr(""));
    checkAction->setToolTip(tr("Check the box for any errors"));

    infoAction->setText(tr("Show information"));
    //infoAction->setShortcut(tr(""));
    infoAction->setToolTip(tr("Information about the content"));

    baseMenu->setTitle(tr("&..."));
    boxesMenu->setTitle(tr("&Boxes"));
    sourceMenu->setTitle(tr("S&ource"));
    selectMenu->setTitle(tr("&Select"));
    actionOnSelectedMenu->setTitle(tr("&Action on selected"));
    restoringMenu->setTitle(tr("&Restoring"));
    fillMenu->setTitle(tr("&Fill"));
    verifyMenu->setTitle(tr("&Verify"));
    infoMenu->setTitle(tr("&Info"));
    languageMenu->setTitle(tr("&Language"));

    boxesToolBar->setWindowTitle(tr("Boxes"));
    writeToolBar->setWindowTitle(tr("Write"));
    readToolBar->setWindowTitle(tr("Read"));
    restoringToolBar->setWindowTitle(tr("Restoring"));
    miscToolBar->setWindowTitle(tr("Misc"));
    baseToolBar->setWindowTitle(tr("Base"));

    m_myBox->setWelcomeLabels();
}

void MainWindow::createLanguageMenu()
{
    languageMenu = new QMenu(this);
    languageActionGroup = new QActionGroup(this);
    connect(languageActionGroup, SIGNAL(triggered(QAction*)), this, SLOT(switchLanguage(QAction*)));
    QDir qmDir(":/lng");

    QStringList fileNames;

    QRegExp exp("^qt_.*\\.qm$");
    foreach(QString n, qmDir.entryList(QStringList("*.qm"))){
        if (!exp.exactMatch(n)) {
            fileNames << n;
        }
    }

    bool langChecked = false;

    for (int i = 0; i < fileNames.size(); ++i){

        QString locale = fileNames[i];
        locale.remove(0, locale.indexOf("_") + 1);
        locale.chop(3);

        QTranslator translator;
        translator.load(fileNames[i], qmDir.absolutePath());
        QString language = translator.translate("MainWindow", "English");

        QAction *action = new QAction(tr("&%1 %2").arg(i + 1).arg(language), this);
        action->setCheckable(true);
        action->setData(locale);
        languageMenu->addAction(action);
        languageActionGroup->addAction(action);

        if (!langChecked && (language == "English")){
            action->setChecked(true);
        }

        if (locale == m_set->value(KEY_GENERAL + KEY_LANGUAGE).toString()){
            action->setChecked(true);
            langChecked = true;
        }
    }           
}

void MainWindow::setAppTranslator(QTranslator *t)
{
    m_appTranslator = t;
}

void MainWindow::setQtTranslator(QTranslator *t)
{
    m_qtTranslator = t;
}


void MainWindow::switchLanguage(QAction *action)
{
    QString locale = action->data().toString();
    QDir qmDir(":/lng");
    QString qmPath = qmDir.absolutePath();
    m_appTranslator->load(locale, qmPath);
    m_qtTranslator->load("qt_" + locale, qmPath);
    m_set->setValue(KEY_GENERAL + KEY_LANGUAGE, locale);
    retranslateUi();
}
