#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "mybox.h"
#include "anybox.h"
#include "mysettings.h"
#include "firstdialog.h"
#include "arrangement.h"
#include "displaydialog.h"
#include "describedialog.h"
#include "searchdialog.h"
#include "templatedialog.h"
#include "withoutcategorydialog.h"
#include "withoutkeyworddialog.h"
#include "withouttitledialog.h"
#include "restoreeditdialog.h"
#include "restoreprintdialog.h"
#include "restoreotherdialog.h"
#include "resembledialog.h"
#include "verifydialog.h"
#include "boxinfo.h"
#include "mysetdialog.h"
#include "boxselect.h"

#include <QMainWindow>
#include <QActionGroup>
#include <QTranslator>
#include <QMenu>
#include <QMenuBar>
#include <QStatusBar>
#include <QToolBar>
#include <QCloseEvent>

const QString STATUS_TAMPLATE = "<b>%1</b> (%2 / %3)";

class MainWindow:public QMainWindow
{
    Q_OBJECT

    MyBox *m_myBox;
    AnyBox *m_currentBox;
    MySettings *m_set;

    QTranslator *m_appTranslator;
    QTranslator *m_qtTranslator;

    QLabel* m_statusLabel;

    QFrame* m_centralFrame;

public:
    MainWindow(MySettings *);
    bool initStatus(QString &message);

    void setAppTranslator(QTranslator *);
    void setQtTranslator(QTranslator *);

protected:
    void closeEvent(QCloseEvent *);

private slots:
    void settingsActing();
    void aboutActing();
    void photoBoxActing();
    void bookBoxActing();
    void videoBoxActing();
    void musicBoxActing();
    void firstStepsActing();
    void arrangementActing();
    void selectDialogActing();
    void selectTemplateActing();
    void selectOutCategoryActing();
    void selectOutKeywordActing();
    void selectOutTitleActing();
    void displayActing();
    void addSimilarActing();
    void restoringForEditingActing();
    void restoringForPrintingActing();
    void restoringForFollowingUsageActing();
    void fillAllActing();
    void fillOnlyCategoryActing();
    void fillOnlyKeywordActing();
    void fillOnlyTitleActing();
    void checkActing();
    void infoActing();
    void updateStatusBar();
    void switchLanguage(QAction*);

private:
    // Actions
    void createActions();
    QAction *settingsAction;
    QAction *aboutAction;
    QAction *quitAction;
    QAction *photoBoxAction;
    QAction *bookBoxAction;
    QAction *videoBoxAction;
    QAction *musicBoxAction;
    QAction *firstStepsAction;
    QAction *arrangementAction;
    QAction *selectDialogAction;
    QAction *selectTemplateAction;
    QAction *selectOutCategoryAction;
    QAction *selectOutKeywordAction;
    QAction *selectOutTitleAction;
    QAction *displayAction;
    QAction *addSimilarAction;
    QAction *restoringForEditingAction;
    QAction *restoringForPrintingAction;
    QAction *restoringForFollowingUsageAction;
    QAction *fillAllAction;
    QAction *fillOnlyCategoryAction;
    QAction *fillOnlyKeywordAction;
    QAction *fillOnlyTitleAction;
    QAction *checkAction;
    QAction *infoAction;

    QActionGroup *boxesActionGroup;
    QActionGroup *languageActionGroup;

    // Menus
    void createMenus();
    void createLanguageMenu();
    QMenu *baseMenu;
    QMenu *boxesMenu;
    QMenu *sourceMenu;
    QMenu *selectMenu;
    QMenu *actionOnSelectedMenu;
    QMenu *restoringMenu;
    QMenu *fillMenu;
    QMenu *verifyMenu;
    QMenu *infoMenu;
    QMenu *languageMenu;

    // ToolBars
    void createToolBars();
    QToolBar *boxesToolBar;
    QToolBar *writeToolBar;
    QToolBar *readToolBar;
    QToolBar *restoringToolBar;
    QToolBar *miscToolBar;
    QToolBar *baseToolBar;

    // StatusBar
    void createStatusBar();

    void clearCentralWidget();
    void retranslateUi();
};

#endif
