#include "myabcfile.h"
#include "remark.h"
#include "global.h"
#include <QFile>
#include <QTextStream>

MyAbcFile::MyAbcFile(MyUnit* p) : MyFile(NAME_ABC_FILE, p)
{
    m_rema = parent()->parent()->objectName() + parent()->objectName();
    setObjectName(NAME_ABC_FILE + "_" + this->m_rema);

    QFile f(this->completeName());

    QTextStream st(&f);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    if (initOk()) {
        if (f.open(QIODevice::ReadOnly | QIODevice::Text)) {
            while (!st.atEnd()){
                Entry parts(st.readLine());
                if (parts.Ok()){
                    m_entries[parts.simpleName()] = parts;
                }
            }
            f.close();
        }
    }

    m_subsets.clear();
}

QString MyAbcFile::rema()
{
    return m_rema;
}

bool MyAbcFile::entryExist(QString rk)
{
    if (m_entries.contains(rk)){
        return true;
    } else {
        return false;
    }
}

Entry* MyAbcFile::getEntry(QString rk)
{ 
    return &m_entries[rk];
}


QString MyAbcFile::getEntryPart(QString rk, int field)
{    
    Entry parts = m_entries.value(rk);
    if (field == ALL) {
        return parts.getAllFields();
    } else {
        return parts.getField(field);
    }
}

QStringList MyAbcFile::getUsedName()
{ 
    QStringList usedName;
    foreach (Entry parts, m_entries.values()){
        if (parts.Ok()){
            Remark name(parts.getField(REMARK));
            usedName.append(name.rk());
        }
    }
    return usedName;
}

QList<Entry> MyAbcFile::getAll()
{   
    return m_entries.values();
}

bool MyAbcFile::update(Entry a)
{    
    bool result = false;

    if (a.Ok()){
        result = true;
        m_entries[a.simpleName()] = a;
    }

    return result;
}

bool MyAbcFile::store()
{   
    QFile f(this->completeName());

    QTextStream st(&f);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    QString line;

    bool result = false;

    if (initOk()) {
        if (f.open(QIODevice::WriteOnly | QIODevice::Text)) {
            result = true;
            foreach(Entry tmp, m_entries.values()){
                line = tmp.getAllFields();
                if (!line.isEmpty()){
                    st << line << endl;
                }
            }
            f.close();
        }
    }
    return result;
}

int MyAbcFile::countEntries()
{
    return m_entries.count();
}

bool MyAbcFile::markEntry(Entry* entry, int subset)
{
    bool result = false;
    Remark remark;
    remark = entry->getField(REMARK);

    if (remark.rema() != m_rema) { return result; }

    QString rk;
    rk = remark.rk();

    if (!m_entries.contains(rk)) { return result; }

    result = true;

    if (subset < m_subsets.count()){
        m_subsets[subset] << &m_entries[rk];
    } else {
        MyAbcFile::subset s;
        s << &m_entries[rk];
        m_subsets << s;
    }

    emit markAbcFile(this, subset);
    return result;
}

bool MyAbcFile::unmarkEntry(Entry* entry, int subset)
{
    bool result = false;
    if (subset < m_subsets.count()){
        result = m_subsets[subset].remove(entry);
        if (m_subsets.at(subset).isEmpty()){
            emit unmarkAbcFile(this, subset);
        }
    }
    return result;
}

int MyAbcFile::markAllEntries(int subset)
{    
    int result = subset;
    if (subset < m_subsets.count()){
        m_subsets[subset].clear();
        foreach(QString rk, m_entries.keys()){
            m_subsets[subset] << &m_entries[rk];
        }
    } else {
        MyAbcFile::subset s;
        foreach(QString rk, m_entries.keys()){
           s <<  &m_entries[rk];
        }
        m_subsets << s;
        result = m_subsets.count() - 1;
    }
    emit markAbcFile(this, subset);
    return result;
}

void MyAbcFile::deleteSubsets()
{
    for(int subset = 0; subset < m_subsets.count(); subset++){
        deleteSubset(subset);
    }
}

void MyAbcFile::deleteSubset(int subset)
{ 
    if (subset < m_subsets.count()){
        m_subsets.removeAt(subset);
        emit unmarkAbcFile(this, subset);
    }
}

QList<Entry*> MyAbcFile::markedEntries()
{
    subset sel;
    foreach(subset s, m_subsets){
        sel.unite(s);
    }
    QList<Entry*> result;
    foreach(Entry* entry, sel){
        result << entry;
    }
    return result;
}

QList<Entry*> MyAbcFile::markedEntries(int subset)
{
    QList<Entry*> result;

    if (subset < m_subsets.count()){
        foreach(Entry* entry, m_subsets.at(subset)){
            result << entry;
        }
    }

    return result;
}

int MyAbcFile::markedEntriesCount()
{
    subset sel;
    foreach(subset s, m_subsets){
        sel.unite(s);
    }
    return sel.count();
}

int MyAbcFile::markedEntriesCount(int subset)
{
    int result = 0;
    if (subset < m_subsets.count()){
        result = m_subsets.at(subset).count();
    }
    return result;
}

QList <Entry*> MyAbcFile::allEntries()
{
    QList<Entry*> result;
    foreach(Entry e,m_entries.values() ){
        result << &e;
    }

    return result;
}
void MyAbcFile::memorizeSubsets()
{
    m_subsetsMemory = m_subsets;
}

void MyAbcFile::restoreSubsetsFromMemory()
{
    m_subsets = m_subsetsMemory;
    m_subsetsMemory.clear();
}

void MyAbcFile::uniteSubsets()
{
    QSet<Entry*> sub;
    foreach(QSet<Entry*> s, m_subsets){
        sub.unite(s);
    }
    m_subsets.clear();
    m_subsets << sub;
}
