#include "mybox.h"
#include "global.h"
#include "myfile.h"
#include "mylogfile.h"
#include "mysssfile.h"
#include "mywrdfile.h"
#include "mytmpfile.h"
#include "category.h"

#include <QDir>

MyBox::MyBox(MySettings *set) : QWidget()
{    
    m_set = set;

    QString strMyBoxDir = set->value(KEY_GENERAL + KEY_BOX_DIR).toString();
    QDir myBoxDir(strMyBoxDir);
    QString pathMyBoxDir;
    pathMyBoxDir = strMyBoxDir.left(strMyBoxDir.lastIndexOf(myBoxDir.dirName()));

    m_baseFolder = new BaseFolder(pathMyBoxDir, myBoxDir.dirName());
    m_baseFolder->setFirstDir(set->value(KEY_GENERAL + KEY_FIRST_DIR).toString());
    m_baseFolder->setBackupDir(set->value(KEY_GENERAL + KEY_CPY_DIR).toString());
    m_baseFolder->setBurnCDApp(set->value(KEY_GENERAL + KEY_BURN_CD).toString());
    m_baseFolder->setExtractApp(set->value(KEY_GENERAL + KEY_EXTRACT_APP).toString());

    //Photo    
    m_photoBox = new PhotoBox(NAME_PHOTO, m_baseFolder);
    m_photoBox->setExt(m_set->value(KEY_PHOTO + KEY_EXT).toString());
    m_photoBox->setExtPlayList(NAME_PHOTO_PLAYLIST_EXT);
    m_photoBox->setExecPreview(m_set->value(KEY_PHOTO + KEY_PREVIEW_APP).toString());
    m_photoBox->setExecPresent(m_set->value(KEY_PHOTO + KEY_PRESENT_APP).toString());
    m_photoBox->setMyCategories(m_set->value(KEY_PHOTO + KEY_CATEGORIES).toString());
    m_photoBox->setCategoryLabels();
    m_photoBox->setBoxPixmap(QPixmap(":img/photo_32x32.png"));

    //Book
    m_bookBox = new BookBox(NAME_BOOK, m_baseFolder);
    m_bookBox->setExt(set->value(KEY_BOOK + KEY_EXT).toString());
    m_bookBox->setExtPlayList(NAME_BOOK_PLAYLIST_EXT);
    m_bookBox->setExecPreview(set->value(KEY_BOOK + KEY_PREVIEW_APP).toString());
    m_bookBox->setExecPresent(set->value(KEY_BOOK + KEY_PRESENT_APP).toString());
    m_bookBox->setMyCategories(m_set->value(KEY_BOOK + KEY_CATEGORIES).toString());
    m_bookBox->setCategoryLabels();
    m_bookBox->setBoxPixmap(QPixmap(":img/book_32x32.png"));

    //Clip
    m_clipBox = new ClipBox (NAME_CLIP, m_baseFolder);
    m_clipBox->setExt(set->value(KEY_CLIP + KEY_EXT).toString());
    m_clipBox->setExtPlayList(NAME_CLIP_PLAYLIST_EXT);
    m_clipBox->setExecPreview(set->value(KEY_CLIP + KEY_PREVIEW_APP).toString());
    m_clipBox->setExecPresent(set->value(KEY_CLIP + KEY_PRESENT_APP).toString());
    m_clipBox->setMyCategories(m_set->value(KEY_CLIP + KEY_CATEGORIES).toString());
    m_clipBox->setCategoryLabels();
    m_clipBox->setBoxPixmap(QPixmap(":img/video_32x32.png"));

    //Music
    m_musicBox = new MusicBox (NAME_MUSIC, m_baseFolder);
    m_musicBox->setExt(set->value(KEY_MUSIC + KEY_EXT).toString());
    m_musicBox->setExtPlayList(NAME_MUSIC_PLAYLIST_EXT);
    m_musicBox->setExecPreview(set->value(KEY_MUSIC + KEY_PREVIEW_APP).toString());
    m_musicBox->setExecPresent(set->value(KEY_MUSIC + KEY_PRESENT_APP).toString());
    m_musicBox->setMyCategories(m_set->value(KEY_MUSIC + KEY_CATEGORIES).toString());
    m_musicBox->setCategoryLabels();
    m_musicBox->setBoxPixmap(QPixmap(":img/music_32x32.png"));

    setWelcomeLabels();

    QString str = set->value(KEY_GENERAL + KEY_EXTRACT_META_APP).toString();
    m_srcFolder = new MySrcFolder(str, NAME_SRC_FOLDER, m_baseFolder);
    new MyFolder(NAME_DST_FOLDER, m_baseFolder);
    new MyFolder(NAME_SLD_FOLDER, m_baseFolder);

    new MySssFile(NAME_SSS_FILE, m_baseFolder);
    new MyWrdFile(NAME_WRD_FILE, m_baseFolder);
    new MyLogFile(NAME_LOG_FILE, m_baseFolder);
    new MyTmpFile(NAME_TMP_FILE, m_baseFolder);
}

bool MyBox::initStatus(QString &message)
{
    QList<MyUnit*> units;

    units << m_baseFolder << m_baseFolder->findChildren<MyUnit*>();

    QString str_file;
    QString str_dir;

    str_dir = QObject::tr("Directory '%1'\n cannot be created...");
    str_file = QObject::tr("File '%1'\n cannot be created...");

    foreach (MyUnit *u, units){        
        if (u->objectName().isEmpty()){ continue; }
        if (!u->initOk()){
            if (u->type() == "Directory"){
                message = str_dir.arg(u->completeName());
            } else {
                message = str_file.arg(u->completeName());
            }

            return false;
        }
    }
    message = QObject::tr("MyBox is ready...");
    return true;
}

BaseFolder* MyBox::baseFolder()
{
    return m_baseFolder;
}

void MyBox::updateSettings()
{
    m_baseFolder->setFirstDir(m_set->value(KEY_GENERAL + KEY_FIRST_DIR).toString());
    m_baseFolder->setBackupDir(m_set->value(KEY_GENERAL + KEY_CPY_DIR).toString());
    m_baseFolder->setBurnCDApp(m_set->value(KEY_GENERAL + KEY_BURN_CD).toString());
    m_baseFolder->setExtractApp(m_set->value(KEY_GENERAL + KEY_EXTRACT_APP).toString());

    m_srcFolder->setExtractApp(m_set->value(KEY_GENERAL + KEY_EXTRACT_META_APP).toString()); 

    //Photo    
    m_photoBox->setExt(m_set->value(KEY_PHOTO + KEY_EXT).toString());
    m_photoBox->setExecPreview(m_set->value(KEY_PHOTO + KEY_PREVIEW_APP).toString());
    m_photoBox->setExecPresent(m_set->value(KEY_PHOTO + KEY_PRESENT_APP).toString());
    m_photoBox->setMyCategories(m_set->value(KEY_PHOTO + KEY_CATEGORIES).toString());
    m_photoBox->setCategoryLabels();

    //Book
    m_bookBox->setExt(m_set->value(KEY_BOOK + KEY_EXT).toString());
    m_bookBox->setExecPreview(m_set->value(KEY_BOOK + KEY_PREVIEW_APP).toString());
    m_bookBox->setExecPresent(m_set->value(KEY_BOOK + KEY_PRESENT_APP).toString());
    m_bookBox->setMyCategories(m_set->value(KEY_BOOK + KEY_CATEGORIES).toString());
    m_bookBox->setCategoryLabels();

    //Clip
    m_clipBox->setExt(m_set->value(KEY_CLIP + KEY_EXT).toString());
    m_clipBox->setExecPreview(m_set->value(KEY_CLIP + KEY_PREVIEW_APP).toString());
    m_clipBox->setExecPresent(m_set->value(KEY_CLIP + KEY_PRESENT_APP).toString());
    m_clipBox->setMyCategories(m_set->value(KEY_CLIP + KEY_CATEGORIES).toString());
    m_clipBox->setCategoryLabels();

    //Music
    m_musicBox->setExt(m_set->value(KEY_MUSIC + KEY_EXT).toString());
    m_musicBox->setExecPreview(m_set->value(KEY_MUSIC + KEY_PREVIEW_APP).toString());
    m_musicBox->setExecPresent(m_set->value(KEY_MUSIC + KEY_PRESENT_APP).toString());
    m_musicBox->setMyCategories(m_set->value(KEY_MUSIC + KEY_CATEGORIES).toString());
    m_musicBox->setCategoryLabels();
}

void MyBox::setWelcomeLabels()
{
    m_photoBox->setWelcomeLabel(tr("The operation mode is switched to work with photos"));
    m_bookBox->setWelcomeLabel(tr("The operation mode is switched to work with books"));
    m_clipBox->setWelcomeLabel(tr("The operation mode is switched to work with clips"));
    m_musicBox->setWelcomeLabel(tr("The operation mode is switched to work with musics"));
}

