#include "mysetdialog.h"
#include "global.h"
#include "altqfile.h"

#include <QDir>
#include <QKeyEvent>

enum {PATH_TO_DEVICE, DESCRIPTION_DEVICE, STATUS};

FirstTable::FirstTable(QWidget *parent) : QTableWidget(parent)
{
    setColumnCount(3);

    QTableWidgetItem *p = new QTableWidgetItem();
    p->setText(tr("Path"));
    setHorizontalHeaderItem(0,p);
    QTableWidgetItem *d = new QTableWidgetItem();
    d->setText(tr("Description"));
    setHorizontalHeaderItem(1,d);
    QTableWidgetItem *s = new QTableWidgetItem();
    s->setText(tr("Status"));
    setHorizontalHeaderItem(2,s);

    setObjectName(QString::fromUtf8("firstDirTable"));
    setFrameShape(QFrame::Box);
    setFrameShadow(QFrame::Plain);
    setLineWidth(1);

    setProperty("showDropIndicator", QVariant(false));
    setDragDropOverwriteMode(false);
    setAlternatingRowColors(false);
    setSelectionMode(QAbstractItemView::SingleSelection);
    setSelectionBehavior(QAbstractItemView::SelectRows);
    setCornerButtonEnabled(false);
    horizontalHeader()->setVisible(true);
    horizontalHeader()->setCascadingSectionResizes(false);
    horizontalHeader()->setHighlightSections(false);
    horizontalHeader()->setStretchLastSection(true);
    horizontalHeader()->setSectionResizeMode(QHeaderView::Fixed);

    verticalHeader()->setVisible(false);
    verticalHeader()->setHighlightSections(false);
}

void FirstTable::showEvent(QShowEvent *event)
{
    setColWidth();
}

void FirstTable::resizeEvent(QResizeEvent *event)
{
    setColWidth();
}

void FirstTable::setColWidth()
{
    int w = width();
    setColumnWidth(PATH_TO_DEVICE, w/4);
    setColumnWidth(DESCRIPTION_DEVICE, w/2);
}

bool FirstTable::event(QEvent *event)
{
    if (event->type() == QEvent::KeyPress){
        QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
        switch (keyEvent->key()){
        case Qt::Key_Backtab:
            emit focusLose(false);
            return true;
        case Qt::Key_Tab:
            emit focusLose(true);
            return true;
        }
    }
    return QTableWidget::event(event);
}

MySetDialog::MySetDialog(MySettings *s, QWidget *parent) : QDialog(parent)
{
    setupUi(this);

    firstDirTable = new FirstTable(mySetGeneralTab);
    horizontalLayout->insertWidget(0, firstDirTable);

    myset = s;

    photoTab = new MySetTab(NAME_PHOTO, myset);
    photoTabLayout->addWidget(photoTab);

    bookTab = new MySetTab(NAME_BOOK, myset);
    bookTabLayout->addWidget(bookTab);

    videoTab = new MySetTab(NAME_CLIP, myset);
    videoTabLayout->addWidget(videoTab);

    musicTab = new MySetTab(NAME_MUSIC, myset);
    musicTabLayout->addWidget(musicTab);

    connect(boxDirLineEdit, SIGNAL(textChanged(QString)), this, SLOT(setBoxDirStatus(QString)));
    connect(cpyDirLineEdit, SIGNAL(textChanged(QString)), this, SLOT(setCpyDirStatus(QString)));
    connect(burnAppLineEdit, SIGNAL(textChanged(QString)), this, SLOT(setBurnAppStatus(QString)));
    connect(extractAppLineEdit,SIGNAL(textChanged(QString)),this,SLOT(setExtractAppStatus(QString)));

    connect(boxDirLineEdit, SIGNAL(textChanged(QString)), this, SLOT(setModeBoxDirDefaultButton(QString)));
    connect(cpyDirLineEdit, SIGNAL(textChanged(QString)), this, SLOT(setModeCpyDirDefaultButton(QString)));
    connect(burnAppLineEdit, SIGNAL(textChanged(QString)), this, SLOT(setModeBurnAppDefaultButton(QString)));
    connect(extractAppLineEdit,SIGNAL(textChanged(QString)),this,SLOT(setModeExtractAppDefaultButton(QString)));

    connect(boxDirDefaultButton,SIGNAL(clicked()),this, SLOT(pressBoxDirDefaultButton()));
    connect(cpyDirDefaultButton,SIGNAL(clicked()),this, SLOT(pressCpyDirDefaultButton()));
    connect(burnAppDefaultButton,SIGNAL(clicked()),this, SLOT(pressBurnAppDefaultButton()));
    connect(extractAppDefaultButton,SIGNAL(clicked()),this,SLOT(pressExtractAppDefaultButton()));

    connect(firstDirAddButton,SIGNAL(clicked()), this, SLOT(pressFirstDirAddButton()));
    connect(firstDirDeleteButton,SIGNAL(clicked()), this, SLOT(pressFirstDirDeleteButton()));

    connect(boxDirLineEdit, SIGNAL(textChanged(QString)), this, SLOT(setModeOkButton(QString)));

    boxDirLineEdit->setText(myset->value(KEY_GENERAL + KEY_BOX_DIR).toString());
    m_originalBoxDir = boxDirLineEdit->text();
    cpyDirLineEdit->setText(myset->value(KEY_GENERAL + KEY_CPY_DIR).toString());
    burnAppLineEdit->setText(myset->value(KEY_GENERAL + KEY_BURN_CD).toString());
    extractAppLineEdit->setText(myset->value(KEY_GENERAL + KEY_EXTRACT_META_APP).toString());

    m_listOfFirstDirs = myset->value(KEY_GENERAL + KEY_FIRST_DIR).toString().split(",");

    int row = 0;

    foreach (QString r, m_listOfFirstDirs) {

        QStringList s = r.split("|");
        QString p, d;

        switch (s.count()){
        case 2:
            d = s.at(1).trimmed();
        case 1:
            p = s.at(0).trimmed();
        default:;
        }

        if (!p.isEmpty()) {
            firstDirTable->insertRow(row);
            firstDirTable->setItem(row,PATH_TO_DEVICE,new QTableWidgetItem(p));
            firstDirTable->setItem(row,DESCRIPTION_DEVICE,new QTableWidgetItem(d));

            QTableWidgetItem *i;
            i = new QTableWidgetItem("");
            QFont f;
            f.setItalic(true);
            i->setFont(f);
            i->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
            firstDirTable->setItem(row,STATUS,i);

            setStatus(row);

            row++;
        }
    }

    connect(firstDirTable, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(tableChange(QTableWidgetItem*)));
    connect(firstDirTable, SIGNAL(itemSelectionChanged()), this, SLOT(setModeFirstDeleteButton()));

    boxDirLineEdit->setFocus();
    boxDirLineEdit->installEventFilter(this);
    cpyDirLineEdit->installEventFilter(this);
    burnAppLineEdit->installEventFilter(this);
    extractAppLineEdit->installEventFilter(this);
    firstDirAddButton->installEventFilter(this);

    setTabOrder(extractAppLineEdit,firstDirTable);
    setTabOrder(firstDirTable,firstDirAddButton);
    connect(firstDirTable, SIGNAL(focusLose(bool)),this, SLOT(setFocusAfterFirstTable(bool)));

    retranslateUi(this);
}

void MySetDialog::setStatus(int r)
{
    QTableWidgetItem *i;
    i = firstDirTable->item(r, STATUS);

    QString p;
    p = firstDirTable->item(r, PATH_TO_DEVICE)->text();
    QDir dir(p);

    if (!p.isEmpty()) {
        QString st;
        if (dir.exists()){
            st = tr("Ok");
        } else {
            st = tr("Absent");
        }
        i->setText(st);
    }
}

void MySetDialog::setBoxDirStatus(QString s)
{
    QDir dir(s.simplified());

    if (!dir.isAbsolute()){
        boxDirStatus->setText(QString(tr("The path must be absolute")));
    } else if (!dir.exists()){
        boxDirStatus->setText(QString(tr("Absent")));
    } else {
        boxDirStatus->setText(QString(tr("Ok")));
    }
}

void MySetDialog::setCpyDirStatus(QString s)
{
    QDir dir(s.simplified());

    if (!dir.isAbsolute()){
        cpyDirStatus->setText(QString(tr("The path must be absolute")));
    } else if (!dir.exists()){
        cpyDirStatus->setText(QString(tr("Absent")));
    } else {
        cpyDirStatus->setText(QString(tr("Ok")));
    }
}

void MySetDialog::setBurnAppStatus(QString s)
{
    QString cmd;
    cmd = s.trimmed();

    QRegExp exp("^\".+\"");
    int pos = exp.indexIn(cmd);
    if (pos > -1){
        cmd = exp.cap();
        cmd.remove("\"");
        cmd = cmd.trimmed();
    } else {
        cmd = cmd.split(" ").first();
    }

    if (AltQFile::exists(cmd)){
        burnAppStatus->setText(QString(tr("Ok")));
    } else {
        burnAppStatus->setText(QString(tr("Absent")));
    }
}

void MySetDialog::setExtractAppStatus(QString s)
{
    QString cmd;
    cmd = s.trimmed();

    QRegExp exp("^\".+\"");
    int pos = exp.indexIn(cmd);
    if (pos > -1){
        cmd = exp.cap();
        cmd.remove("\"");
        cmd = cmd.trimmed();
    } else {
        cmd = cmd.split(" ").first();
    }

    QString st;
    if (AltQFile::exists(cmd)){
        st = tr("Ok");
    } else {
        st = tr("Absent");
    }

    extractAppStatus->setText(st);
}

void MySetDialog::pressBoxDirDefaultButton()
{
    boxDirLineEdit->setText(BOX_DIR_DEFAULT);
}

void MySetDialog::pressCpyDirDefaultButton()
{
    cpyDirLineEdit->setText(CPY_DIR_DEFAULT);
}

void MySetDialog::pressBurnAppDefaultButton()
{
    burnAppLineEdit->setText(CMD_BURN_CD_DEFAULT);
}

void MySetDialog::pressExtractAppDefaultButton()
{
    extractAppLineEdit->setText(EXTRACT_META_APP);
}

void MySetDialog::setModeBoxDirDefaultButton(QString s)
{
    if (s.simplified() == BOX_DIR_DEFAULT) {
        boxDirDefaultButton->setEnabled(false);
    } else {
        boxDirDefaultButton->setEnabled(true);
    }
}

void MySetDialog::setModeCpyDirDefaultButton(QString s)
{
    if (s.simplified() == CPY_DIR_DEFAULT) {
        cpyDirDefaultButton->setEnabled(false);
    } else {
        cpyDirDefaultButton->setEnabled(true);
    }
}

void MySetDialog::setModeBurnAppDefaultButton(QString s)
{
    if (s.simplified() == CMD_BURN_CD_DEFAULT) {
        burnAppDefaultButton->setEnabled(false);
    } else {
        burnAppDefaultButton->setEnabled(true);
    }
}

void MySetDialog::setModeExtractAppDefaultButton(QString s)
{
    if (s.simplified() == EXTRACT_META_APP){
        extractAppDefaultButton->setEnabled(false);
    } else {
        extractAppDefaultButton->setEnabled(true);
    }
}

void MySetDialog::pressFirstDirAddButton()
{
    firstDirAddButton->setEnabled(false);

    disconnect(firstDirTable, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(tableChange(QTableWidgetItem*)));

    int lastRow;
    lastRow = firstDirTable->rowCount();
    firstDirTable->insertRow(lastRow);
    m_listOfFirstDirs.append("");
    firstDirTable->setItem(lastRow,PATH_TO_DEVICE,new QTableWidgetItem(""));
    firstDirTable->setItem(lastRow,DESCRIPTION_DEVICE,new QTableWidgetItem(""));

    QTableWidgetItem *i;
    i = new QTableWidgetItem("");
    QFont f;
    f.setItalic(true);
    i->setFont(f);
    i->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
    firstDirTable->setItem(lastRow,STATUS,i);

    setStatus(lastRow);

    QTableWidgetItem *editCell;
    editCell = firstDirTable->item(lastRow, PATH_TO_DEVICE);

    firstDirTable->setCurrentItem(editCell);
    firstDirTable->editItem(editCell);

    connect(firstDirTable, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(tableChange(QTableWidgetItem*)));
}

void MySetDialog::pressFirstDirDeleteButton()
{
    QList<QTableWidgetItem *> rs;

    foreach (QTableWidgetItem *item, firstDirTable->selectedItems()){
        if (item->column() == PATH_TO_DEVICE) {
            rs.append(item);
        }
    }

    if (!rs.isEmpty()){
        int r;
        r = rs.first()->row();
        firstDirTable->removeRow(r);
        m_listOfFirstDirs.removeAt(r);
    }

    setModeFirstAddButton();

    if (firstDirTable->rowCount() == 0) {
        firstDirAddButton->setFocus();
    }
}

void MySetDialog::setModeFirstAddButton()
{
    firstDirAddButton->setEnabled(true);

    int i = 0;
    while (i < firstDirTable->rowCount()){
        if (firstDirTable->item(i, PATH_TO_DEVICE)->text().isEmpty()){
            firstDirAddButton->setEnabled(false);
            break;
        }
        i++;
    }
}

void MySetDialog::setModeFirstDeleteButton()
{
    if (firstDirTable->selectedItems().isEmpty()){
        firstDirDeleteButton->setEnabled(false);
    } else {
        firstDirDeleteButton->setEnabled(true);
    }
}

void MySetDialog::tableChange(QTableWidgetItem *i)
{
    disconnect(firstDirTable, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(tableChange(QTableWidgetItem*)));

    if (i->column() != PATH_TO_DEVICE){
        i->setText(i->text().trimmed());
        connect(firstDirTable, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(tableChange(QTableWidgetItem*)));
        return;
    }

    int r;
    r = i->row();

    QString s;
    s = firstDirTable->item(r, PATH_TO_DEVICE)->text().trimmed();

    if (s.isEmpty()){
        i->setText(m_listOfFirstDirs.at(r));
    }

    m_listOfFirstDirs.replace(r, i->text().trimmed());
    setModeFirstAddButton();
    setStatus(r);

    connect(firstDirTable, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(tableChange(QTableWidgetItem*)));
}

bool MySetDialog::eventFilter(QObject *target, QEvent *event)
{
    if (target == boxDirLineEdit || target == cpyDirLineEdit || target == burnAppLineEdit ||
        target == extractAppLineEdit ||target == firstDirAddButton){
        if (event->type() == QEvent::KeyPress){
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
            if (target == boxDirLineEdit){
                if ((keyEvent->key() == Qt::Key_Return) || (keyEvent->key() == Qt::Key_Enter)){
                    if (keyEvent->modifiers() & Qt::AltModifier){
                        pressBoxDirDefaultButton();
                    }
                    cpyDirLineEdit->setFocus(Qt::TabFocusReason);
                    return true;
                }
            }
            if (target == cpyDirLineEdit){
                if ((keyEvent->key() == Qt::Key_Return) || (keyEvent->key() == Qt::Key_Enter)){
                    if (keyEvent->modifiers() & Qt::AltModifier){
                        pressCpyDirDefaultButton();
                    }
                    burnAppLineEdit->setFocus(Qt::TabFocusReason);
                    return true;
                }
            }
            if (target == burnAppLineEdit){
                if ((keyEvent->key() == Qt::Key_Return) || (keyEvent->key() == Qt::Key_Enter)){
                    if (keyEvent->modifiers() & Qt::AltModifier){
                        pressBurnAppDefaultButton();
                    }
                    extractAppLineEdit->setFocus(Qt::TabFocusReason);
                    return true;
                }
            }
            if (target == extractAppLineEdit){
                switch(keyEvent->key()){
                case Qt::Key_Return:
                case Qt::Key_Enter:
                    if (keyEvent->modifiers() & Qt::AltModifier){
                        pressExtractAppDefaultButton();
                    }
                case Qt::Key_Tab:
                    if (firstDirTable->rowCount() == 0) {
                        firstDirAddButton->setFocus();
                    } else {
                        firstDirTable->setFocus(Qt::TabFocusReason);
                    }
                    return true;
                default:;
                }
            }
            if (target == firstDirAddButton){
                if (keyEvent->key() == Qt::Key_Backtab){
                    if (firstDirTable->rowCount() == 0) {
                        extractAppLineEdit->setFocus(Qt::TabFocusReason);
                    } else {
                        firstDirTable->setFocus(Qt::TabFocusReason);
                    }
                    return true;
                }
            }
        }
    }
    return QDialog::eventFilter(target, event);
}

void MySetDialog::setFocusAfterFirstTable(bool d)
{
    if (d){
        firstDirAddButton->setFocus(Qt::TabFocusReason);
    } else {
        extractAppLineEdit->setFocus(Qt::TabFocusReason);
    }
}

void MySetDialog::saveSettings()
{
    myset->setValue(KEY_GENERAL + KEY_BOX_DIR, boxDirLineEdit->text().simplified());
    myset->setValue(KEY_GENERAL + KEY_CPY_DIR, cpyDirLineEdit->text().simplified());
    myset->setValue(KEY_GENERAL + KEY_BURN_CD, burnAppLineEdit->text().simplified());
    myset->setValue(KEY_GENERAL + KEY_EXTRACT_META_APP, extractAppLineEdit->text().simplified());

    QStringList str;

    for(int r = 0; r < firstDirTable->rowCount(); r++){
        QTableWidgetItem* p = firstDirTable->item(r, PATH_TO_DEVICE);
        QTableWidgetItem* d = firstDirTable->item(r, DESCRIPTION_DEVICE);
        QString s;
        s = p->text().simplified();
        if (!d->text().isEmpty())
        {
            s.append("|" + d->text().simplified());
        }
        str << s;
    }

    myset->setValue(KEY_GENERAL + KEY_FIRST_DIR, str.join(","));

    photoTab->saveSettings();
    bookTab->saveSettings();
    videoTab->saveSettings();
    musicTab->saveSettings();

    myset->sync();
}

bool MySetDialog::needReboot()
{
    QString currentBoxDir;
    currentBoxDir = boxDirLineEdit->text();

    if (currentBoxDir.simplified() == m_originalBoxDir.simplified()){
        return false;
    } else {
        return true;
    }
}

void MySetDialog::setModeOkButton(QString s)
{
    QString strMyBoxDir = s.simplified();
    QDir myBoxDir(strMyBoxDir);

    if (myBoxDir.exists() && myBoxDir.isAbsolute()){
        buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
    } else {
        buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    }
}

