#include "mysettab.h"
#include "altqfile.h"

#include <QKeyEvent>

enum {CATEGORY_COLUMN, DESCRIPTION_COLUMN};

MySetTab::MySetTab(QString name, MySettings *sets, QWidget *parent):
    QWidget(parent)
{
    setupUi(this);

    m_boxName = name;
    m_sets = sets;
    m_myCatEditMode = false;

    foreach(QString cat, m_sets->defaultValue(m_boxName + KEY_CATEGORIES).toString().split(",")){
        m_defCatLst.append(cat.split("|").first().trimmed());
    }

    m_defCatLstTr = retranslateMyDefaultCat(m_defCatLst);

    connect(extAddButton,SIGNAL(clicked()),this,SLOT(pressExtAddButton()));
    connect(extDeleteButton,SIGNAL(clicked()),this,SLOT(pressExtDeleteButton()));
    connect(extClearButton,SIGNAL(clicked()),this,SLOT(pressExtClearButton()));
    connect(extDefaultButton,SIGNAL(clicked()),this,SLOT(pressExtDefaultButton()));

    connect(extLineEdit,SIGNAL(textChanged(QString)),this,SLOT(setModeExtAddButton(QString)));
    connect(extList,SIGNAL(itemSelectionChanged()),this,SLOT(setModeExtDeleteButton()));

    connect(previewAppDefaultButton,SIGNAL(clicked()),this,SLOT(pressPreviewAppDefaultButton()));
    connect(presentAppDefaultButton,SIGNAL(clicked()),this,SLOT(pressPresentAppDefaultButton()));    

    connect(previewAppLineEdit,SIGNAL(textChanged(QString)),this,SLOT(setModePreviewAppDefaultButton(QString)));
    connect(presentAppLineEdit,SIGNAL(textChanged(QString)),this,SLOT(setModePresentAppDefaultButton(QString)));    

    connect(previewAppLineEdit,SIGNAL(textChanged(QString)),this,SLOT(setPreviewAppStatus(QString)));
    connect(presentAppLineEdit,SIGNAL(textChanged(QString)),this,SLOT(setPresentAppStatus(QString)));

    connect(myCatEditButton,SIGNAL(clicked()),this,SLOT(pressMyCatEditButton()));
    connect(myCatDefaultButton,SIGNAL(clicked()),this,SLOT(pressMyCatDefaultButton()));

    previewAppLineEdit->setText(m_sets->value(m_boxName + KEY_PREVIEW_APP).toString());
    presentAppLineEdit->setText(m_sets->value(m_boxName + KEY_PRESENT_APP).toString());

    QStringList exts;
    exts = m_sets->value(m_boxName + KEY_EXT).toString().split(",");
    foreach(QString e, exts){
        QString et = e.trimmed();
        if (!et.isEmpty()){
            extList->addItem(new QListWidgetItem(et));
        }
    }

    setModeExtDeleteButton();
    setModeExtClearButton();
    setModeExtDefaultButton();


    //     ( ).  -    
    foreach(QString cat, m_sets->value(m_boxName + KEY_CATEGORIES).toString().split(",")){
        QStringList lst;
        lst = cat.split("|");
        m_catSettings << lst.at(CATEGORY_COLUMN).trimmed();
    }

    m_catTrSettings = retranslateMyDefaultCat(m_catSettings);

    int row = 0;

    foreach(QString cat, m_sets->value(m_boxName + KEY_CATEGORIES).toString().split(",")){

        QStringList lst;
        lst = cat.split("|");

        QString c,d;

        switch (lst.count()){
        case 2:
            d = lst.at(DESCRIPTION_COLUMN).trimmed();           
        case 1:            
            //      
            c = m_catTrSettings.at(row);
        default:;
        }

        myCatTable->setItem(row,CATEGORY_COLUMN, new QTableWidgetItem(c));
        myCatTable->setItem(row,DESCRIPTION_COLUMN, new QTableWidgetItem(d));

        row++;
    }

    QSize sz;
    sz = myCatTable->minimumSize();
    sz.setHeight(myCatTable->verticalHeader()->sectionSize(0) * 5 + myCatTable->horizontalHeader()->size().height());
    myCatTable->setMinimumSize(sz);
    myCatTable->verticalHeader()->setSectionResizeMode(QHeaderView::Fixed);

    for(int r = 0; r < myCatTable->rowCount(); r++){
        myCatTable->item(r, CATEGORY_COLUMN)->setFlags(Qt::ItemIsEnabled);
        myCatTable->item(r, DESCRIPTION_COLUMN)->setFlags(Qt::ItemIsEnabled);
    }

    connect(myCatTable,SIGNAL(itemChanged(QTableWidgetItem*)),this,SLOT(myCatTableChange(QTableWidgetItem*)));

    extLineEdit->installEventFilter(this);
    myCatTable->installEventFilter(this);

    retranslateUi(this);
}

void MySetTab::pressExtAddButton()
{
    QString e;
    e = extLineEdit->text().trimmed();
    extList->addItem(new QListWidgetItem(e));
    extLineEdit->clear();

    setModeExtDefaultButton();
    extLineEdit->setFocus(Qt::TabFocusReason);
}

void MySetTab::setModeExtAddButton(QString s)
{
    if (!s.isEmpty() && extList->findItems(s, Qt::MatchFixedString).isEmpty()){
        extAddButton->setEnabled(true);
    } else {
        extAddButton->setEnabled(false);
    }
    setModeExtClearButton();
}

void MySetTab::pressExtDeleteButton()
{
    QListWidgetItem *item;
    item = extList->selectedItems().first();

    delete extList->takeItem(extList->row(item));

    if (extList->count() == 0) {
        extLineEdit->setFocus(Qt::TabFocusReason);
    }

    setModeExtClearButton();
    setModeExtDefaultButton();
}

void MySetTab::setModeExtDeleteButton()
{
    if (extList->selectedItems().isEmpty()){
        extDeleteButton->setEnabled(false);
    } else {
        extDeleteButton->setEnabled(true);
    }
}

void MySetTab::pressExtClearButton()
{
    extList->clear();

    setModeExtClearButton();
    setModeExtDefaultButton();

    extLineEdit->setFocus(Qt::TabFocusReason);
}

void MySetTab::setModeExtClearButton()
{
    if (extList->count() == 0){
        extClearButton->setEnabled(false);
        extList->setEnabled(false);
    } else {
        extClearButton->setEnabled(true);
        extList->setEnabled(true);
    }
}

void MySetTab::pressExtDefaultButton()
{
    extList->clear();

    QStringList exts;
    exts = m_sets->defaultValue(m_boxName + KEY_EXT).toString().split(",");
    foreach(QString e, exts){
        extList->addItem(new QListWidgetItem(e.trimmed()));
    }

    setModeExtClearButton();
    setModeExtDefaultButton();
    extLineEdit->setFocus(Qt::TabFocusReason);
}

void MySetTab::setModeExtDefaultButton()
{
    QStringList la;
    la = m_sets->defaultValue(m_boxName + KEY_EXT).toString().remove(QChar(' ')).split(QChar(','));
    la.sort();

    QStringList lb;
    for(int r = 0; r < extList->count(); r++){
        lb.append(extList->item(r)->text());
    }
    lb.sort();

    QString a = la.join(",");
    QString b = lb.join(",");

    if (a == b) {
        extDefaultButton->setEnabled(false);
    } else {
        extDefaultButton->setEnabled(true);
    }
}

void MySetTab::pressPreviewAppDefaultButton()
{
    previewAppLineEdit->setText(m_sets->defaultValue(m_boxName + KEY_PREVIEW_APP).toString());
}

void MySetTab::pressPresentAppDefaultButton()
{
    presentAppLineEdit->setText(m_sets->defaultValue(m_boxName + KEY_PRESENT_APP).toString());
}

void MySetTab::setModePreviewAppDefaultButton(QString s)
{
    if (s.simplified() == m_sets->defaultValue(m_boxName + KEY_PREVIEW_APP).toString()){
        previewAppDefaultButton->setEnabled(false);
    } else {
        previewAppDefaultButton->setEnabled(true);
    }
}

void MySetTab::setModePresentAppDefaultButton(QString s)
{
    if (s.simplified() == m_sets->defaultValue(m_boxName + KEY_PRESENT_APP).toString()){
        presentAppDefaultButton->setEnabled(false);
    } else {
        presentAppDefaultButton->setEnabled(true);
    }
}

void MySetTab::setPreviewAppStatus(QString s)
{
    QString cmd;
    cmd = s.trimmed();

    QRegExp exp("^\".+\"");
    int pos = exp.indexIn(cmd);
    if (pos > -1){
        cmd = exp.cap();
        cmd.remove("\"");
        cmd = cmd.trimmed();
    } else {
        cmd = cmd.split(" ").first();
    }

    QString st;
    if (AltQFile::exists(cmd)){
        st = tr("Ok");
    } else {
        st = tr("Absent");
    }
    previewAppStatus->setText(st);
}

void MySetTab::setPresentAppStatus(QString s)
{
    QString cmd;
    cmd = s.trimmed();

    QRegExp exp("^\".+\"");
    int pos = exp.indexIn(cmd);
    if (pos > -1){
        cmd = exp.cap();
        cmd.remove("\"");
        cmd = cmd.trimmed();
    } else {
        cmd = cmd.split(" ").first();
    }

    QString st;
    if (AltQFile::exists(cmd)){
        st = tr("Ok");
    } else {
        st = tr("Absent");
    }
    presentAppStatus->setText(st);
}

void MySetTab::pressMyCatEditButton()
{
    disconnect(myCatTable,SIGNAL(itemChanged(QTableWidgetItem*)),this,SLOT(myCatTableChange(QTableWidgetItem*)));
    if (m_myCatEditMode) {
        for(int r = 0; r < myCatTable->rowCount(); r++){
            myCatTable->item(r, CATEGORY_COLUMN)->setFlags(Qt::ItemIsEnabled);
            myCatTable->item(r, DESCRIPTION_COLUMN)->setFlags(Qt::ItemIsEnabled);
        }
        myCatEditStatus->setText(tr("disable"));
        myCatDefaultButton->setEnabled(false);
        m_myCatEditMode = false;
    } else {
        for(int r = 0; r < myCatTable->rowCount(); r++){
            myCatTable->item(r, CATEGORY_COLUMN)->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
            myCatTable->item(r, DESCRIPTION_COLUMN)->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
        }
        myCatEditStatus->setText(tr("enable"));

        QStringList a;
        for(int r = 0; r < myCatTable->rowCount(); r++){
            a.append(myCatTable->item(r,CATEGORY_COLUMN)->text().trimmed());
        }

        if (a.join(",") == m_defCatLstTr.join(",")) {
            myCatDefaultButton->setEnabled(false);
        } else {
            myCatDefaultButton->setEnabled(true);
        }

        m_myCatEditMode = true;
    }
    connect(myCatTable,SIGNAL(itemChanged(QTableWidgetItem*)),this,SLOT(myCatTableChange(QTableWidgetItem*)));
}

void MySetTab::pressMyCatDefaultButton()
{
    disconnect(myCatTable,SIGNAL(itemChanged(QTableWidgetItem*)),this,SLOT(myCatTableChange(QTableWidgetItem*)));

    int row = 0;

    m_defCatLstTr = retranslateMyDefaultCat(m_defCatLst);

    foreach(QString cat, m_sets->defaultValue(m_boxName + KEY_CATEGORIES).toString().split(",")){

        QStringList lst;
        lst = cat.split("|");

        QString c,d;

        switch (lst.count()){
        case 2:
            d = lst.at(DESCRIPTION_COLUMN).trimmed();
        case 1:            
            c = m_defCatLstTr.at(row);
        default:;
        }

        myCatTable->item(row,CATEGORY_COLUMN)->setText(c);
        myCatTable->item(row,DESCRIPTION_COLUMN)->setText(d);

        row++;
    }

    connect(myCatTable,SIGNAL(itemChanged(QTableWidgetItem*)),this,SLOT(myCatTableChange(QTableWidgetItem*)));

    myCatDefaultButton->setEnabled(false);
}

void MySetTab::setModeMyCatDefaultButton()
{        
    QStringList a;
    for(int r = 0; r < myCatTable->rowCount(); r++){
        a.append(myCatTable->item(r,CATEGORY_COLUMN)->text().trimmed());
    }

    if (a.join(",") == m_defCatLstTr.join(",")) {
        myCatDefaultButton->setEnabled(false);
    } else {
        myCatDefaultButton->setEnabled(true);
    }
}

void MySetTab::myCatTableChange(QTableWidgetItem* item)
{    
    disconnect(myCatTable,SIGNAL(itemChanged(QTableWidgetItem*)),this,SLOT(myCatTableChange(QTableWidgetItem*)));

    QString s;
    s = item->text().trimmed();
    if (s.isEmpty() && (item->column() == CATEGORY_COLUMN)) {
        s = m_defCatLstTr.at(item->row());
    }
    item->setText(s);
    connect(myCatTable,SIGNAL(itemChanged(QTableWidgetItem*)),this,SLOT(myCatTableChange(QTableWidgetItem*)));

    setModeMyCatDefaultButton();
}

bool MySetTab::eventFilter(QObject *target, QEvent *event)
{
    if ((target == extLineEdit) || (target == myCatTable) ){
        if (event->type() == QEvent::KeyPress){
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
            if (target == extLineEdit){
                switch(keyEvent->key()){
                case Qt::Key_Enter:
                case Qt::Key_Return:
                    if (extAddButton->isEnabled()){
                        pressExtAddButton();
                    }
                    return true;
                default:;
                }
            }
            if (target == myCatTable){
                switch(keyEvent->key()){
                case Qt::Key_Backtab:
                    focusPreviousChild();
                    return true;
                case Qt::Key_Tab:
                    focusNextChild();
                    return true;
                default:;
                }
            }
        }
    }
    return QWidget::eventFilter(target, event);
}

void MySetTab::saveSettings()
{
    QStringList exts;

    for(int r = 0; r < extList->count(); r++){
        QListWidgetItem *i = extList->item(r);
        exts << i->text();
    }

    m_sets->setValue(m_boxName + KEY_EXT, exts.join(","));

    m_sets->setValue(m_boxName + KEY_PREVIEW_APP, previewAppLineEdit->text().simplified());
    m_sets->setValue(m_boxName + KEY_PRESENT_APP, presentAppLineEdit->text().simplified());

    QStringList lst;

    for(int r = 0; r < myCatTable->rowCount(); r++){
        QTableWidgetItem* c = myCatTable->item(r, CATEGORY_COLUMN);
        QTableWidgetItem* d = myCatTable->item(r, DESCRIPTION_COLUMN);

        QString cn;
        cn = c->text().simplified();
        QString str;
        if (cn == m_defCatLstTr.at(r)){             //     , 
            str.append(m_defCatLst.at(r));          //        ()
        } else {
            if (cn == m_catTrSettings.at(r)){       //     ,
                str.append(m_catSettings.at(r));    //   ,     , ..   
            } else {                                // ,      
                str.append(cn);
            }
        }

        if (!d->text().isEmpty())
        {
            str.append("|" + d->text().simplified());
        }

        lst << str;
    }

    m_sets->setValue(m_boxName + KEY_CATEGORIES, lst.join(","));
}

QStringList MySetTab::retranslateMyDefaultCat(QStringList lst)
{
    //       
    QString str = lst.join(",");
    str.replace("chronicle", tr("chronicle"));
    str.replace("informative", tr("informative"));
    str.replace("event", tr("event"));
    str.replace("situation", tr("situation"));
    str.replace("moment", tr("moment"));
    str.replace("detail", tr("detail"));
    str.replace("pictorial", tr("pictorial"));
    str.replace("composition", tr("composition"));
    str.replace("series", tr("series"));
    str.replace("essay", tr("essay"));
    str.replace("blank", tr("blank"));
    str.replace("belles", tr("belles"));
    str.replace("information", tr("information"));
    str.replace("popular_science", tr("popular_science"));
    str.replace("periodical", tr("periodical"));
    str.replace("educational", tr("educational"));
    str.replace("technical", tr("technical"));
    str.replace("folklore", tr("folklore"));
    str.replace("fantasy", tr("fantasy"));
    str.replace("detective", tr("detective"));
    str.replace("adventure", tr("adventure"));
    str.replace("creation", tr("creation"));
    str.replace("art", tr("art"));
    str.replace("home", tr("home"));
    str.replace("music", tr("music"));
    str.replace("classic", tr("classic"));
    str.replace("folk", tr("folk"));
    str.replace("pop", tr("pop"));
    str.replace("baby", tr("baby"));
    str.replace("blues", tr("blues"));
    str.replace("jazz", tr("jazz"));
    str.replace("rock", tr("rock"));
    str.replace("electronic", tr("electronic"));
    str.replace("retro", tr("retro"));
    str.replace("dancing", tr("dancing"));
    str.replace("empty_A", tr("empty_A"));
    str.replace("empty_B", tr("empty_B"));
    str.replace("empty_C", tr("empty_C"));
    str.replace("empty_D", tr("empty_D"));
    str.replace("empty_E", tr("empty_E"));
    str.replace("empty_F", tr("empty_F"));
    str.replace("empty_G", tr("empty_G"));
    str.replace("empty_H", tr("empty_H"));
    str.replace("empty_I", tr("empty_I"));
    str.replace("empty_J", tr("empty_J"));
    str.replace("empty_K", tr("empty_K"));
    str.replace("empty_L", tr("empty_L"));
    str.replace("empty_M", tr("empty_M"));
    str.replace("empty_N", tr("empty_N"));
    //    ,   
    return str.split(",");
}
