#include "mysourcefile.h"
#include "basefolder.h"

#include <QFileInfo>

MySourceFile::MySourceFile(const QString &name, MyUnit* parent) : MyUnit(name, parent)
{
    m_initOk = true;

    QFile file(completeName());
    QFileInfo fileInfo(file);

    setObjectName(fileInfo.completeBaseName());

    QCryptographicHash md5(QCryptographicHash::Md5);

    if (file.open(QIODevice::ReadOnly)) {
        md5.reset();
        md5.addData(file.readAll());
        file.close();
        m_checkSum = md5.result().toHex();
    } else {
        m_checkSum = "";
    }

    m_exifDate = fileInfo.lastModified().date();
    m_exifTime = fileInfo.lastModified().time();
    m_ext = fileInfo.suffix();
}

QString MySourceFile::checkSum()
{
    return m_checkSum;
}

QDate MySourceFile::exifDate()
{
    return m_exifDate;
}

QTime MySourceFile::exifTime()
{
    return m_exifTime;
}

QString MySourceFile::ext()
{
    return m_ext;
}

void MySourceFile::setExifDate(const QDate &date)
{
    m_exifDate = date;
}

void MySourceFile::setExifTime(const QTime &time)
{
    m_exifTime = time;
}

