#include "mysplash.h"
#include <QDesktopWidget>
#include <QTimer>

MySplash::MySplash(const QString &img, Qt::Alignment alignment, const QColor &color):
    QWidget()
{
    setWindowFlags(Qt::SplashScreen);
    setAttribute(Qt::WA_DeleteOnClose);

    m_displayTime=700;

    m_pixmap = new QPixmap(img);

    QDesktopWidget scr;
    QWidget *wScr;

    wScr = scr.screen();

    int pixmapW = m_pixmap->width();
    int pixmapH = m_pixmap->height();

    setGeometry((wScr->width() - pixmapW)/2,(wScr->height() - pixmapH)/2,pixmapW,pixmapH);

    m_background = new QLabel(this);
    m_background->setPixmap(*m_pixmap);

    m_palette.setColor(QPalette::WindowText,color);

    m_message = new QLabel(this);
    m_message->setGeometry(5,5,pixmapW - 10,pixmapH - 10);
    m_message->setAlignment(alignment);
    m_message->setPalette(m_palette);
}

void MySplash::activateUserAction()
{
    connect(this, SIGNAL(workComplete()), this, SLOT(close()));
}

void MySplash::keyPressEvent(QKeyEvent *event)
{
    emit workComplete();
}

void MySplash::mousePressEvent(QMouseEvent *event)
{
    emit workComplete();
}

void MySplash::setDisplayMessageTime(int t)
{
    m_displayTime = t;
}

void MySplash::showMessage(const QString &message)
{
    m_message->setText(message);
    activateWindow();
    QTimer::singleShot(m_displayTime, &loop, SLOT(quit()));
    loop.exec();
}
