#include "mysrcfolder.h"

#include <QSet>
#include <QMap>
#include <QDir>
#include <QFileInfo>

MySrcFolder::MySrcFolder(QString extractApp, const QString &name, MyUnit* parent) : MyFolder(name, parent)
{
    m_sourceDir = completeName();

    if (extractApp.trimmed().endsWith("%1")){
        m_extractApp = extractApp.arg(m_sourceDir);
    } else {
        m_extractApp = extractApp + " " + m_sourceDir;
    }

    addFiles();

    process = new QProcess();
    connect (process,SIGNAL(finished(int)),SLOT(setDateAndTime()));    
    process->start(m_extractApp);
    process->waitForFinished(-1);
}

MySrcFolder::~MySrcFolder()
{
    delete process;
}

void MySrcFolder::setDateAndTime()
{
    QString allExifDate = process->readAllStandardOutput();
    QStringList lstExifDate;
    lstExifDate = allExifDate.split("\n");

    QStringList tmpLst;
    QMap<QString, QString> date;
    QMap<QString, QString> time;

    foreach (QString tmp,lstExifDate){

        tmpLst = (tmp.trimmed()).split("|");

        if (tmpLst.count() != 3) {
            continue;
        }

        date[tmpLst.at(0)] = tmpLst[1];
        time[tmpLst.at(0)] = tmpLst[2];
    }

    foreach(MySourceFile* tmp, this->findChildren<MySourceFile*>()){

        QString filename = tmp->name();

        if (!date.contains(filename)){
            continue;
        }

        QDate d;
        QTime t;

        d = QDate::fromString(date.value(filename),"dd.MM.yyyy");
        t = QTime::fromString(time.value(filename),"hh:mm");

        if (d.isValid() && t.isValid()) {
            tmp->setExifDate(d);
            tmp->setExifTime(t);
        }
    }
}

QList<MySourceFile*> MySrcFolder::getInConformityExt(const QSet<QString> &exts)
{
    QList<MySourceFile*> lst;  

    foreach(MySourceFile* tmp, this->findChildren<MySourceFile*>()){       
       if (exts.contains(tmp->ext().toLower())) {
            lst.append(tmp);
        }
    }
    return lst;
}

void MySrcFolder::refresh()
{
    foreach(MySourceFile* tmp, this->findChildren<MySourceFile*>()){
        delete tmp;
    }

    addFiles();

    process->start(m_extractApp);
    process->waitForFinished(-1);
}

void MySrcFolder::setExtractApp(QString app)
{
    if (app.trimmed().endsWith("%1")){
        m_extractApp = app.arg(m_sourceDir);
    } else {
        m_extractApp = app + " " + m_sourceDir;
    }    
}

void MySrcFolder::addFiles(){

    QDir dir(this->completeName());

    foreach(QFileInfo tmp, dir.entryInfoList(QDir::Files)){
        QString filename = tmp.fileName();
        new MySourceFile(filename, this);
    }
}
