#include "mysssfile.h"

#include <QTextStream>
#include <QStringList>
#include <QFile>
#include <QDateTime>

MySssFile::MySssFile(const QString &name, MyUnit* parent) : MyFile(name, parent)
{     
    QFile f(this->completeName());

    QTextStream st(&f);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    int max = 0;
    int count = 0;

    if (initOk()) {
        if (f.open(QIODevice::ReadOnly | QIODevice::Text)) {
            while (!st.atEnd()){
                QStringList parts;
                parts = st.readLine().split("~");                
                if (parts.count() == 4){
                    for (int i = 0; i < 4; i++){
                        parts[i] = parts[i].trimmed();
                    }
                    int x = parts[NO].toInt();
                    if (x && !parts[CREATION_DATE].isEmpty() && !parts[TYPE].isEmpty()) {
                        m_sessionDateCreation.insert(x, parts[CREATION_DATE]);                        
                        if (parts[ANNOTATION].indexOf("unknown",0,Qt::CaseInsensitive) > -1){
                            m_sessionAnnatation.insert(x, QString());
                        } else {
                            m_sessionAnnatation.insert(x, parts[ANNOTATION]);
                        }
                        count++;
                        if (x > max) { max = x; }
                    }
                }
            }
            f.close();
        }
    }

    m_currentSessionNumber = max+1;
    m_countSessions = count;
}

bool MySssFile::add(const QString &boxName, QString ann)
{    
    ann = ann.simplified();

    QString mAnn;
    if (ann.isEmpty()){
        mAnn = "unknown";
    } else {
        mAnn = ann;
    }

    QFile f(this->completeName());

    QTextStream st(&f);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    bool result = false;
    if (initOk()) {
        QString line;
        QString dt;
        dt = QDateTime::currentDateTime().toString("dd.MM.yyyy hh:mm:ss");
        line = QString::number(m_currentSessionNumber) + "~"  + dt + "~" + boxName + "~" + mAnn;
        if (f.open(QIODevice::Append | QIODevice::Text)) {
            result = true;
            st << line << endl;           

            m_sessionDateCreation.insert(m_currentSessionNumber, dt);
            m_sessionAnnatation.insert(m_currentSessionNumber, ann);

            m_currentSessionNumber++;
            m_countSessions++;

            f.close();            
        }
    }
    return result;
}

QString MySssFile::currentSessionNumber()
{       
    return QString::number(m_currentSessionNumber);
}

QString MySssFile::countSessions()
{
    return QString::number(m_countSessions);
}

QString MySssFile::sessionDateCreation(uint s)
{
    return m_sessionDateCreation.value(s);
}

QString MySssFile::sessionAnnotation(uint s)
{
    return m_sessionAnnatation.value(s);
}
