#include "mytmpfile.h"
#include "remark.h"

#include <QFile>
#include <QDate>
#include <QStringList>
#include <QTextStream>

MyTmpFile::MyTmpFile(const QString &name, MyUnit* parent) : MyFile(name, parent)
{    
    wrd = static_cast<MyWrdFile*>(unit(NAME_WRD_FILE));

    m_describe.clear();    

    symbolSession   = "A:";
    symbolRemark    = "B:";
    symbolPrint     = "C:";
    symbolCategory  = "D:";
    symbolKeyword   = "E:";
    symbolTitle     = "F:";
    symbolDate      = "G:";

    symbolSearch    = ":search";
    symbolAddTask   = ":plus";    
}

void MyTmpFile::loadTemplates(QString boxName)
{
    QFile f(completeName());

    QTextStream st(&f);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    QString l;
    tasks data;
    task a;

    QString taskName;
    taskName = "";

    m_templates.clear();
    m_describe.clear();
    m_own.clear();

    if (initOk()) {
        if (f.open(QIODevice::ReadOnly | QIODevice::Text)) {
            while (!st.atEnd()){
                l = st.readLine().simplified();
                if (l.isEmpty()) { continue; }
                if (!taskName.isEmpty()){
                    if (l.indexOf(symbolSession) == 0){
                        a.session = l.mid(symbolSession.length());
                        continue;
                    }
                    if (l.indexOf(symbolRemark) == 0){
                        a.remark = l.mid(symbolRemark.length());
                        continue;
                    }
                    if (l.indexOf(symbolPrint) == 0){
                        a.print = l.mid(symbolPrint.length());
                        continue;
                    }
                    if (l.indexOf(symbolCategory) == 0){
                        a.category = l.mid(symbolCategory.length());
                        continue;
                    }
                    if (l.indexOf(symbolKeyword) == 0){
                       a.keyword = l.mid(symbolKeyword.length());
                       continue;
                    }
                    if (l.indexOf(symbolTitle) == 0){
                        a.title = l.mid(symbolTitle.length());
                        continue;
                    }
                    if (l.indexOf(symbolDate) == 0){
                        a.date = l.mid(symbolDate.length());
                        continue;
                    }
                    if (l.indexOf(symbolAddTask) == 0){
                        a.addTask = true;
                        continue;
                    }
                    if (l.indexOf(symbolSearch) == 0){
                        data.append(a);
                        a.session.clear();
                        a.remark.clear();
                        a.print.clear();
                        a.category.clear();
                        a.keyword.clear();
                        a.title.clear();
                        a.date.clear();
                        a.addTask = false;
                        continue;
                    }
                    if ((l.left(1) == "<") && (l.right(1) == ">")){
                        m_templates[taskName] = data;
                        taskName.clear();
                    }
                }
                if (taskName.isEmpty() && (l.left(1) == "<") && (l.right(1) == ">")){
                    QStringList s;
                    s = l.mid(1,l.length() - 2).split(":");

                    if ((s.count() == 3) && (s.at(0).trimmed() == boxName)) {

                        s[NAME] = s[NAME].trimmed();
                        s[DESCRIBE] = s[DESCRIBE].trimmed();
                        s[OWN] = s[OWN].trimmed();

                        if (s[DESCRIBE].isEmpty()) {
                            s[DESCRIBE] = "---";
                        }

                        if (!s[NAME].isEmpty()){
                            taskName = s[NAME];
                            m_describe[taskName] = s[DESCRIBE];
                            m_own[taskName] = s[OWN];
                            a.session.clear();
                            a.remark.clear();
                            a.print.clear();
                            a.category.clear();
                            a.keyword.clear();
                            a.title.clear();
                            a.date.clear();
                            a.addTask = false;
                            data.clear();
                        }
                    }
                }
            }
            if (!taskName.isEmpty()) {
                m_templates[taskName] = data;
            }
            f.close();
        }
    }
}

QMap<QString,QString>* MyTmpFile::describes()
{    
    return &m_describe;
}

QMap<QString, MyTmpFile::tasks>* MyTmpFile::templates()
{
    return &m_templates;
}

MyTmpFile::tasks MyTmpFile::getTask(QString taskName)
{
    return m_templates.value(taskName);
}

void MyTmpFile::addTask(QString own, QString taskName, QString taskDescribe, MyTmpFile::tasks data)
{
    m_own[taskName] = own;
    m_templates[taskName] = data;
    m_describe[taskName] = taskDescribe;

    QFile f(completeName());

    QTextStream st(&f);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    if (initOk()) {
        if (f.open(QIODevice::Append | QIODevice::Text)) {            
            st << "<" << own << ":" << taskName << ":" << taskDescribe << ">" << endl;
            foreach(task t, data) {
                if (t.addTask) {
                    st << "\t" << symbolAddTask << endl;
                }
                st << "\t\t" << symbolSession << t.session << endl;
                st << "\t\t" << symbolRemark << t.remark << endl;
                st << "\t\t" << symbolPrint << t.print << endl;
                st << "\t\t" << symbolCategory << t.category << endl;
                st << "\t\t" << symbolKeyword << t.keyword << endl;
                st << "\t\t" << symbolTitle << t.title << endl;
                st << "\t\t" << symbolDate << t.date << endl;
                st << "\t" << symbolSearch << endl;
            }
            st << endl;
            f.close();
        }
    }
}

QString MyTmpFile::viewTask(QString taskName, bool* ok)
{
    QString result;
    int s;

    AnyBox* anybox;
    anybox = static_cast<AnyBox*>(this->unit(m_own.value(taskName)));

    s = 1;

    result.clear();
    result.append("<table width = 100%>");
    result.append("<tr> <td width = 20%> <b>" + tr("Name") + "</b> </td> <td>" + taskName + "</td> </tr> ");
    result.append("<tr> <td> <b>" + tr("Description") + "</b> </td> <td>" + m_describe.value(taskName) + "</td> </tr> ");
    result.append("</table>");

    QString stageView;

    stageView = "<tr> <td width = 20%> <b> %1 </b> </td> <td> %2 </td> </tr>";

    bool f;
    f = true;

    foreach(task t, getTask(taskName)){

        bool test;
        test = checkTask(anybox, t);

        *ok = *ok && test;

        QString ss;

        if (f) {
            result.append("<table width = 100%>");
            result.append("<tr> <td align = right> <b>" + tr("Stage") + " " + ss.setNum(s) + "</b> </td> </tr>");
            result.append("</table>");
            result.append("<hr />");
            f = false;
        } else {
            if (t.addTask) {
                s++;
                result.append("<table width = 100%>");
                result.append("<tr> <td align = right> <b>" + tr("Stage") + " " + ss.setNum(s) + "</b> </td> </tr>");
                result.append("</table>");
                result.append("<hr />");
            } else {
                result.append("<table width = 100%>");
                result.append("<tr> <td width = 10% align = right>" + tr("specify") + "... </td> <td> </td> </tr>");
                result.append("</table>");
            }
        }

        result.append("<table width = 100%>");
        if (!t.remark.isEmpty()){
            result.append(stageView.arg(QString(tr("Remark")),checkedRemark));
        }
        if (!t.category.isEmpty()){
            result.append(stageView.arg(QString(tr("Category")),checkedCategory));
        }
        if (!t.keyword.isEmpty()){
            result.append(stageView.arg(QString(tr("Keyword")),checkedKeyword));
        }
        if (!t.date.isEmpty()){
            result.append(stageView.arg(QString(tr("Date")),checkedDate));
        }
        if (!t.session.isEmpty()){
            result.append(stageView.arg(QString(tr("Session")),checkedSession));
        }
        if (!t.print.isEmpty()){
            result.append(stageView.arg(QString(tr("Hardcopy")),checkedPrint));
        }
        if (!t.title.isEmpty()){
            result.append(stageView.arg(QString(tr("Caption")),checkedTitle));
        }
        result.append("</table>");

    }    
    return result;
}

int MyTmpFile::templatesCount()
{
    return m_templates.count();
}

int MyTmpFile::templatesCount(QString name)
{
    int c;
    c = 0;

    foreach(QString tmp, m_own.values()){
        if (tmp == name) {
            c++;
        }
    }

    return c;
}

bool MyTmpFile::checkTask(AnyBox* anybox, task t)
{
    bool result;
    result = true;

    int l;
    l = SIGN_INV.length();
    bool f;

    //Remark
    QString r;    
    r = t.remark.replace(" ","");
    checkedRemark = r;
    f = false;
    if (r.left(l) == SIGN_INV) {
        r.remove(0,l);
        f = true;
    }        
    QStringList remarkParts;
    remarkParts = r.split(",");
    for (int i = 0; i < remarkParts.count(); i++){
        if (remarkParts[i].isEmpty()) { continue; }
        Remark a(remarkParts[i]);
        if (!a.Ok()){
            remarkParts[i] = QString("<s>") + remarkParts[i] + QString("</s>");
            result = false;
        }
    }
    checkedRemark = remarkParts.join(", ");
    if (f) {
        checkedRemark.prepend(SIGN_INV + " ");
    }

    //Category
    QString c;
    c = t.category.replace(" ","");
    f = false;
    if (c.left(l) == SIGN_INV) {
        c.remove(0,l);
        f = true;
    }
    QStringList categoryParts;
    categoryParts = c.split(",");

    int add;
    add = 6 - categoryParts.count();
    while (add > 0){
        categoryParts.append("");
        add--;
    }

    QStringList expStr;
    Category cat;
    cat = anybox->category();

    expStr = cat.listSubCategoryValues_A();  

    expStr << cat.listSubCategoryValues_B();

    QRegExp exp(expStr.join("|"));
    foreach (QString w, categoryParts[0].split(QRegExp("\\+|\\-"))){
        if (w.isEmpty()) { continue; }
        if (!exp.exactMatch(w)){
            categoryParts[0].replace(w,QString("<s>") + w + QString("</s>"));
            result = false;
        }
    }

    expStr = cat.listSubCategoryValues_C();

    exp.setPattern(expStr.join("|"));

    foreach (QString w, categoryParts[1].split("|")){
        if (w.isEmpty()) { continue; }
        if (!exp.exactMatch(w)){
            categoryParts[1].replace(w,QString("<s>") + w + QString("</s>"));
            result = false;
        }
    }

    expStr = cat.listSubCategoryValues_D();

    exp.setPattern(expStr.join("|"));

    foreach (QString w, categoryParts[2].split("|")){
        if (w.isEmpty()) { continue; }
        if (!exp.exactMatch(w)){
            categoryParts[2].replace(w,QString("<s>") + w + QString("</s>"));
            result = false;
        }
    }

    expStr = cat.listSubCategoryValues_E();

    exp.setPattern(expStr.join("|"));

    foreach (QString w, categoryParts[3].split("|")){
        if (w.isEmpty()) { continue; }
        if (!exp.exactMatch(w)){
            categoryParts[3].replace(w,QString("<s>") + w + QString("</s>"));
            result = false;
        }
    }

    expStr = cat.listSubCategoryValues_F();

    exp.setPattern(expStr.join("|"));

    foreach (QString w, categoryParts[4].split("|")){
        if (w.isEmpty()) { continue; }
        if (!exp.exactMatch(w)){
            categoryParts[4].replace(w,QString("<s>") + w + QString("</s>"));
            result = false;
        }
    }

    expStr = cat.listSubCategoryValues_G();

    exp.setPattern(expStr.join("|"));

    foreach (QString w, categoryParts[5].split("|")){
        if (w.isEmpty()) { continue; }
        if (!exp.exactMatch(w)){
            categoryParts[5].replace(w,QString("<s>") + w + QString("</s>"));
            result = false;
        }
    }
    checkedCategory = categoryParts.join(", ");
    if (f) {
        checkedCategory.prepend(SIGN_INV + " ");
    }

    //Keyword    
    QString k;
    k = t.keyword.simplified();
    f = false;
    if (k.left(l) == SIGN_INV) {
        k.remove(0,l);
        f = true;
    }
    QStringList keywordParts;
    keywordParts = k.split(",");

    for (int i = 0; i < keywordParts.count(); i++){
        if (keywordParts[i].isEmpty()) { continue; }
        QStringList keywordPartsSecond;
        keywordPartsSecond = keywordParts[i].split(QRegExp("\\+|\\-"));
        for (int j = 0; j < keywordPartsSecond.count(); j++){
            keywordPartsSecond[j] = keywordPartsSecond[j].trimmed();
            if (keywordPartsSecond[j].isEmpty()) { continue; }
            if (!wrd->wordExist(keywordPartsSecond[j])){
                keywordParts[i].replace(keywordPartsSecond[j],QString("<s>") + keywordPartsSecond[j] + QString("</s>"));
                result = false;
            }
        }
    }
    checkedKeyword = keywordParts.join(", ");
    if (f) {
        checkedKeyword.prepend(SIGN_INV + " ");
    }

    //Date
    QString d;
    d = t.date.replace(" ","");
    f = false;
    if (d.left(l) == SIGN_INV) {
        d.remove(0,l);
        f = true;
    }
    QDate a,b;
    QStringList q;
    QStringList dateParts;
    dateParts = d.split(",");

    for (int i = 0; i < dateParts.count(); i++){
        if (dateParts[i].isEmpty()) { continue; }
        q = dateParts[i].split("-");
        switch (q.count()){
        case 1:            
            a = QDate::fromString(q[0],anybox->dateFormat());
            if (!a.isValid()){
                dateParts[i] = QString("<s>") + dateParts[i] + QString("</s>");
                result = false;
            }
            break;
        case 2:            
            a = QDate::fromString(q[0],anybox->dateFormat());
            b = QDate::fromString(q[1],anybox->dateFormat());
            if ( (a.isValid() && b.isValid() && (a <= b)) ||
                 ((q[0] == "...") && (b.isValid())) ||
                 (a.isValid() && (q[1] == "..."))){
            } else {
                dateParts[i] = QString("<s>") + dateParts[i] + QString("</s>");
                result = false;
            }
            break;
        default:;
        }
    }
    checkedDate = dateParts.join(", ");
    if (f) {
        checkedDate.prepend(SIGN_INV + " ");
    }

    //Session
    QString s;
    s = t.session.replace(" ","");
    f = false;
    if (s.left(l) == SIGN_INV) {
        s.remove(0,l);
        f = true;
    }
    QStringList sessionParts;
    sessionParts = s.split(",");
    for (int i = 0; i < sessionParts.count(); i++){
        if (sessionParts[i].isEmpty()) { continue; }
        bool ok;
        uint z;
        z = sessionParts[i].toUInt(&ok);
        if (!ok || (z == 0)){
            sessionParts[i] = QString("<s>") + sessionParts[i] + QString("</s>");
            result = false;
        }
    }
    checkedSession = sessionParts.join(", ");
    if (f) {
        checkedSession.prepend(SIGN_INV + " ");
    }

    //Print
    QString p;
    p = t.print.replace(" ","");
    f = false;
    if (p.left(l) == SIGN_INV) {
        p.remove(0,l);
        f = true;
    }
    QStringList printParts;
    printParts = p.split(",");
    for (int i = 0; i < printParts.count(); i++){
        if (printParts[i].isEmpty()) { continue; }
        bool ok;
        printParts[i].toUInt(&ok);
        if (!ok){
            printParts[i] = QString("<s>") + printParts[i] + QString("</s>");
            result = false;
        }
    }
    checkedPrint = printParts.join(", ");
    if (f) {
        checkedPrint.prepend(SIGN_INV + " ");
    }

    //Title & author
    checkedTitle = t.title.trimmed();

    return result;
}

QString MyTmpFile::own(QString name)
{
    return m_own.value(name);
}
