#ifndef MYUNIT_H
#define MYUNIT_H

#include <QObject>

/*
  Элемент
  Элемент содержит имя элемента (m_name), путь к элементу (m_path) и тип элемента (m_type)
  Элементы объединяются в древовидную структуру (при создании элемента указывается элемент-предок)
  Возможность использования элемента контролируется методом initOk()
  Элемент, не имеющий предка называется основным (baseUnit). Он может быть получен методом baseUnit()
  Элемент (из дерева) с именем 'name' может быть найден методом unit(...).
  Метод completeName() возвращает полное имя элемента (содержит путь и имя элемента, соединенные
  разделителем '\' или '/' в зависимости от ОС)

  baseUnit
    |
    |--unit_A
    |--unit_B
    ...
    |--unit_X
    |    |--unit_a
    |    |--unit_b
    ...  ...
    |--unit_Z

*/

class MyUnit : public QObject
{

protected:    
    QString m_path;
    QString m_name;
    QString m_type;
    bool m_initOk; 

public:
    MyUnit(const QString &name, MyUnit *parent = 0);

    QString type();
    bool initOk();
    QString path();
    QString name();
    QString completeName();

    MyUnit* unit(const QString&);
    MyUnit* baseUnit();

};

#endif // MYUNIT_H
