#include "mywrdfile.h"
#include <QFile>
#include <QTextStream>
#include <QSet>
#include <stdlib.h>

MyWrdFile::MyWrdFile(const QString &name, MyUnit* parent) : MyFile(name, parent)
{
    needUpdate = false;    

    QFile f(this->completeName());

    QTextStream st(&f);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    QString l;
    QStringList w;

    QRegExp exp(ABC_WRD);

    if (initOk()) {
        if (f.open(QIODevice::ReadOnly | QIODevice::Text)) {
            while (!st.atEnd()){
                l = st.readLine();
                w = l.split("~");
                QString k = w.takeFirst();
                if (exp.exactMatch(k) && (!w.isEmpty())) {
                    QStringList q;
                    foreach(QString tmp, w){
                        if (!tmp.isEmpty()) {q.append(tmp);}
                    }
                    words[k] = "~" + q.join("~") + "~";
                }
            }
            f.close();
            {
                QStringList abcNames;
                for(char i = 'a'; i <= 'a'; i++){
                    for(char j = 'a'; j <= 'z'; j++){
                        for(char k = 'a'; k <= 'z'; k++){
                            for(char l = 'a'; l <= 'z'; l++){
                                abcNames << QString(i) + QString(j)+ QString(k) + QString(l);
                            }
                        }
                    }
                }
                QSet<QString> a,b;
                b = QSet<QString>::fromList(abcNames);
                a = b.subtract(QSet<QString>::fromList(words.keys()));
                freeWords = a.toList();
            }
        }
    }
}

void MyWrdFile::setNeedUpdate(bool f)
{
    needUpdate = f;
}

bool MyWrdFile::addNewWord(QString s)
{
    if (freeWords.isEmpty()) {return false;}

    int n = qrand() % freeWords.count();
    QString tmp = freeWords.takeAt(n);
    newWords[tmp] = "~" + s + "~";
    words[tmp] = newWords[tmp];

    return true;
}

bool MyWrdFile::addSynonym(QString k, QString s)
{
    bool result = true;
    if (newWords.contains(k) || words.contains(k)) {
        if (newWords.contains(k)) {
            newWords[k].append(s + "~");
        } else { needUpdate = true; }
        words[k].append(s + "~");
    } else { result = false; }
    return result;
}

bool MyWrdFile::setSynonym(QString k, QStringList s)
{
    bool result = true;
    if (newWords.contains(k) || words.contains(k)) {
        if (newWords.contains(k)) {
            newWords[k] = "~" + s.join("~") + "~";
        } else { needUpdate = true; }
        words[k] = "~" + s.join("~") + "~";
    } else { result = false; }
    return result;
}

bool MyWrdFile::storeNewWords()
{
    QFile f(this->completeName());

    QTextStream st(&f);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    bool result = false;

    if (initOk()) {
        if (f.open(QIODevice::Append | QIODevice::Text)) {            
            foreach(QString k,newWords.keys()){
                st << k + newWords[k] << endl;                
            }            
            newWords.clear();
            result = true;
            f.close();
        }
    }
    return result;
}

bool MyWrdFile::storeAllWords()
{
    QFile f(this->completeName());

    QTextStream st(&f);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    bool result = false;

    if (initOk()) {
        if (f.open(QIODevice::WriteOnly | QIODevice::Text)) {
            foreach(QString k,words.keys()){
                st << k + words[k] << endl;
            }
            newWords.clear();
            result = true;
            needUpdate = false;            
            f.close();
        }
    }
    return result;
}

bool MyWrdFile::storeWords()
{
    bool result = false;

    if (needUpdate) {
        result = storeAllWords();
    } else {
        if (!newWords.isEmpty()){
            result = storeNewWords();
        }
    }
    return result;
}

QHash<QString,QString>* MyWrdFile::getWords()
{
    return &words;
}

QString MyWrdFile::replaceWordToKey(QString s)
{
    s = s.simplified();
    foreach (QString w, s.split(QRegExp("\\+|\\-"))){
        w = w.trimmed();
        if (w.isEmpty()) { continue; }
        QString p;        
        p ="~" + w + "~";
        int t;
        t = words.count();
        foreach(QString k, words.keys()){
            if (words.value(k).contains(p, Qt::CaseInsensitive)){
                s.replace(QRegExp("\\b" + w + "\\b"), k);
                break;
            }
            t--;
        }
        if (t == 0){
            s.replace(QRegExp("(\\+|\\-)\\b" + w + "\\b"), "");
        }
    }
    if (s.isEmpty()) { s = "nothing"; }
    return s;
}

bool MyWrdFile::wordExist(QString s)
{
    bool result;
    result = false;
    s = s.simplified();
    QString q;    
    q ="~" + s + "~";
    foreach(QString v, words.values()){
        if (v.contains(q, Qt::CaseInsensitive)){
            result = true;
            break;
        }
    }
    return result;
}

QStringList MyWrdFile::likeBegin(QString s)
{
    QStringList result;

    if (s.isEmpty()) { return result; }

    int n = 10;
    int p = -1;
    int e = -1;

    foreach(QString v, words.values()){
        p = v.indexOf("~" + s, 0, Qt::CaseInsensitive);
        if (p > -1) {
            e = v.indexOf("~", p + 1);
            if (e > -1) {
                result << v.mid(p + 1, e - (p + 1));
            }
        }
        if (result.count() > 10) {
            break;
        }
    }
    return result;
}

QStringList MyWrdFile::allWords()
{
    QStringList result;
    foreach(QString w, words.values()){
        w = w.mid(1, w.length()-2);
        result.append(w.split("~"));
    }
    return result;
}


QString MyWrdFile::getKey(QString s)
{
    QString result;
    result = "";
    s = s.simplified();
    if (s.isEmpty()) { return result; }
    QString q;    
    q ="~" + s + "~";
    foreach(QString k, words.keys()){
        if (words.value(k).contains(q, Qt::CaseInsensitive)){
            result = k;
            break;
        }
    }
    return result;
}
