#include "resembledialog.h"
#include "myabcfile.h"
#include "remark.h"
#include "global.h"

#include <QDir>
#include <QFileInfo>
#include <QKeyEvent>
#include <QMessageBox>

ResembleDialog::ResembleDialog(AnyBox* b, QFrame* w) : QFrame(w)
{
    m_box = b;
    setupUi(w);

    boxPixmap->setPixmap(m_box->boxPixmap());

    m_subset = 0;
    m_applyNotPressed = true;
    m_box->memorizeMarked();

    m_processPrime = new QProcess(this);
    m_processGroup = new QProcess(this);
    m_processPool = new QProcess(this);

    m_associateLabel = tr("associate with %1 following remarks");

    //.oO0
    label_B->setHidden(true);
    mode->setHidden(true);
    //0Oo.

    int num = m_box->markedEntriesCount();

    if (num == 0) {
        //.oO0
        label_A->setHidden(true);
        label_B->setVisible(true);
        //0Oo.
        return;
    }

    QTime curTime;
    curTime = QTime::currentTime();
    m_playlistNamePrime = curTime.toString("hhmmsszzz");
    m_playlistNameGroup = m_playlistNamePrime + "_2";
    m_playlistNamePool = m_playlistNamePrime + "_3";

    QApplication::setOverrideCursor(Qt::WaitCursor);

    foreach(Entry* entry, m_box->markedEntries(m_subset)){
        m_poolEntries[entry->getField(REMARK)] = entry;
        m_cross[entry->getField(REMARK)] = QSet<QString>();        
    } 

    m_box->unmarkAllEntries();

    foreach(MyAbcFile* abcFile, m_box->markAllEntries(m_subset)){
        foreach(Entry* entry, abcFile->markedEntries(m_subset)){
            QString r;
            r = entry->getField(REMARK);
            QString e;
            e = entry->getField(RESEMBLE);
            if (m_poolEntries.keys().contains(e)){
                if (r != e) {                    
                    m_poolEntries[r] = entry;                    
                    m_cross[e].insert(r);
                }
            } else {
                abcFile->unmarkEntry(entry, m_subset);
            }
        }
    }

    foreach(QString r, m_cross.keys()){
        int c;
        c = m_cross[r].count();
        m_viewList.append(r);
        m_state[r] = NOT_USE;
        if (c == 0){
            QListWidgetItem *a = new QListWidgetItem(r + " ...");
            a->setIcon(QIcon(":img/space.png"));
            resembleList->addItem(a);
        } else {
            QListWidgetItem *b = new QListWidgetItem(r + " ...\t" + m_associateLabel.arg(QString::number(c)));
            b->setIcon(QIcon(":img/space.png"));
            resembleList->addItem(b);
            foreach(QString re,m_cross[r].values()){
               QListWidgetItem *c =  new QListWidgetItem("... " + re);
               c->setIcon(QIcon(":img/space.png"));
               resembleList->addItem(c);
               m_viewList.append(re);
               m_state[re] = NOT_USE;
            }
        }
    }

    //.oO0
    label_A->setHidden(true);
    progressBar->setValue(0);
    progressBar->setMaximum(m_state.count());
    spring->setHidden(true);
    mode->setVisible(true);
    //0Oo.

    showPool();

    QApplication::restoreOverrideCursor();

    //.oO0
    setCurrentPrime("");
    resembleList->setCurrentRow(0);
    setModeAddButton(0);
    setModeUniqueButton(0);
    setModeChangeButton(0);
    setModeApplyButton();
    resembleList->setFocus();
    //0Oo.

    resembleList->installEventFilter(this);

    connect(applyButton, SIGNAL(clicked()), this, SLOT(pressApplyButton()));
    connect(addButton, SIGNAL(clicked()), this, SLOT(pressAddButton()));
    connect(uniqueButton, SIGNAL(clicked()), this, SLOT(pressUniqueButton()));
    connect(changeButton, SIGNAL(clicked()), this, SLOT(pressChangeButton()));

    connect(resembleList, SIGNAL(currentRowChanged(int)), this, SLOT(setModeAddButton(int)));
    connect(resembleList, SIGNAL(currentRowChanged(int)), this, SLOT(setModeUniqueButton(int)));
    connect(resembleList, SIGNAL(currentRowChanged(int)), this, SLOT(setModeChangeButton(int)));

}

ResembleDialog::~ResembleDialog()
{
    if (m_processPool->state() == QProcess::Running){
        m_processPool->terminate();
        m_processPool->waitForFinished(-1);        
    }

    if (QFile::exists(m_completePlaylistNamePool)) {
        QFile::remove(m_completePlaylistNamePool);
    }

    if (m_processPrime->state() == QProcess::Running) {
        m_processPrime->terminate();
        m_processPrime->waitForFinished(-1);        
    }

    if (QFile::exists(m_completePlaylistNamePrime)) {
        QFile::remove(m_completePlaylistNamePrime);
    }

    if (m_processGroup->state() == QProcess::Running) {
        m_processGroup->terminate();
        m_processGroup->waitForFinished(-1);        
    }

    if (QFile::exists(m_completePlaylistNameGroup)) {
        QFile::remove(m_completePlaylistNameGroup);
    }

    if (m_applyNotPressed) {
        m_box->restoreMarked();
    } else {        
        foreach(MyAbcFile* abcFile, m_box->markedAbcFiles(m_subset)){
            foreach(Entry* entry, abcFile->markedEntries(m_subset)){
                if (entry->getField(REMARK) == entry->getField(RESEMBLE)){
                } else {
                    abcFile->unmarkEntry(entry, m_subset);
                }
            }
        }

        emit needUpdateStatus();
    }
}

bool ResembleDialog::eventFilter(QObject *target, QEvent *event)
{
    bool result;
    result = false;

    if (target == resembleList) {
        if (event->type() == QEvent::KeyPress) {
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
            switch (keyEvent->key()){
            case Qt::Key_Up:
                if (resembleList->currentRow() == 0){
                    result = true;
                }
                break;
            case Qt::Key_Down:
                if (resembleList->currentRow() == resembleList->count() - 1){
                    result = true;
                }
                break;                
            case Qt::Key_Plus:
                addButton->click();
                result = true;
                break;
            case Qt::Key_Minus:
                uniqueButton->click();
                result = true;
                break;
            case Qt::Key_Space:
                changeButton->click();
                result = true;
                break;
            case Qt::Key_Enter:
            case Qt::Key_Return:
                if (keyEvent->modifiers() & Qt::ShiftModifier) {
                    applyButton->click();
                }
                result = true;
                break;
            default:;
            }
        }
    }
    return result;
}

void ResembleDialog::setCurrentPrime(QString newPrime)
{
    int oldRow = m_viewList.indexOf(m_currentPrime);
    int newRow = m_viewList.indexOf(newPrime);

    QListWidgetItem *c;

    if (oldRow > -1) {

        c = resembleList->item(oldRow);

        //.oO0
        boldText.setBold(false);
        c->setFont(boldText);
        //0Oo.
    }

    if (newRow > -1){

        c = resembleList->item(newRow);

        //.oO0
        boldText.setBold(true);
        c->setFont(boldText);

        prime->setText(newPrime);
        //0Oo.

    } else {
        prime->setText("- - -");
    }

    m_currentPrime = newPrime;
}

void ResembleDialog::pressApplyButton()
{
    m_applyNotPressed = false;

    QApplication::setOverrideCursor(Qt::WaitCursor);

    //.oO0
    applyButton->setDisabled(true);
    progressBar->setMaximum(m_state.count());
    //0Oo.

    m_cross.clear();

    foreach(MyAbcFile* abcFile, m_box->markedAbcFiles(m_subset)){
        foreach(Entry* entry, abcFile->markedEntries(m_subset)){
            QString r;
            r = entry->getField(REMARK);
            QString e;
            e = entry->getField(RESEMBLE);
            switch(m_state.value(r)){
            case USE:
                if (r == m_currentPrime) {
                    if (!m_cross.contains(r)){
                        m_cross[r] = QSet<QString>();
                    }
                } else {
                    m_cross[m_currentPrime].insert(r);
                }
                entry->setField(RESEMBLE, m_currentPrime);
                break;
            case NOT_USE:
                if (r == e){
                    if (!m_cross.contains(e)){
                        m_cross[e] = QSet<QString>();
                    }
                } else {
                    m_cross[e].insert(r);
                }
                break;
            case MAKE_FREE:
                m_cross[r] = QSet<QString>();
                entry->setField(RESEMBLE,r);
                break;
            default:;
            }

            //0Oo.
            progressBar->setValue(progressBar->value() + 1);
            //0Oo.

        }
        abcFile->store();
    }

    setCurrentPrime("");
    m_viewList.clear();

    int currentRow = 0;
    QListWidgetItem* i;

    foreach(QString tmp, m_cross.keys()){
        int c;
        c = m_cross[tmp].count();
        m_viewList.append(tmp);
        setState(tmp, NOT_USE);
        if (c == 0){
            i = resembleList->item(currentRow);
            i->setText(tmp + " ...");
            currentRow++;
        } else {
            i = resembleList->item(currentRow);                        
            i->setText(tmp + " ...\t" + m_associateLabel.arg(QString::number(c)));
            currentRow++;
            foreach(QString re,m_cross[tmp].values()){
               i = resembleList->item(currentRow);               
               i->setText("... " + re);
               m_viewList.append(re);
               setState(re, NOT_USE);
               currentRow++;
            }
        }
    }

    m_poolEntries.unite(m_groupEntries);
    m_groupEntries.clear();

    //.oO0
    resembleList->setCurrentRow(0);
    setModeAddButton(0);
    setModeUniqueButton(0);
    setModeChangeButton(0);
    setModeApplyButton();
    resembleList->setFocus();
    //0Oo.        

    QApplication::restoreOverrideCursor();
}

void ResembleDialog::setModeApplyButton()
{ 
    bool result = false;
    foreach(QString r, m_state.keys()){
        if (((m_currentPrime != r) && (m_state.value(r) == USE)) || (m_state.value(r) == MAKE_FREE)){
            result = true;
            break;
        }
    }
    applyButton->setEnabled(result);
}

void ResembleDialog::showGroup()
{
    if (m_processPrime->state() == QProcess::Running){
        m_processPrime->terminate();
        m_processPrime->waitForFinished(-1);        
    }

    if (m_processGroup->state() == QProcess::Running){
        m_processGroup->terminate();
        m_processGroup->waitForFinished(-1);        
    }

    QList<Entry*> prime;
    QMap<QString, Entry*> group;
    group = m_groupEntries;

    if (group.contains(m_currentPrime)){
        prime << group.take(m_currentPrime);
    }

    if (!group.values().isEmpty()){
        m_completePlaylistNameGroup = m_box->makePlayListFile(m_playlistNameGroup, group.values());        
        m_processGroup->start(m_box->execPreview().arg(m_completePlaylistNameGroup));
    }

    if (!prime.isEmpty()){
       m_completePlaylistNamePrime = m_box->makePlayListFile(m_playlistNamePrime, prime);       
       m_processPrime->start(m_box->execPreview().arg(m_completePlaylistNamePrime));
    }
}

void ResembleDialog::showPool()
{
    if (m_processPool->state() == QProcess::Running){
        m_processPool->terminate();
        m_processPool->waitForFinished(-1);        
    }

    if (!m_poolEntries.values().isEmpty()){
        m_completePlaylistNamePool = m_box->makePlayListFile(m_playlistNamePool, m_poolEntries.values());
        m_processPool->start(m_box->execPreview().arg(m_completePlaylistNamePool));
    }    
}

void ResembleDialog::setState(QString r, int st)
{
    m_state[r] = st;

    QListWidgetItem *c;
    c = resembleList->item(m_viewList.indexOf(r));

    switch (st){
    case USE:
        c->setIcon(QIcon(":img/nextStage.png"));
        break;
    case NOT_USE:
        c->setIcon(QIcon(":img/space.png"));
        break;
    case MAKE_FREE:
        c->setIcon(QIcon(":img/delStage.png"));
        break;
    default:
        m_state[r] = USE;
        c->setIcon(QIcon(":img/nextStage.png"));
    }
}

void ResembleDialog::pressAddButton()
{
    int row = resembleList->currentRow();
    QString remark = m_viewList.at(row);

    QListWidgetItem *c;
    c = resembleList->item(row);

    QString curPrime = m_currentPrime;

    switch(m_state[remark]){
    case NOT_USE:
        if (curPrime.isEmpty()) {
            curPrime = remark;
        }
        setState(remark, USE);
        m_groupEntries[remark] = m_poolEntries.take(remark);

        foreach(QString r, m_cross.value(remark)){
            setState(r, USE);
            m_groupEntries[r] = m_poolEntries.take(r);
        }
        break;
    case USE:
        if (curPrime == remark){
            foreach(QString r, m_groupEntries.keys()){
                m_poolEntries[r] = m_groupEntries.take(r);
                setState(r, NOT_USE);
            }
            curPrime.clear();
        } else {
            m_poolEntries[remark] = m_groupEntries.take(remark);
            setState(remark, NOT_USE);
            foreach(QString r, m_cross.value(remark)){
                m_poolEntries[r] = m_groupEntries.take(r);
                setState(r, NOT_USE);
            }
        }
        break;
    default:        ;
    }
    setCurrentPrime(curPrime);
    setModeUniqueButton(row);
    setModeAddButton(row);
    setModeChangeButton(row);
    setModeApplyButton();

    showPool();
    showGroup();
}

void ResembleDialog::pressUniqueButton()
{
    int row = resembleList->currentRow();
    QString remark = m_viewList.at(row);

    if (m_cross.keys().contains(remark)){
        bool f = false;
        foreach(QString r, m_cross.value(remark)){
            if (m_state.value(r) != MAKE_FREE){
                f = true;
                break;
            }
        }
        foreach (QString r, m_cross.value(remark)){
            if (f) {
                setState(r, MAKE_FREE);
            } else {
                setState(r, NOT_USE);
            }
        }
    } else {
        if (m_state.value(remark) == MAKE_FREE){
            setState(remark, NOT_USE);
        } else {
            setState(remark, MAKE_FREE);
        }
    }
    setModeUniqueButton(row);
    setModeAddButton(row);
    setModeApplyButton();
}

void ResembleDialog::pressChangeButton()
{
    int row = resembleList->currentRow();
    QString remark = m_viewList.at(row);

    setCurrentPrime(remark);
    setModeChangeButton(row);

    showGroup();
}

void ResembleDialog::setModeAddButton(int row)
{
    QString labelStr = addButton->text();
    QString remark = m_viewList.at(row);

    foreach(QString r, m_viewList){
        if (m_state.value(r) == MAKE_FREE){
            //.oO0
            addButton->setEnabled(false);
            //0Oo.
            return;
        }
    }

    if (m_cross.keys().contains(remark)){
        //.oO0
        addButton->setEnabled(true);
        //0Oo.

        switch (m_state.value(remark)){
        case NOT_USE:
            if (m_groupEntries.isEmpty()){
                labelStr = tr("&Create");
            } else {
                labelStr = tr("A&dd");
            }
            break;
        case USE:
            labelStr = tr("&Remove");
            break;
        default:
            ;
        }
    } else {
        //.oO0
        addButton->setEnabled(false);
        //0Oo.
    }
    addButton->setText(labelStr);
}

void ResembleDialog::setModeUniqueButton(int row)
{
     QString labelStr = uniqueButton->text();
     QString remark = m_viewList.at(row);

     if (!m_currentPrime.isEmpty()){
         //.oO0
         uniqueButton->setEnabled(false);
         //0Oo.
         return;
     }

     if (m_cross.keys().contains(remark)){
         labelStr = tr("All &unique");
         if (m_cross.value(remark).isEmpty()){
            uniqueButton->setEnabled(false);
         } else {
             uniqueButton->setEnabled(true);
             bool f = false;
             foreach(QString r, m_cross.value(remark)){
                 if (m_state.value(r) != MAKE_FREE){
                     f = true;
                     break;
                 }
             }
             if (f) {
                 labelStr = tr("All &unique");
            } else {
                labelStr = tr("&Return all");
            }
         }
     } else {
         uniqueButton->setEnabled(true);
         if (m_state[remark] == MAKE_FREE){
             labelStr = tr("&Return back");
         } else {
             labelStr = tr("&Unique");
         }
     }
     uniqueButton->setText(labelStr);
}

void ResembleDialog::setModeChangeButton(int row)
{
    if (m_currentPrime.isEmpty()){
        changeButton->setEnabled(false);
    }
    QString remark = m_viewList.at(row);
    if ((m_currentPrime != remark) && (m_groupEntries.contains(remark)) && (m_cross.contains(remark))){
        changeButton->setEnabled(true);
    } else {
        changeButton->setEnabled(false);
    }
}

void ResembleDialog::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
        retranslateThis();
    }
    QFrame::changeEvent(event);
}

void ResembleDialog::retranslateThis()
{
    int r = resembleList->currentRow();

    if (r > -1) {
        setModeAddButton(r);
        setModeUniqueButton(r);
    }

    m_associateLabel = tr("associate with %1 following remarks");

    QRegExp str_exp("^\\w+\\s\\.\\.\\.\\s\\w.+");
    QRegExp num_exp("\\d+");
    for (int r = 0; r < resembleList->count(); r++){
        QString s = resembleList->item(r)->text();
        if (str_exp.exactMatch(s)){
            int a = s.indexOf(" ");
            QString rem = s.mid(0, a);
            int i = num_exp.indexIn(s);
            int j = s.indexOf(" ", i);
            QString num = s.mid(i, j - i);
            resembleList->item(r)->setText(rem + " ...\t" + m_associateLabel.arg(num));
        }
    }
}
