#include "restoredialog.h"
#include "global.h"
#include "remark.h"
#include "altqfile.h"

#include <QDir>
#include <QTime>

RestoreDialog::RestoreDialog(AnyBox* b, QFrame* w) : QFrame(w)
{
    m_box = b;
    setupUi(w);

    boxPixmap->setPixmap(m_box->boxPixmap());

    m_useOriginalName = false;
    m_needUpdateAbcFiles = false;
    m_emptyDest = false;

    log = static_cast<MyLogFile*>(m_box->unit(NAME_LOG_FILE));

    //.oO0    
    label_B->setHidden(true);    
    mode->setHidden(true);
    useOriginalFilename->setHidden(true);    
    //0Oo.

    num = m_box->markedEntriesCount();

    if (num == 0) {
        //.oO0
        label_A->setHidden(true);
        label_B->setVisible(true);
        //0Oo.
        return;
    }

    totalSize = 0;
    foreach(Entry* entry, m_box->markedEntries()){
        Remark r(entry->getField(REMARK));
        QString o(entry->getField(ORIGINALFILENAME));
        QString str = m_box->completeName() + QDir::separator() + r.re() + QDir::separator() + r.ma() +
                      QDir::separator() + r.rk() + "." + o.section(".",-1,-1);
        QFile f(str);
        totalSize += f.size();
    }

    // .oO0    
    select->setText(QString::number(num));
    selectSize->setText(QString::number(qreal(totalSize)/1048576,'f',1));
    restore->setText("0");
    restoreSize->setText("0.0");
    remain->setText(QString::number(num));
    remainSize->setText(QString::number(qreal(totalSize)/1048576,'f',1));
    progressBar->setMaximum(num);
    label_A->setHidden(true);
    mode->setVisible(true);
    startButton->setFocus();
    // 0Oo.

    connect(startButton,SIGNAL(clicked()),this,SLOT(pressStartButton()));    
    connect(clearButton,SIGNAL(clicked()),this,SLOT(pressClearButton()));
}

QString RestoreDialog::getStatus()
{
    QDir dstContent(dst->completeName());
    QString status;

    int dstCount;
    dstCount = dstContent.entryList(QDir::Files).count();

    if (dstCount == 0) {
        status = tr("<b>empty</b>");
        m_emptyDest = true;
    } else {
        status = tr("<b>not empty</b> / existing files: ") + QString::number(dstCount);
        m_emptyDest = false;
    }
    return status;
}

void RestoreDialog::setModeClearButton()
{
    if (m_emptyDest){
        clearButton->setEnabled(false);
    } else {
        clearButton->setEnabled(true);
    }
}

void RestoreDialog::pressStartButton()
{

    QApplication::setOverrideCursor(Qt::WaitCursor);

    //.oO0
    label_F->setEnabled(true);
    currentRemark->setEnabled(true);
    clearButton->setDisabled(true);
    startButton->setDisabled(true);
    //0Oo.

    bool copyOk;

    int restoreCount, remainCount;

    restoreCount = 0;
    remainCount = num;

    quint64 restoreCountSize, remainCountSize;
    restoreCountSize = 0;
    remainCountSize = totalSize;

    QTime elapsedTime;
    elapsedTime.start();

    QTime tm(0,0,0,0);  

    log->openLog("RESTORE");    

    foreach(Entry* entry, m_box->markedEntries()){
        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);

        Remark r(entry->getField(REMARK));
        QString o(entry->getField(ORIGINALFILENAME));
        QString filename = m_box->completeName() + QDir::separator() + r.re() + QDir::separator() + r.ma() +
                           QDir::separator() + r.rk() + "." + o.section(".",-1,-1);
        AltQFile f(filename);

        //.oO0
        currentRemark->setText(entry->getField(REMARK));
        //0Oo.

        QString copiedFilename;

        if (m_useOriginalName) {
            copiedFilename = dst->completeName() + QDir::separator() + entry->originalFileName();
        } else {            
            copiedFilename = dst->completeName() + QDir::separator() + entry->remarkFileName();
        }

        copyOk = f.copy(copiedFilename);
        if (copyOk){

            QFile copiedFile(copiedFilename);
            copiedFile.setPermissions(QFile::ReadOwner|QFile::WriteOwner|
                                      QFile::ReadUser|QFile::WriteUser|
                                      QFile::ReadGroup|QFile::WriteGroup|
                                      QFile::ReadOther|QFile::WriteOther);

            restoreCount++;            
            remainCount--;
            restoreCountSize += f.size();
            remainCountSize = totalSize - restoreCountSize;

            *log << "    " +
                    QString::number(restoreCount) +
                    " Copy " + filename +
                    " to " + copiedFilename;

            updateCurrentEntry(entry);

            //.oO0
            restore->setText(QString::number(restoreCount));
            restoreSize->setText(QString::number(qreal(restoreCountSize)/1048576,'f',1));
            remain->setText(QString::number(remainCount));
            remainSize->setText(QString::number(qreal(remainCountSize)/1048576,'f',1));            
            //0Oo.
        }

        //.oO0
        progressBar->setValue(progressBar->value() + 1);
        time->setText(tm.addMSecs(elapsedTime.elapsed()).toString("hh:mm:ss:zzz"));
        //0Oo.

    }

    log->closeLog();

    if (m_needUpdateAbcFiles){ m_box->storeMarkedEntries(); }

    //.oO0
    label_F->setDisabled(true);
    currentRemark->setText(" ");
    currentRemark->setDisabled(true);
    destinationStatus->setText(getStatus());
    destinationStatus->setDisabled(true);
    clearButton->setDisabled(true);
    //0Oo.

    QApplication::restoreOverrideCursor();
}

void RestoreDialog::pressClearButton()
{
    QDir dstContent(dst->completeName());

    QApplication::setOverrideCursor(Qt::WaitCursor);

    foreach(QString tmp, dstContent.entryList(QDir::Files))
    {
        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
        dstContent.remove(tmp);
    }

    QApplication::restoreOverrideCursor();

    // .oO0
    destinationStatus->setText(getStatus());
    setModeClearButton();
    // 0Oo.
}

void RestoreDialog::updateCurrentEntry(Entry* entry)
{

}

void RestoreDialog::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
        retranslateThis();
    }
    QFrame::changeEvent(event);
}

void RestoreDialog::retranslateThis()
{

}
