#include "restoreprintdialog.h"
#include "remark.h"
#include "global.h"
#include "basefolder.h"

#include <QTextStream>
#include <QDate>

RestorePrintDialog::RestorePrintDialog(AnyBox* b, QFrame* w) : RestoreDialog(b, w)
{    
    dst = m_box->unit(NAME_DST_FOLDER);

    m_needUpdateAbcFiles = true;

    processBurn = new QProcess(this);
    burnNotFinish = true;
    burnStatusCode = 0;

    // .oO0    
    title->setText(tr("For printing"));
    destination->setText(dst->completeName());
    destinationStatus->setText(getStatus());
    setModeClearButton();

    horizontalLayout_L = new QHBoxLayout();
    horizontalLayout_L->setObjectName(QString::fromUtf8("horizontalLayout_L"));
    horizontalSpacer_L = new QSpacerItem(40, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
    horizontalLayout_L->addItem(horizontalSpacer_L);
    makeLabelsButton = new QPushButton(mode);
    makeLabelsButton->setObjectName(QString::fromUtf8("makeLabelsButton"));
    makeLabelsButton->setHidden(true);
    horizontalLayout_L->addWidget(makeLabelsButton);
    makeLabelsButton->setMinimumSize(90, 0);
    makeLabelsButton->setText(tr("Ma&ke labels"));

    horizontalLayout_B = new QHBoxLayout();
    horizontalLayout_B->setObjectName(QString::fromUtf8("horizontalLayout_B"));
    label_K = new QLabel(mode);
    label_K->setObjectName(QString::fromUtf8("label_K"));
    label_K->setHidden(true);
    label_K->setText(setBurnStatusLabel());

    horizontalLayout_B->addWidget(label_K);
    horizontalSpacer_B = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
    horizontalLayout_B->addItem(horizontalSpacer_B);
    onCDButton = new QPushButton(mode);
    onCDButton->setObjectName(QString::fromUtf8("onCDButton"));
    onCDButton->setEnabled(false);
    onCDButton->setText(tr("B&urn CD"));
    horizontalLayout_B->addWidget(onCDButton);

    buttonsLayout->addLayout(horizontalLayout_L);
    buttonsLayout->addLayout(horizontalLayout_B);

    onCDButton->setMinimumSize(90, 0);    
    // 0Oo.

    connect(makeLabelsButton,SIGNAL(clicked()), this, SLOT(makeLabels()));
    connect(onCDButton,SIGNAL(clicked()),this,SLOT(pressOnCDButton()));
    connect(processBurn,SIGNAL(finished(int)),this,SLOT(setBurnFinish()));  
}

void RestorePrintDialog::updateCurrentEntry(Entry* entry)
{
    QString p;
    p = entry->getField(PRINT);
    p = QString::number(p.toUInt() + 1);
    entry->setField(PRINT, p);
}

void RestorePrintDialog::pressStartButton()
{    
    RestoreDialog::pressStartButton();    
    makeLabels();
    //.oO0
    label_K->setVisible(true);
    onCDButton->setEnabled(true);
    //0Oo.
}

void RestorePrintDialog::pressOnCDButton()
{       
    log->openLog("BURNING");

    //.oO0
    label_K->setText(tr("Burning... "));
    onCDButton->setDisabled(true);
    //0Oo.

    QDate currentDate;
    currentDate = QDate::currentDate();
    QString diskLabel;
    diskLabel = m_box->objectName() + "_" + currentDate.toString("ddMMyyyy");

    QString burnCDApp = static_cast<BaseFolder*>(m_box->baseUnit())->burnCDApp();
    processBurn->start(burnCDApp.arg(diskLabel, dst->completeName()));

    if (processBurn->waitForStarted(-1)){

        QApplication::setOverrideCursor(Qt::WaitCursor);

        while(burnNotFinish){
            QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
        }

        QApplication::restoreOverrideCursor();

        burnStatusCode = 1;

    } else {

        burnStatusCode = 2;
    }

    //.oO0
    label_K->setText(setBurnStatusLabel());
    //0Oo.

    log->closeLog();
}

void RestorePrintDialog::setBurnFinish()
{
    burnNotFinish = false;
}

bool RestorePrintDialog::makeLabels()
{        
    QApplication::setOverrideCursor(Qt::WaitCursor);

    bool result = true;

    //.oO0
    makeLabelsButton->setDisabled(true);
    useOriginalFilename->setDisabled(true);
    //0Oo.

    QFile lbl_file(dst->completeName() + "/labels.txt");

    if (m_useOriginalName){
        if (lbl_file.exists()){
            lbl_file.remove();
        }
        return true;
    }

    QTextStream st(&lbl_file);
    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    if (lbl_file.open(QIODevice::WriteOnly | QIODevice::Text)){
        foreach(Entry* entry, m_box->markedEntries()) {
            QString r;
            r = entry->getField(REMARK);
            QString d;
            d = entry->getField(DATEOFCREATION);
            QString ta;
            ta = entry->getField(TITLE);
            QStringList l;
            l = ta.split(",");
            QString t,a;
            if (l.at(0) == "unknown") {
                t = "";
            } else {
                t = l.at(0);
            }
            if (l.count() == 2) {
                if (l.at(1) == "unknown") {
                    a = "";
                } else {
                    a = l.at(1);
                }
            }

            st << r << "~" << d << "~" << t << "~" << a << "\n";
        }

        lbl_file.close();

    } else {
        result = false;
    }

    QApplication::restoreOverrideCursor();
    return result;
}

QString RestorePrintDialog::setBurnStatusLabel()
{
    QString result;

    switch (burnStatusCode){
    case 0:
        result = tr("Insert disk in CDROM drive and press button...");
        break;
    case 1:
        result = tr("The burning process is complete");
        break;
    case 2:
        result = tr("The burning process cannot be started...");
        break;
    default:;
    }

    return result;
}

void RestorePrintDialog::retranslateThis()
{
    title->setText(tr("For printing"));
    destinationStatus->setText(getStatus());
    makeLabelsButton->setText(tr("Ma&ke labels"));
    label_K->setText(setBurnStatusLabel());
    onCDButton->setText(tr("B&urn CD"));
}

