#include "searchdialog.h"

#include "myabcfile.h"
#include "mytmpfile.h"
#include "selectionframe.h"

#include <QKeyEvent>
#include <QMessageBox>

SearchDialog::SearchDialog(AnyBox* b, QFrame* w) : QFrame(w)
{    
    m_box = b;
    setupUi(w);

    boxPixmap->setPixmap(m_box->boxPixmap());

    process = new QProcess(this);

    //.oO0
    label_A->hide();
    mode->hide();
    //0Oo. 

    if (m_box->getCount() == 0) {
        return;
    }

    //.oO0
    label->hide();
    label_A->show();
    //0Oo.

    QApplication::setOverrideCursor(Qt::WaitCursor);

    foreach(MyAbcFile* abcFile, m_box->findChildren<MyAbcFile*>(QRegExp("^" + NAME_ABC_FILE + "_"))){
        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
        foreach (Entry tmp,  abcFile->getAll()){

            uint entrySession;
            entrySession = tmp.getField(SESSION).toUInt();
            m_sessionNumbers.insert(entrySession);

            QDate entryDate;
            entryDate = QDate::fromString(tmp.getField(DATEOFCREATION), m_box->dateFormat());

            if (!m_sessionBegins.contains(entrySession)){
                m_sessionBegins.insert(entrySession,entryDate);
                m_sessionEnds.insert(entrySession,entryDate);
                m_sessionCounts.insert(entrySession, 1);
            } else {
                if (entryDate < m_sessionBegins.value(entrySession)) {
                    m_sessionBegins[entrySession] = entryDate;
                }
                if (entryDate > m_sessionEnds.value(entrySession)) {
                    m_sessionEnds[entrySession] = entryDate;
                }
                m_sessionCounts[entrySession]++;
            }
        }
    }

    QApplication::restoreOverrideCursor();

    // .oO0
    label_A->hide();
    mode->show();

    createStageButton->setDisabled(true);
    // 0Oo.

    QString frameName;
    frameName = "Stage_" + QString::number(stages->currentIndex() + 1);
    new SelectionFrame(frameName, m_box, this, dialogFrame);

    updateStagesCount();

    connect(deleteStageButton,SIGNAL(clicked()),this,SLOT(pressDeleteStage()));
    connect(createStageButton,SIGNAL(clicked()),this,SLOT(pressCreateStage()));
    connect(saveButton,SIGNAL(clicked()),this,SLOT(pressSave()));

    connect(prevStageButton,SIGNAL(clicked()),this,SLOT(pressPrevStage()));
    connect(nextStageButton,SIGNAL(clicked()),this,SLOT(pressNextStage()));

    connect(m_box, SIGNAL(showReduce()), this, SLOT(warningShowReduce()));
}

SearchDialog::~SearchDialog()
{ 
    closeShowStage();
    m_box->uniteSubsets();
}

void SearchDialog::pressDeleteStage()
{    
    SelectionFrame *s;

    this->closeShowStage();

    m_box->deleteSubset(stages->currentIndex());

    emit needUpdateStatus();

    QWidget *c;
    c = stages->currentWidget();
    stages->removeWidget(c);
    delete c;   

    if (stages->count() == 0) {

        QFrame *f;
        f = new QFrame();
        stages->addWidget(f);
        stages->setCurrentWidget(f);

        QString frameName;
        frameName = "Stage_" + QString::number(stages->currentIndex() + 1);
        new SelectionFrame(frameName, m_box, this, f);

    } else {

        for (int i = stages->currentIndex(); i < stages->count(); i++){
            s = stages->findChild<SelectionFrame*>("Stage_" + QString::number(i + 2));
            if (s) {
                s->setObjectName("Stage_" + QString::number(i + 1));                
            }
        }

        this->showStage();
    }

    updateStagesCount();
    setModePrevStageButton();
    setModeNextStageButton();
    setModeCreateStageButton();
    setModeSaveButton();
}

void SearchDialog::pressCreateStage()
{    
    SelectionFrame *s;

    this->closeShowStage();

    QFrame *f;
    f = new QFrame();
    stages->addWidget(f);
    stages->setCurrentWidget(f);

    QString frameName;
    frameName = "Stage_" + QString::number(stages->currentIndex() + 1);
    new SelectionFrame(frameName, m_box, this, f);

    //.oO0
    createStageButton->setDisabled(true);
    saveButton->setDisabled(true);
    //0Oo.

    updateStagesCount();    
    setModePrevStageButton();
    setModeNextStageButton();
}

void SearchDialog::pressSave()
{
    saveButton->setDisabled(true);

    MyTmpFile::tasks allSteps, steps;
    bool firstStage = true;
    foreach(SelectionFrame* part, stages->findChildren<SelectionFrame*>()){
        steps = part->getSteps();
        if (firstStage) {
            firstStage = false;
        } else {
            steps[0].addTask = true;
        }
        allSteps.append(steps);               
    }    

    m_box->makePlayListFile(templateName->text());

    QString tempName;
    tempName = templateName->text().simplified();
    if (!tempName.isEmpty() && tempName != "anonymous"){
        MyTmpFile* tmp;
        tmp = static_cast<MyTmpFile*>(m_box->unit(NAME_TMP_FILE));
        tmp->addTask(m_box->objectName(), tempName, templateDescription->text(), allSteps);
    }
}

void SearchDialog::updateStagesCount()
{   
    // .oO0    
    label_CurrentStage->setText(QString::number(stages->currentIndex() + 1));
    label_TotalStage->setText(QString::number(stages->count()));
    // 0Oo.
}

void SearchDialog::pressPrevStage()
{
    SelectionFrame *f;

    int c;    
    c = stages->currentIndex();

    this->closeShowStage();

    if (c > 0){
        c--;
        stages->setCurrentIndex(c);
    }

    this->showStage();

    updateStagesCount();
    setModePrevStageButton();
    setModeNextStageButton();
}

void SearchDialog::pressNextStage()
{
    SelectionFrame *f;

    int c,t;
    c = stages->currentIndex();

    this->closeShowStage();

    t = stages->count();
    if (c < (t - 1)){
        c++;
        stages->setCurrentIndex(c);
    }

    this->showStage();

    updateStagesCount();
    setModePrevStageButton();
    setModeNextStageButton();
}

void SearchDialog::setModePrevStageButton()
{
    int c;
    c = stages->currentIndex();
    if (c > 0){
        prevStageButton->setEnabled(true);
    } else {
        prevStageButton->setDisabled(true);
    }
}

void SearchDialog::setModeNextStageButton()
{
    int c,t;
    c = stages->currentIndex();
    t = stages->count();
    if (c < (t - 1)){
        nextStageButton->setEnabled(true);
    } else {
        nextStageButton->setDisabled(true);
    }
}

void SearchDialog::setModeSaveButton()
{
    bool found;
    found = true;

    foreach(SelectionFrame* part, stages->findChildren<SelectionFrame*>()){
        if (part->getRemarkFoundCount() == 0) {
            found = false;
        }
    }

    if (found) {
        saveButton->setEnabled(true);
    } else {
        saveButton->setDisabled(true);
    }
}

bool SearchDialog::needSave()
{
    return saveButton->isEnabled();
}

void SearchDialog::setModeCreateStageButton()
{
    SelectionFrame *s;
    s = stages->findChild<SelectionFrame*>("Stage_" + QString::number(stages->currentIndex() + 1));
    if (s->getRemarkFoundCount() > 0) {
        createStageButton->setEnabled(true);
    } else {
        createStageButton->setDisabled(true);
    }
}

void SearchDialog::showStage()
{
    if (process->state() == QProcess::Running) {
        process->terminate();
        process->waitForFinished(-1);        
    }

    if (QFile::exists(m_sldFilename)){
        QFile::remove(m_sldFilename);
    }

    int stageNumber = stages->currentIndex() + 1;

    SelectionFrame* s;
    s = stages->findChild<SelectionFrame*>("Stage_" + QString::number(stageNumber));    

    if (s->getRemarkFoundCount() > 0) {
        m_sldFilename = m_box->previewSelected(stageNumber - 1, process);
    }
}

void SearchDialog::closeShowStage()
{
    if (process->state() == QProcess::Running){
        process->terminate();
        process->waitForFinished(-1);
    }

    if (QFile::exists(m_sldFilename)){
        QFile::remove(m_sldFilename);
    }
}

void SearchDialog::warningShowReduce()
{
    QMessageBox::information(this,tr("Warning"), tr("Selected list is very big. The output will be truncated..."));
}

QSet<uint>* SearchDialog::sessionNumbers()
{
    return &m_sessionNumbers;
}

QMap<uint, QDate>* SearchDialog::sessionBegins()
{
    return &m_sessionBegins;
}

QMap<uint, QDate>* SearchDialog::sessionEnds()
{
    return &m_sessionEnds;
}

QMap<uint, uint>* SearchDialog::sessionCounts()
{
    return &m_sessionCounts;
}

void SearchDialog::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
    }
    QFrame::changeEvent(event);
}
