#include "selectionframe.h"
#include "global.h"
#include "myabcfile.h"
#include "remark.h"
#include "selector.h"
#include "category.h"
#include "mysssfile.h"

#include <QKeyEvent>
#include <QMessageBox>
#include <QTableWidgetItem>

SelectionFrame::SelectionFrame(const QString &n, AnyBox *b, SearchDialog* dialog, QFrame *frame)
    : QFrame(frame)
{
    setObjectName(n);
    setupUi(frame);

    m_box = b;

    fromLabel = tr("&From");
    beforeLabel = tr("&Before");

    firstStep = true;

    retranslateCategory();

    wrd = static_cast<MyWrdFile*>(m_box->unit(NAME_WRD_FILE));

    keyPresentList.clear();
    keyAbsentList.clear();

    setTabTitles();

    modifySymbol = "*";

    remarkFound = 0;

    // .oO0
    progressBar->setMinimum(0);
    progressBar->setValue(0);
    foundRecords->setText(QString::number(remarkFound));
    // 0Oo.

    //Init tab remark
    remark = "";
    QRegExp remarkExp(ABC_REMARK);
    remarkEdit->setValidator(new QRegExpValidator(remarkExp, this));
    connect(remarkEdit, SIGNAL(textChanged(QString)),
            this, SLOT(setModeRemarkAddButton(QString)));
    connect(remarkAddButton,SIGNAL(clicked()),
            this, SLOT(pressRemarkAddButton()));
    connect(remarkList,SIGNAL(itemSelectionChanged()),
            this, SLOT(setModeRemarkRemoveButton()));
    connect(remarkClearButton, SIGNAL(clicked()),
            this, SLOT(pressRemarkClearButton()));
    connect(remarkRemoveButton, SIGNAL(clicked()),
            this, SLOT(pressRemarkRemoveButton()));


    //Init tab session
    session = "";

    connect(sessionClearButton, SIGNAL(clicked()), this, SLOT(pressSessionClearButton()));

    MySssFile* s;
    s = static_cast<MySssFile*>(m_box->unit(NAME_SSS_FILE));

    sessionTable->setColumnWidth(NO, 40);
    sessionTable->setColumnWidth(CREATION_DATE, 150);
    sessionTable->setColumnWidth(COUNT, 50);
    sessionTable->setColumnWidth(TIME_LENGTH, 180);

    QList<uint> ssl;
    ssl = dialog->sessionNumbers()->values();
    qSort(ssl.begin(), ssl.end());

    int r;
    r = 0;

    QTableWidgetItem* i;

    foreach(uint tmpSession, ssl){
        sessionTable->resizeRowToContents(r);
        sessionTable->setRowCount(r+1);
        i = new QTableWidgetItem(QString::number(tmpSession));
        i->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        sessionTable->setItem(r,NO,i);

        i = new QTableWidgetItem(s->sessionDateCreation(tmpSession));
        i->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
        sessionTable->setItem(r,CREATION_DATE,i);

        i = new QTableWidgetItem(QString::number(dialog->sessionCounts()->value(tmpSession)));
        i->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        sessionTable->setItem(r,COUNT,i);

        QString tl;
        if (dialog->sessionBegins()->value(tmpSession) == dialog->sessionEnds()->value(tmpSession)){
            tl = dialog->sessionBegins()->value(tmpSession).toString(m_box->dateFormat());
        } else {
            tl = dialog->sessionBegins()->value(tmpSession).toString(m_box->dateFormat()) + " - " +
                 dialog->sessionEnds()->value(tmpSession).toString(m_box->dateFormat());
        }

        i = new QTableWidgetItem(tl);
        i->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
        sessionTable->setItem(r,TIME_LENGTH,i);

        i = new QTableWidgetItem(s->sessionAnnotation(tmpSession));
        i->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
        sessionTable->setItem(r,ANNOTATION,i);

        r++;
    }

    connect(sessionAnnFilterEdit, SIGNAL(textChanged(QString)),
            this, SLOT(setModeSessionClearAnnFilterButton()));
    connect(sessionAnnFilterEdit, SIGNAL(textChanged(QString)),
            this, SLOT(applySessionAnnFilter(QString)));
    connect(sessionClearAnnFilterButton, SIGNAL(clicked()),
            this, SLOT(pressSessionClearAnnFilterButton()));
    connect(sessionTable, SIGNAL(itemSelectionChanged()),
            this, SLOT(setSessionLst()));

    m_sessionLst.clear();    

    //Init tab title&author
    title = "";
    connect(titleEdit, SIGNAL(textChanged(QString)),
            this, SLOT(setModeTitleAddButton(QString)));
    connect(titleAddButton,SIGNAL(clicked()),
            this, SLOT(pressTitleAddButton()));
    connect(titleList,SIGNAL(itemSelectionChanged()),
            this, SLOT(setModeTitleRemoveButton()));
    connect(titleClearButton, SIGNAL(clicked()),
            this, SLOT(pressTitleClearButton()));
    connect(titleRemoveButton, SIGNAL(clicked()),
            this, SLOT(pressTitleRemoveButton()));

    //Init tab keyword
    keyword = "";    
    wordCompleter = new QCompleter(wrd->allWords(), this);
    wordCompleter->setCaseSensitivity(Qt::CaseInsensitive);
    wordCompleter->setCompletionMode(QCompleter::PopupCompletion);
    wordEdit->setCompleter(wordCompleter);

    connect(wordEdit, SIGNAL(textChanged(QString)),
            this, SLOT(setModeWordPresentAndAbsentButton(QString)));   
    connect(wordPresentButton, SIGNAL(clicked()),
            this, SLOT(pressWordPresentButton()));
    connect(wordAbsentButton, SIGNAL(clicked()),
            this, SLOT(pressWordAbsentButton()));
    connect(wordPresentClearButton, SIGNAL(clicked()),
            this, SLOT(pressWordPresentClearButton()));
    connect(wordAbsentClearButton, SIGNAL(clicked()),
            this, SLOT(pressWordAbsentClearButton()));
    connect(wordPresentRemoveButton, SIGNAL(clicked()),
            this, SLOT(pressWordPresentRemoveButton()));
    connect(wordAbsentRemoveButton, SIGNAL(clicked()),
            this, SLOT(pressWordAbsentRemoveButton()));
    connect(wordPresentList,SIGNAL(itemSelectionChanged()),
            this, SLOT(setModeWordPresentRemoveButton()));
    connect(wordAbsentList,SIGNAL(itemSelectionChanged()),
            this, SLOT(setModeWordAbsentRemoveButton()));
    connect(wordAddButton,SIGNAL(clicked()),
            this, SLOT(pressWordAddButton()));
    connect(wordList,SIGNAL(itemSelectionChanged()),
            this, SLOT(setModeWordRemoveButton()));
    connect(wordClearButton, SIGNAL(clicked()),
            this, SLOT(pressWordClearButton()));
    connect(wordRemoveButton, SIGNAL(clicked()),
            this, SLOT(pressWordRemoveButton()));

    //Init tab date
    date ="";
    setModeDateAddButton();

    dateFromGroupBox->setTitle(fromLabel + ": " + (fromCalendar->selectedDate()).toString(m_box->dateFormat()));
    dateBeforeGroupBox->setTitle(beforeLabel + ": " + (beforeCalendar->selectedDate()).toString(m_box->dateFormat()));

    connect(fromCalendar, SIGNAL(selectionChanged ()),
            this, SLOT(setTitleFromGroupBox()));
    connect(beforeCalendar, SIGNAL(selectionChanged ()),
            this, SLOT(setTitleBeforeGroupBox()));
    connect(dateFromGroupBox, SIGNAL(toggled(bool)),
            this, SLOT(setTitleFromGroupBox(bool)));
    connect(dateBeforeGroupBox, SIGNAL(toggled(bool)),
            this, SLOT(setTitleBeforeGroupBox(bool)));
    connect(dateClearButton, SIGNAL(clicked()),
            this, SLOT(pressDateClearButton()));
    connect(dateRemoveButton, SIGNAL(clicked()),
            this, SLOT(pressDateRemoveButton()));
    connect(dateAddButton,SIGNAL(clicked()),
            this, SLOT(pressDateAddButton()));
    connect(dateList,SIGNAL(itemSelectionChanged()),
            this, SLOT(setModeDateRemoveButton()));

    //Init print
    print = "";

    //Init tab category
    category = "";

    setWhichPossibleValue();

    connect(which_A, SIGNAL(currentIndexChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(which_B, SIGNAL(currentIndexChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(which_C, SIGNAL(currentIndexChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(which_D, SIGNAL(currentIndexChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(which_E, SIGNAL(currentIndexChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(which_F, SIGNAL(currentIndexChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(which_G, SIGNAL(currentIndexChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(which_H, SIGNAL(currentIndexChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(which_I, SIGNAL(currentIndexChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(where_A, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(where_B, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(where_B, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(daytime_A, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(daytime_B, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(daytime_C, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(daytime_D, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(season_A, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(season_B, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(season_C, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(season_D, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_A, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_B, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_C, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_D, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_E, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_F, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_G, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_H, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_I, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_J, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_K, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_L, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_M, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_N, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_O, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(picture_P, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(method_A, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(method_B, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));

    connect(printed, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));
    connect(notPrinted, SIGNAL(stateChanged(int)), this, SLOT(setModeStartButton(int)));

    remarkEdit->setFocus();

    setModeStartButton();

    connect(startButton, SIGNAL(clicked()), this, SLOT(pressStart()));
    connect(this, SIGNAL(searchOver()),dialog, SLOT(setModeSaveButton()));
    connect(this, SIGNAL(searchOver()),dialog, SLOT(setModeCreateStageButton()));
    connect(this, SIGNAL(searchOver()),dialog, SLOT(showStage()));
    connect(this, SIGNAL(searchOver()),dialog, SIGNAL(needUpdateStatus()));

    searchTab->installEventFilter(this);
    remarkEdit->installEventFilter(this);
    wordEdit->installEventFilter(this);
    titleEdit->installEventFilter(this);
    which_A->installEventFilter(this);
    which_B->installEventFilter(this);
    which_C->installEventFilter(this);
    which_D->installEventFilter(this);
    which_E->installEventFilter(this);
    which_F->installEventFilter(this);
    which_G->installEventFilter(this);
    which_H->installEventFilter(this);
    which_I->installEventFilter(this);
}

SelectionFrame::~SelectionFrame()
{          
}

bool SelectionFrame::eventFilter(QObject *target, QEvent *event)
{
    if (target == remarkEdit ||
        target == wordEdit ||
        target == titleEdit ||
        target == which_A || target == which_B || target == which_C ||
        target == which_D || target == which_E || target == which_F ||
        target == which_G || target == which_H || target == which_I)
    {
        if (event->type() == QEvent::KeyPress) {

            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);

            if (target == which_A || target == which_B || target == which_C ||
                target == which_D || target == which_E || target == which_F ||
                target == which_G || target == which_H || target == which_I)
            {
                if (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter){
                    focusNextChild();
                }
                return false;
            }

            if (target == remarkEdit &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter))
            {
                remarkAddButton->click();
                return true;
            }

            if (target == wordEdit &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter)) {
                if (keyEvent->modifiers() & Qt::ShiftModifier) {
                    wordAbsentButton->click();
                } else {
                    wordPresentButton->click();
                }
                return true;
            }

            if (target == titleEdit &&
                (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter))
            {
                titleAddButton->click();
                return true;
            }            
        }
    }
    return QFrame::eventFilter(target, event);
}

bool SelectionFrame::event(QEvent *event)
{
    if (event->type() == QEvent::KeyPress) {        
        QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
        if (keyEvent->key() == Qt::Key_F1) {
            return true;
        }
    }
    return QFrame::event(event);
}

void SelectionFrame::keyPressEvent(QKeyEvent *event)
{
    int c = searchTab->count();
    int i = searchTab->currentIndex();

    Qt::KeyboardModifiers modi = event->modifiers();
    switch (event->key()){
    case Qt::Key_R:
        if (modi & Qt::ControlModifier){
            searchTab->setCurrentWidget(tab_A);
        }
        break;
    case Qt::Key_C:
        if (modi & Qt::ControlModifier){
            searchTab->setCurrentWidget(tab_B);
        }
        break;
    case Qt::Key_K:
        if (modi & Qt::ControlModifier){
            searchTab->setCurrentWidget(tab_C);
        }
        break;
    case Qt::Key_D:
        if (modi & Qt::ControlModifier){
            searchTab->setCurrentWidget(tab_D);
        }
        break;
    case Qt::Key_S:
        if (modi & Qt::ControlModifier){
            searchTab->setCurrentWidget(tab_E);
        }
        break;
    case Qt::Key_P:
        if (modi & Qt::ControlModifier){
            searchTab->setCurrentWidget(tab_F);
        }
        break;
    case Qt::Key_T:
        if (modi & Qt::ControlModifier){
            searchTab->setCurrentWidget(tab_G);
        }
        break;
    case Qt::Key_PageUp:
        if (i != (c-1)) { searchTab->setCurrentIndex(i+1); }
        break;
    case Qt::Key_PageDown:
        if (i > 0) { searchTab->setCurrentIndex(i-1); }
        break;
    case Qt::Key_Enter:
        break;
    case Qt::Key_Return:
        break;
    default:
        QWidget::keyPressEvent(event);
    }
}

void SelectionFrame::setValue()
{
    QStringList l;

    //Set remark
    l.clear();
    for (int i = 0; i < remarkList->count(); i++){
        l << remarkList->item(i)->text();
    }
    remark = l.join(",");
    if (!l.isEmpty() && remarkInvert->isChecked()) {remark.prepend(SIGN_INV);}

    //Set session

    session = m_sessionLst.join(",");  
    if (!m_sessionLst.isEmpty() && sessionInvert->isChecked()) {session.prepend(SIGN_INV);}

    //Set title&author
    l.clear();
    for (int i = 0; i < titleList->count(); i++){
        l << titleList->item(i)->text();
    }
    title = l.join(",");
    if (!l.isEmpty() && titleInvert->isChecked()) {title.prepend(SIGN_INV);}

    //Set keyword
    l.clear();
    for (int i = 0; i < wordList->count(); i++){
        l << wrd->replaceWordToKey(wordList->item(i)->text());
    }
    keyword = l.join(",");
    if (!l.isEmpty() && wordInvert->isChecked()) {keyword.prepend(SIGN_INV);}

    //Set date
    l.clear();
    for (int i = 0; i < dateList->count(); i++){
        l << dateList->item(i)->text();
    }
    date = l.join(",");
    if (!l.isEmpty() && dateInvert->isChecked()) {date.prepend(SIGN_INV);}

    //Set print
    l.clear();
    if (notPrinted->isChecked()) {
        l << "0";
    }
    if (printed->isChecked()) {
        l << "1";
    }
    print = l.join(",");
    if (!l.isEmpty() && printInvert->isChecked()) {print.prepend(SIGN_INV);}

    //Set category
    l.clear();
    QString s,r;
    QComboBox* currentBox;
    QCheckBox* currentCheckBox;
    foreach(QObject* c, whichGroupBox->children()){
        if (c->objectName().left(5) != "label"){
            continue;
        }
        QLabel *currentLabel = qobject_cast<QLabel*>(c);
        s = currentLabel->text().toLower();
        currentBox = qobject_cast<QComboBox*>(currentLabel->buddy());
        switch (whichPossibleValue.indexOf(currentBox->currentText()))
        {
        case 1:
            r.append("+" + s);
            break;
        case 2:
            r.append("-" + s);
            break;
        case 3:
            r.append("+" + s.left(1).toUpper() + s.mid(1));
            break;
        case 4:
            r.append("-" + s.left(1).toUpper() + s.mid(1));
            break;
        case 0:;
        default:;
        }
    }
    l << r;
    QStringList p;
    foreach(QObject* c, whereGroupBox->children()){
        if (c->objectName().left(5) != "where"){
            continue;
        }
        currentCheckBox = qobject_cast<QCheckBox*>(c);
        if (currentCheckBox->isChecked()){
            p << currentCheckBox->text();
        }
    }
    l << p.join("|");
    p.clear();
    foreach(QObject* c, daytimeGroupBox->children()){
        if (c->objectName().left(7) != "daytime"){
            continue;
        }
        currentCheckBox = qobject_cast<QCheckBox*>(c);
        if (currentCheckBox->isChecked()){
            p << currentCheckBox->text();
        }
    }
    l << p.join("|");
    p.clear();
    foreach(QObject* c, seasonGroupBox->children()){
        if (c->objectName().left(6) != "season"){
            continue;
        }
        currentCheckBox = qobject_cast<QCheckBox*>(c);
        if (currentCheckBox->isChecked()){
            p << currentCheckBox->text();
        }
    }
    l << p.join("|");
    p.clear();
    foreach(QObject* c, pictureGroupBox->children()){
        if (c->objectName().left(7) != "picture"){
            continue;
        }
        currentCheckBox = qobject_cast<QCheckBox*>(c);
        if (currentCheckBox->isChecked()){
            p << currentCheckBox->text();
        }
    }
    l << p.join("|");
    p.clear();
    foreach(QObject* c, methodGroupBox->children()){
        if (c->objectName().left(6) != "method"){
            continue;
        }
        currentCheckBox = qobject_cast<QCheckBox*>(c);
        if (currentCheckBox->isChecked()){
            p << currentCheckBox->text();
        }
    }
    l << p.join("|");
    while (!l.isEmpty()){
        if (l.last().isEmpty()){
            l.takeLast();
        } else {
            break;
        }
    }
    category = l.join(",");
    if (!l.isEmpty() && categoryInvert->isChecked()) { category.prepend(SIGN_INV); }
}

//Tab date
void SelectionFrame::setTitleFromGroupBox(bool v)
{
    if (v) {
        dateFromGroupBox->setTitle(fromLabel + ": " + fromCalendar->selectedDate().toString(m_box->dateFormat()));
    } else {
        dateFromGroupBox->setTitle(fromLabel + ": ...");
    }
    setModeDateAddButton();
}
void SelectionFrame::setTitleBeforeGroupBox(bool v)
{
    if (v) {
        dateBeforeGroupBox->setTitle(beforeLabel + ": " + beforeCalendar->selectedDate().toString(m_box->dateFormat()));
    } else {
        dateBeforeGroupBox->setTitle(beforeLabel + ": ...");
    }
    setModeDateAddButton();
}
void SelectionFrame::setTitleFromGroupBox()
{
    QDate a,b;
    a = fromCalendar->selectedDate();
    b = beforeCalendar->selectedDate();
    if (a > b) {
        beforeCalendar->setSelectedDate(a);
    }
    dateFromGroupBox->setTitle(fromLabel + ": " + a.toString(m_box->dateFormat()));
    setModeDateAddButton();
}
void SelectionFrame::setTitleBeforeGroupBox()
{
    QDate a,b;
    a = fromCalendar->selectedDate();
    b = beforeCalendar->selectedDate();
    if (a > b) {
        fromCalendar->setSelectedDate(b);
    }
    dateBeforeGroupBox->setTitle(beforeLabel + ": " + b.toString(m_box->dateFormat()));
    setModeDateAddButton();
}
void SelectionFrame::setModeDateClearButton()
{
    if (dateList->count() == 0){
        dateClearButton->setEnabled(false);        
        dateList->setEnabled(false);
    } else {
        dateClearButton->setEnabled(true);        
        dateList->setEnabled(true);
    }
}
void SelectionFrame::setModeDateRemoveButton()
{
    if (dateList->selectedItems().isEmpty()){
        dateRemoveButton->setEnabled(false);
    } else {
        dateRemoveButton->setEnabled(true);
    }
}
void SelectionFrame::setModeDateAddButton()
{
    if (dateList->findItems(makeDateRequest(),Qt::MatchFixedString).isEmpty() &&
        (dateFromGroupBox->isChecked() || dateBeforeGroupBox->isChecked()))
    {
        dateAddButton->setEnabled(true);
    } else {
        dateAddButton->setEnabled(false);
    }
}
void SelectionFrame::pressDateClearButton()
{
    dateList->clear();
    if (dateFromGroupBox->isChecked()){
        fromCalendar->setFocus();
    } else {
        dateFromGroupBox->setFocus();
    }    
    setModeDateClearButton();
    setModeDateAddButton();
    setModeStartButton();
}
void SelectionFrame::pressDateRemoveButton()
{
    QList<QListWidgetItem*> l;
    l = dateList->selectedItems();
    while (!l.isEmpty()){
        dateList->takeItem(dateList->row(l.takeFirst()));
    }
    if (dateList->count() == 0) {
        if (dateFromGroupBox->isChecked()){
            fromCalendar->setFocus();
        } else {
            dateFromGroupBox->setFocus();
        }
    } else {
        dateList->setFocus();
    }
    setModeDateClearButton();
    setModeDateAddButton();
    setModeStartButton();
}
void SelectionFrame::pressDateAddButton()
{
    dateList->addItem(makeDateRequest());
    if (dateFromGroupBox->isChecked()){
        fromCalendar->setFocus();
    } else {
        dateFromGroupBox->setFocus();
    }    
    setModeDateClearButton();
    setModeDateAddButton();
    setModeStartButton();
}
QString SelectionFrame::makeDateRequest()
{
    QString s;    
    QString a,b;
    if (dateFromGroupBox->isChecked() && dateBeforeGroupBox->isChecked()){
        a = fromCalendar->selectedDate().toString(m_box->dateFormat());
        b = beforeCalendar->selectedDate().toString(m_box->dateFormat());
        if (a == b) {
            s = a;
        } else {
            s = a + "-" + b;
        }
    }
    if (dateFromGroupBox->isChecked() && !dateBeforeGroupBox->isChecked()){
        a = fromCalendar->selectedDate().toString(m_box->dateFormat());
        s = a + QString("-...");
    }
    if (!dateFromGroupBox->isChecked() && dateBeforeGroupBox->isChecked()){
        b = beforeCalendar->selectedDate().toString(m_box->dateFormat());
        s = QString("...-") + b;
    }
    return s;
}

//Tab keyword
void SelectionFrame::setModeWordPresentAndAbsentButton(QString s)
{
/*
    QStringList likeWrd;
    likeWrd = wrd->likeBegin(s);
    if (likeWrd.isEmpty()) {
        wordEdit->setToolTip("");
    } else {
        wordEdit->setToolTip(wrd->likeBegin(s).join(", "));
    }
*/

    QString k;
    k = wrd->getKey(s);    
    if (!s.isEmpty() && wrd->wordExist(s) && !keyPresentList.contains(k) && !keyAbsentList.contains(k)){
        wordPresentButton->setEnabled(true);
        wordAbsentButton->setEnabled(true);
    } else {
        wordPresentButton->setEnabled(false);
        wordAbsentButton->setEnabled(false);
    }
}
void SelectionFrame::setModeWordPresentClearButton()
{
    if (wordPresentList->count() == 0){
        wordPresentClearButton->setEnabled(false);        
        wordPresentList->setEnabled(false);
    } else {
        wordPresentClearButton->setEnabled(true);        
        wordPresentList->setEnabled(true);
    }
}
void SelectionFrame::setModeWordPresentRemoveButton()
{
    if (wordPresentList->selectedItems().isEmpty()){
        wordPresentRemoveButton->setEnabled(false);
    } else {
        wordPresentRemoveButton->setEnabled(true);
    }
}
void SelectionFrame::setModeWordAbsentClearButton()
{
    if (wordAbsentList->count() == 0){
        wordAbsentClearButton->setEnabled(false);        
        wordAbsentList->setEnabled(false);
    } else {
        wordAbsentClearButton->setEnabled(true);        
        wordAbsentList->setEnabled(true);
    }
}
void SelectionFrame::setModeWordAbsentRemoveButton()
{
    if (wordAbsentList->selectedItems().isEmpty()){
        wordAbsentRemoveButton->setEnabled(false);
    } else {
        wordAbsentRemoveButton->setEnabled(true);
    }
}
void SelectionFrame::setModeWordClearButton()
{
    if (wordList->count() == 0){
        wordClearButton->setEnabled(false);        
        wordList->setEnabled(false);
    } else {
        wordClearButton->setEnabled(true);        
        wordList->setEnabled(true);
    }
}
void SelectionFrame::setModeWordRemoveButton()
{
    if (wordList->selectedItems().isEmpty()){
        wordRemoveButton->setEnabled(false);
    } else {
        wordRemoveButton->setEnabled(true);
    }
}
void SelectionFrame::setModeWordAddButton()
{
    if ((wordPresentList->count() > 0 ||
        wordAbsentList->count() > 0) &&
        wordList->findItems(makeWordRequest(),Qt::MatchFixedString).isEmpty())
    {
        wordAddButton->setEnabled(true);
    } else {
        wordAddButton->setEnabled(false);
    }
}
void SelectionFrame::pressWordPresentButton()
{
    QString s;
    s = wordEdit->text().simplified();
    wordPresentList->addItem(s);
    keyPresentList.insert(wrd->getKey(s));
    wordEdit->clear();
    wordEdit->setFocus();    
    setModeWordPresentClearButton();
    setModeWordAddButton();
}
void SelectionFrame::pressWordAbsentButton()
{
    QString s;
    s = wordEdit->text().simplified();
    wordAbsentList->addItem(s);
    keyAbsentList.insert(wrd->getKey(s));
    wordEdit->clear();
    wordEdit->setFocus();    
    setModeWordAbsentClearButton();
    setModeWordAddButton();
}
void SelectionFrame::pressWordPresentClearButton()
{
    wordPresentList->clear();
    keyPresentList.clear();
    wordEdit->setFocus();    
    setModeWordPresentClearButton();
    setModeWordPresentAndAbsentButton(remarkEdit->text());
    setModeWordAddButton();
}
void SelectionFrame::pressWordPresentRemoveButton()
{
    QList<QListWidgetItem*> l;
    l = wordPresentList->selectedItems();
    int i;
    QListWidget q;
    while (!l.isEmpty()){
        i = wordPresentList->row(l.takeFirst());
        keyPresentList.remove(wrd->getKey(wordPresentList->item(i)->text()));
        wordPresentList->takeItem(i);
    }
    if (wordPresentList->count() == 0) {
        wordEdit->setFocus();
    } else {
        wordPresentList->setFocus();
    }
    setModeWordPresentClearButton();
    setModeWordPresentAndAbsentButton(wordEdit->text());
    setModeWordAddButton();
}
void SelectionFrame::pressWordAbsentClearButton()
{
    wordAbsentList->clear();
    keyAbsentList.clear();
    wordEdit->setFocus();    
    setModeWordAbsentClearButton();
    setModeWordPresentAndAbsentButton(remarkEdit->text());
    setModeWordAddButton();
}
void SelectionFrame::pressWordAbsentRemoveButton()
{
    QList<QListWidgetItem*> l;
    l = wordAbsentList->selectedItems();
    int i;
    while (!l.isEmpty()){
        i = wordAbsentList->row(l.takeFirst());
        keyAbsentList.remove(wrd->getKey(wordAbsentList->item(i)->text()));
        wordAbsentList->takeItem(i);
    }
    if (wordAbsentList->count() == 0) {
        wordEdit->setFocus();
    } else {
        wordAbsentList->setFocus();
    }
    setModeWordAbsentClearButton();
    setModeWordPresentAndAbsentButton(wordEdit->text());
    setModeWordAddButton();
}
void SelectionFrame::pressWordClearButton()
{
    wordList->clear();
    wordEdit->setFocus();    
    setModeWordClearButton();
    setModeWordAddButton();
    ;
}
void SelectionFrame::pressWordRemoveButton()
{
    QList<QListWidgetItem*> l;
    l = wordList->selectedItems();
    while (!l.isEmpty()){
        wordList->takeItem(wordList->row(l.takeFirst()));
    }
    if (wordList->count() == 0) {
        wordEdit->setFocus();
    } else {
        wordList->setFocus();
    }
    setModeWordClearButton();
    setModeWordAddButton();
    setModeStartButton();
}
void SelectionFrame::pressWordAddButton()
{
    wordList->addItem(makeWordRequest());
    wordPresentList->clear();
    keyPresentList.clear();
    wordAbsentList->clear();
    keyAbsentList.clear();
    wordEdit->setFocus();    
    setModeWordClearButton();
    setModeWordPresentClearButton();
    setModeWordAbsentClearButton();
    wordAddButton->setEnabled(false);
    setModeStartButton();
}
QString SelectionFrame::makeWordRequest()
{
    QStringList l;
    QString s;
    for (int r = 0; r < wordPresentList->count(); r++){
        l << wordPresentList->item(r)->text();
    }
    if (!l.isEmpty()){
        s = "+" + l.join("+");
    }
    l.clear();
    for (int r = 0; r < wordAbsentList->count(); r++){
        l << wordAbsentList->item(r)->text();
    }
    if (!l.isEmpty()){
        s += "-" + l.join("-");
    }
    return s;
}
//remark
void SelectionFrame::pressRemarkRemoveButton()
{
    QList<QListWidgetItem*> l;
    l = remarkList->selectedItems();
    while (!l.isEmpty()){
        remarkList->takeItem(remarkList->row(l.takeFirst()));
    }
    if (remarkList->count() == 0) {
        remarkEdit->setFocus();
    } else {
        remarkList->setFocus();
    }
    setModeRemarkClearButton();
    setModeRemarkAddButton(remarkEdit->text());
    setModeStartButton();
}

void SelectionFrame::pressRemarkClearButton()
{
    remarkList->clear();
    remarkEdit->setFocus();    
    setModeRemarkClearButton();
    setModeRemarkAddButton(remarkEdit->text());
    setModeStartButton();
}

void SelectionFrame::setModeRemarkAddButton(QString s)
{
    if (s.length() == 6 && (remarkList->findItems(s,Qt::MatchFixedString)).isEmpty()) {
        remarkAddButton->setEnabled(true);
    } else {
        remarkAddButton->setEnabled(false);
    }
}

void SelectionFrame::setModeRemarkRemoveButton()
{
    if (remarkList->selectedItems().isEmpty()){
        remarkRemoveButton->setEnabled(false);
    } else {
        remarkRemoveButton->setEnabled(true);
    }
}

void SelectionFrame::setModeRemarkClearButton()
{
    if (remarkList->count() == 0){
        remarkClearButton->setEnabled(false);
        remarkList->setEnabled(false);
    } else {
        remarkClearButton->setEnabled(true);
        remarkList->setEnabled(true);
    }    
}

void SelectionFrame::pressRemarkAddButton()
{
    remarkList->addItem(remarkEdit->text());
    remarkEdit->clear();
    remarkEdit->setFocus();    
    setModeRemarkClearButton();
    setModeStartButton();
}

//session
void SelectionFrame::pressSessionClearButton()
{    
    sessionTable->clearSelection();
    sessionTable->setFocus();

    setModeSessionClearButton();    
    setModeStartButton();
}

void SelectionFrame::setModeSessionClearButton()
{
    if(m_sessionLst.isEmpty()){
        sessionClearButton->setEnabled(false);        
    } else {
        sessionClearButton->setEnabled(true);        
    }
}

//title
void SelectionFrame::pressTitleRemoveButton()
{
    QList<QListWidgetItem*> l;
    l = titleList->selectedItems();
    while (!l.isEmpty()){
        titleList->takeItem(titleList->row(l.takeFirst()));
    }
    if (titleList->count() == 0) {
        titleEdit->setFocus();
    } else {
        titleList->setFocus();
    }
    setModeTitleClearButton();
    setModeTitleAddButton(titleEdit->text());
    setModeStartButton();
}

void SelectionFrame::pressTitleClearButton()
{
    titleList->clear();
    titleEdit->setFocus();    
    setModeTitleClearButton();
    setModeTitleAddButton(titleEdit->text());
    setModeStartButton();
}

void SelectionFrame::setModeTitleAddButton(QString s)
{
    if (!(titleEdit->text().isEmpty()) &&
        titleList->findItems(s.simplified(),Qt::MatchFixedString).isEmpty())
    {
        titleAddButton->setEnabled(true);
    } else {
        titleAddButton->setEnabled(false);
    }
}

void SelectionFrame::setModeTitleRemoveButton()
{
    if (titleList->selectedItems().isEmpty()){
        titleRemoveButton->setEnabled(false);
    } else {
        titleRemoveButton->setEnabled(true);
    }
}

void SelectionFrame::setModeTitleClearButton()
{
    if (titleList->count() == 0){
        titleClearButton->setEnabled(false);
        titleList->setEnabled(false);
    } else {
        titleClearButton->setEnabled(true);
        titleList->setEnabled(true);

    }
}
void SelectionFrame::pressTitleAddButton()
{
    titleList->addItem(titleEdit->text().simplified());
    titleEdit->clear();
    titleEdit->setFocus();    
    setModeTitleClearButton();
    setModeStartButton();
}

void SelectionFrame::pressStart()
{
    setValue();

    MyTmpFile::task step;
    step.remark = remark;
    step.category = category;
    step.keyword = keyword;
    step.date = date;
    step.session = session;
    step.print = print;
    step.title = title;
    step.addTask = false;

    QString remarkWithAccountOfResemble;
    remarkWithAccountOfResemble = remark;
    for (int i = 0; i < remarkList->count(); i++){
        QString s;
        s = remarkList->item(i)->text();
        remarkWithAccountOfResemble.replace(s, remarkToResemble(s));
    }

    Selector s(remarkWithAccountOfResemble, category, keyword, date, session, print, title, m_box->category(), m_box->dateFormat());

    int currentStageIndex;
    currentStageIndex = this->stageNumber();

    m_box->memorizeMarked();
    if (firstStep){
        m_box->markAllEntries(this->stageNumber());
    }

    // .oO0
    int remarkCount = 1;
    remarkFound = 0;
    foundRecords->setText(QString::number(remarkFound)); 
    progressBar->setMaximum(m_box->markedEntriesCount(currentStageIndex));
    progressBar->setValue(0);
    // 0Oo.

    QApplication::setOverrideCursor(Qt::WaitCursor);

    foreach(MyAbcFile* abcFile, m_box->markedAbcFiles(currentStageIndex)){
        foreach(Entry* entry, abcFile->markedEntries(currentStageIndex)){
            if (s.match(entry)){
                remarkFound++;
                foundRecords->setText(QString::number(remarkFound));     
            } else {
                abcFile->unmarkEntry(entry, currentStageIndex);
            }
            // .oO0
            progressBar->setValue(remarkCount);
            remarkCount++;
            // 0Oo.
        }
    }

    QApplication::restoreOverrideCursor();

    QStringList l;

    for (int i = 0; i < wordList->count(); i++){
        l << wordList->item(i)->text();
    }
    keyword = l.join(",");
    if (!l.isEmpty() && wordInvert->isChecked()) {keyword.prepend(SIGN_INV);}
    step.keyword = keyword;

    if (remarkFound > 0) {
        steps.append(step);
        firstStep = false;

        emit searchOver();

    } else {

        m_box->restoreMarked();

        if (!firstStep){
            QMessageBox msgBox;
            msgBox.setWindowTitle(tr("Warning"));
            msgBox.setIcon(QMessageBox::Information);
            msgBox.setText(tr("Return previous result, because of search list is empty..."));
            msgBox.exec();

            remarkFound = m_box->markedEntriesCount(currentStageIndex);

            //.oO0
            foundRecords->setText(QString::number(remarkFound));
            //0Oo.

            emit searchOver();
        }
    }
}

MyTmpFile::tasks SelectionFrame::getSteps()
{
    return steps;
}

quint32 SelectionFrame::getRemarkFoundCount()
{
    return remarkFound;
}

void SelectionFrame::setModeStartButton()
{
    bool result;

    result = true;

    result = !setHeaderRemark() && result;
    result = !setHeaderCategory() && result;
    result = !setHeaderKeyword() && result;
    result = !setHeaderDate() && result;
    result = !setHeaderSession() && result;
    result = !setHeaderPrint() && result;
    result = !setHeaderTitle() && result;

    if (result){
        startButton->setDisabled(true);
    } else {
        startButton->setEnabled(true);
    }
}

void SelectionFrame::setModeStartButton(int i)
{
    setModeStartButton();
}

bool SelectionFrame::setHeaderRemark()
{
    bool result = true;

    if(remarkList->count() == 0){
        result = false;
    }
    if (result){        
        searchTab->setTabText(searchTab->indexOf(tab_A),tabHdrRemark + " " + modifySymbol);
    } else {        
        searchTab->setTabText(searchTab->indexOf(tab_A),tabHdrRemark);
    }
    return result;
}

bool SelectionFrame::setHeaderCategory()
{
    bool result = true;

    if((which_A->currentIndex() == 0) &&
       (which_B->currentIndex() == 0) &&
       (which_C->currentIndex() == 0) &&
       (which_D->currentIndex() == 0) &&
       (which_E->currentIndex() == 0) &&
       (which_F->currentIndex() == 0) &&
       (which_G->currentIndex() == 0) &&
       (which_H->currentIndex() == 0) &&
       (which_I->currentIndex() == 0) &&
       (!where_A->isChecked()) && (!where_B->isChecked()) &&
       (!daytime_A->isChecked()) && (!daytime_B->isChecked()) && (!daytime_C->isChecked()) && (!daytime_D->isChecked()) &&
       (!season_A->isChecked()) && (!season_B->isChecked()) && (!season_C->isChecked()) && (!season_D->isChecked()) &&
       (!method_A->isChecked()) && (!method_B->isChecked()) &&
       (!picture_A->isChecked()) && (!picture_B->isChecked()) && (!picture_C->isChecked()) && (!picture_D->isChecked()) &&
       (!picture_E->isChecked()) && (!picture_F->isChecked()) && (!picture_G->isChecked()) && (!picture_H->isChecked()) &&
       (!picture_I->isChecked()) && (!picture_J->isChecked()) && (!picture_K->isChecked()) && (!picture_L->isChecked()) &&
       (!picture_M->isChecked()) && (!picture_N->isChecked()) && (!picture_O->isChecked()) && (!picture_P->isChecked())){
        result = false;
    }
    if (result){
        searchTab->setTabText(searchTab->indexOf(tab_B),tabHdrCategory + " " + modifySymbol);
    } else {
        searchTab->setTabText(searchTab->indexOf(tab_B),tabHdrCategory);
    }
    return result;
}

bool SelectionFrame::setHeaderKeyword()
{
    bool result = true;

    if(wordList->count() == 0){
        result = false;
    }
    if (result){        
        searchTab->setTabText(searchTab->indexOf(tab_C),tabHdrKeyword + " " + modifySymbol);
    } else {        
        searchTab->setTabText(searchTab->indexOf(tab_C),tabHdrKeyword);
    }
    return result;
}

bool SelectionFrame::setHeaderDate()
{
    bool result = true;

    if(dateList->count() == 0){
        result = false;
    }
    if (result){        
        searchTab->setTabText(searchTab->indexOf(tab_D),tabHdrDate + " " + modifySymbol);
    } else {        
        searchTab->setTabText(searchTab->indexOf(tab_D),tabHdrDate);
    }
    return result;
}

bool SelectionFrame::setHeaderSession()
{
    bool result = true;

//    if(sessionList->count() == 0){
    if(m_sessionLst.count() == 0){
        result = false;
    }
    if (result){        
        searchTab->setTabText(searchTab->indexOf(tab_E),tabHdrSession + " " + modifySymbol);
    } else {        
        searchTab->setTabText(searchTab->indexOf(tab_E),tabHdrSession);
    }
    return result;
}

bool SelectionFrame::setHeaderPrint()
{
    bool result = true;

    if((!printed->isChecked()) &&
       (!notPrinted->isChecked())){
        result = false;
    }
    if (result){
        searchTab->setTabText(searchTab->indexOf(tab_F),tabHdrPrint + " " + modifySymbol);
    } else {
        searchTab->setTabText(searchTab->indexOf(tab_F),tabHdrPrint);
    }
    return result;
}

bool SelectionFrame::setHeaderTitle()
{
    bool result = true;

    if(titleList->count() == 0){
        result = false;
    }
    if (result){       
        searchTab->setTabText(searchTab->indexOf(tab_G),tabHdrTitle + " " + modifySymbol);
    } else {        
        searchTab->setTabText(searchTab->indexOf(tab_G),tabHdrTitle);
    }
    return result;
}

int SelectionFrame::stageNumber()
{
    int result;    
    result = objectName().remove(QString("Stage_")).toInt() - 1; 
    return result;
}

void SelectionFrame::setModeSessionClearAnnFilterButton()
{
    if (sessionAnnFilterEdit->text().isEmpty()){
        sessionClearAnnFilterButton->setDisabled(true);
    } else {
        sessionClearAnnFilterButton->setEnabled(true);
    }
}

void SelectionFrame::pressSessionClearAnnFilterButton()
{
    sessionAnnFilterEdit->clear();
    sessionAnnFilterEdit->setFocus();
}

void SelectionFrame::applySessionAnnFilter(QString s)
{
    QTableWidgetItem *i;

    for(int r = 0; r < sessionTable->rowCount(); r++)
    {
        i = sessionTable->item(r, ANNOTATION);

        QString str;
        str = i->text();

        if (str.indexOf(s.simplified(),0,Qt::CaseInsensitive) < 0){
            sessionTable->hideRow(r);
        } else {
            sessionTable->showRow(r);
        }
    }
}

void SelectionFrame::setSessionLst()
{
    QSet<QString> s;

    foreach(QTableWidgetItem *i, sessionTable->selectedItems()){
        if (i->column() != NO) { continue; }
        s << i->text();
    }

    m_sessionLst = s.toList();

    setModeSessionClearButton();
    setModeStartButton();
}

void SelectionFrame::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
        retranslateCategory();
        setTabTitles();
        setWhichPossibleValue();
        retranslateDateGroupBoxTitle();
    }
    QFrame::changeEvent(event);
}

void SelectionFrame::retranslateCategory()
{
    m_box->setCategoryLabels();
    Category cat;
    cat = m_box->category();
    Category::subCategory subCat;

    subCat = cat.subCategory_A();
    if (subCat.use) {
        whichGroupBox->setTitle(subCat.title);
        label_A->setText(subCat.contents.at(0).name);
        label_B->setText(subCat.contents.at(1).name);
        label_C->setText(subCat.contents.at(2).name);
        label_D->setText(subCat.contents.at(3).name);
        label_E->setText(subCat.contents.at(4).name);
        label_F->setText(subCat.contents.at(5).name);
        label_G->setText(subCat.contents.at(6).name);
        label_H->setText(subCat.contents.at(7).name);
        label_I->setText(subCat.contents.at(8).name);
    } else {
        whichGroupBox->hide();
    }

    subCat = cat.subCategory_C();
    if (subCat.use) {
        whereGroupBox->setTitle(subCat.title);
        where_A->setText(subCat.contents.at(0).name);
        where_B->setText(subCat.contents.at(1).name);
    } else {
        whereGroupBox->hide();
    }

    subCat = cat.subCategory_G();
    if (subCat.use) {
        methodGroupBox->setTitle(subCat.title);
        method_A->setText(subCat.contents.at(0).name);
        method_B->setText(subCat.contents.at(1).name);
    } else {
        methodGroupBox->hide();
    }

    subCat = cat.subCategory_D();
    if (subCat.use) {
        daytimeGroupBox->setTitle(subCat.title);
        daytime_A->setText(subCat.contents.at(0).name);
        daytime_B->setText(subCat.contents.at(1).name);
        daytime_C->setText(subCat.contents.at(2).name);
        daytime_D->setText(subCat.contents.at(3).name);
    } else {
        daytimeGroupBox->hide();
    }

    subCat = cat.subCategory_E();
    if (subCat.use) {
        seasonGroupBox->setTitle(subCat.title);
        season_A->setText(subCat.contents.at(0).name);
        season_B->setText(subCat.contents.at(1).name);
        season_C->setText(subCat.contents.at(2).name);
        season_D->setText(subCat.contents.at(3).name);
    } else {
        seasonGroupBox->hide();
    }

    subCat = cat.subCategory_F();
    if (subCat.use) {
        pictureGroupBox->setTitle(subCat.title);
        picture_A->setText(subCat.contents.at(0).name);
        picture_B->setText(subCat.contents.at(1).name);
        picture_C->setText(subCat.contents.at(2).name);
        picture_D->setText(subCat.contents.at(3).name);
        picture_E->setText(subCat.contents.at(4).name);
        picture_F->setText(subCat.contents.at(5).name);
        picture_G->setText(subCat.contents.at(6).name);
        picture_H->setText(subCat.contents.at(7).name);
        picture_I->setText(subCat.contents.at(8).name);
        picture_J->setText(subCat.contents.at(9).name);
        picture_K->setText(subCat.contents.at(10).name);
        picture_L->setText(subCat.contents.at(11).name);
        picture_M->setText(subCat.contents.at(12).name);
        picture_N->setText(subCat.contents.at(13).name);
        picture_O->setText(subCat.contents.at(14).name);
        picture_P->setText(subCat.contents.at(15).name);
    } else {
        pictureGroupBox->hide();
    }
}

void SelectionFrame::setTabTitles()
{
    tabHdrRemark = searchTab->tabText(searchTab->indexOf(tab_A));
    tabHdrCategory = searchTab->tabText(searchTab->indexOf(tab_B));
    tabHdrKeyword = searchTab->tabText(searchTab->indexOf(tab_C));
    tabHdrDate = searchTab->tabText(searchTab->indexOf(tab_D));
    tabHdrSession = searchTab->tabText(searchTab->indexOf(tab_E));
    tabHdrPrint = searchTab->tabText(searchTab->indexOf(tab_F));
    tabHdrTitle = searchTab->tabText(searchTab->indexOf(tab_G));
}

void SelectionFrame::setWhichPossibleValue()
{
    which_A->clear();
    which_B->clear();
    which_C->clear();
    which_D->clear();
    which_E->clear();
    which_F->clear();
    which_G->clear();
    which_H->clear();
    which_I->clear();

    whichPossibleValue.clear();

    whichPossibleValue << tr("Ignore") << tr("Present") << tr("Absent") << tr("Present and major") << tr("Present, but not major");

    which_A->addItems(whichPossibleValue);
    which_B->addItems(whichPossibleValue);
    which_C->addItems(whichPossibleValue);
    which_D->addItems(whichPossibleValue);
    which_E->addItems(whichPossibleValue);
    which_F->addItems(whichPossibleValue);
    which_G->addItems(whichPossibleValue);
    which_H->addItems(whichPossibleValue);
    which_I->addItems(whichPossibleValue);
}

void SelectionFrame::retranslateDateGroupBoxTitle()
{
    fromLabel = tr("&From");
    beforeLabel = tr("&Before");

    QString str;
    int i;
    str = dateFromGroupBox->title();
    i = str.indexOf(":");
    str.replace(0, i, fromLabel);
    dateFromGroupBox->setTitle(str);

    str = dateBeforeGroupBox->title();
    i = str.indexOf(":");
    str.replace(0, i, beforeLabel);
    dateBeforeGroupBox->setTitle(str);
}

QString SelectionFrame::remarkToResemble(const QString &r)
{
    QString result = r;

    MyAbcFile* abcFile;
    Remark remark(r);

    abcFile = m_box->findChild<MyAbcFile*>(NAME_ABC_FILE + "_" + remark.rema());

    if (abcFile) {
        QString s;
        s = abcFile->getEntryPart(remark.rk(), RESEMBLE);
        if (!s.isEmpty()){
            result = s;
        }
    }

    return result;
}
