#ifndef SELECTOR_H
#define SELECTOR_H

#include <QMap>
#include <QDate>
#include <QString>
#include <QStringList>

#include "category.h"
#include "entry.h"

/*
   Селектор
*/

class Selector
{
    // Заданные критерии поиска

    QString remark;             // ремарки, перечисленные через запятую, в начале может стоять символ инверсии
    QString session;            // номера сессий, перечисленные через запятую, в начале может быть символ инверсии
    QString title;              // слова, перечисленные через запятую, в начале может быть символ инверсии
    QString keyword;            // слова, перечисленные через запятую, в начале может быть символ инверсии
    QString date;               // даты или диапазоны дат, перечисленные чере запятую, в начале может быть символ инверсии
    QString print;              // числа, перечисленные через запятую, в начале может быть символ инверсии
    QString category;           // единственная категория, запятыми отделены поля этой категории,
                                // в начале может быть символ инверсии

    QString regExpDropCaps;     // регулярное выражение полученное из поля 'B' заданной категории

    // Заданные критерии поиска, но уже в виде списков и без символов инверсии

    QStringList remarkParts;    // список ремарок
    QStringList sessionParts;   // список сессий
    QStringList titleParts;     // список названий или авторов
    QStringList keywordParts;   // список ключевых слов
    QStringList dateParts;      // список дат или диапазонов
    QStringList printParts;     // напечатано / не напечатано
    QStringList categoryParts;  // список полей заданной категории

    bool iRemark, iCategory, iKeyword, iDate, iSession, iPrint, iTitle; // признаки инверсии критериев

    QString dateFormat;         // формат дат, в котором задаются диапазоны

    // Связь имен отдельных битов категории с их "весом"

    QMap<QString, quint32> aa;  // поле A (which)
    QMap<QString, quint32> bb;  // поле C (where)
    QMap<QString, quint32> cc;  // поле D (daytime)
    QMap<QString, quint32> dd;  // поле E(season)
    QMap<QString, quint32> ee;  // поле F (picture)
    QMap<QString, quint32> ff;  // полу G (method)

    enum {RANGE, BEFORE, FROM, AT}; // виды диапазонов дат: A - B, ... - B, A - ..., A

    bool check_category(quint32);   // сравнение категории, заданной числом, с критерием

    struct dt_template{             // диапазон дат
        QDate date_A;               // начало диапазона
        QDate date_B;               // конец диапазона
        int mode;                   // тип диапазона (RANGE, BEFORE, FROM, AT)
    };

    QList<dt_template> cmpDateParts; // Критерий поиска по дате в виде списка
                                     // это dateParts, в котором удалены некорректные диапазоны

    // Параметры функций cmp_...() - это поля записей как они выглядят в Abc-файле

    bool cmp_remark(QString);   // сравнение ремарки, заданной строкой, с критерием
    bool cmp_category(QString); // сравнение категории, заданной числом в виде строки, с критерием
    bool cmp_session(QString);  // сравнение сессии, заданной строкой, с критерием
    bool cmp_title(QString);    // сравнение слова с критерием title
    bool cmp_date(QString);     // сравнение даты, заданой строкой, с критерием
    bool cmp_print(QString);    // сравнение строки с критерием
    bool cmp_keyword(QString);  // сравнение ключевых слов с критерием

public:    
    Selector(QString, QString, QString, QString, QString, QString, QString, Category, QString);

    bool match(Entry*);         // Сравнение заданной записи с критериями

};

#endif // SELECTOR_H
