#include "templatedialog.h"

#include "selector.h"
#include "remark.h"

#include <QDir>
#include <QFileInfo>
#include <QKeyEvent>
#include <QMessageBox>

TemplateDialog::TemplateDialog(AnyBox *b, QFrame* w) : SelectDialog(b, w)
{   
    if (boxIsEmpty) { return; }

    wrd = static_cast<MyWrdFile*>(m_box->unit(NAME_WRD_FILE));
    tmpFile = static_cast<MyTmpFile*>(m_box->unit(NAME_TMP_FILE));
    tmpFile->loadTemplates(m_box->objectName());

    if (tmpFile->templatesCount(m_box->objectName()) == 0){
        // .oO0
        this->label_A->setHidden(true);
        this->label_B->setVisible(true);
        // 0Oo.
    } else {
        // .oO0        
        label_A->setHidden(true);
        label_C->setVisible(true);
        onlySelected->setVisible(false);
        templateList->setVisible(true);
        label_D->setVisible(true);
        templateBrowser->setVisible(true);
        spring->setHidden(true);
        mode->setVisible(true);
        templateList->setFocus();
        // 0Oo.

        QMap<QString,QString>* d;
        d = tmpFile->describes();
        templateName = d->keys();
        foreach(QString tmp, templateName){            
            templateList->addItem(tmp + "\t\t" + d->value(tmp));
        }

        // .oO0
        templateList->setCurrentRow(0);
        // 0Oo.        

        templateList->installEventFilter(this);
        templateBrowser->installEventFilter(this);

        connect(templateList,SIGNAL(currentRowChanged(int)),this,SLOT(changeTemplate(int)));

        retranslateThis();
    }    
}

bool TemplateDialog::eventFilter(QObject *target, QEvent *event)
{
    bool result;
    result = false;

    if (target == templateList ||
        target == templateBrowser){
        if (event->type() == QEvent::KeyPress) {
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
            if (target == templateList) {
                switch (keyEvent->key()){
                case Qt::Key_Up:
                    if (templateList->currentRow() == 0){
                        result = true;
                    }
                    break;
                case Qt::Key_Down:
                    if (templateList->currentRow() == templateList->count() - 1){
                        result = true;
                    }
                    break;
                case Qt::Key_Enter:
                case Qt::Key_Return:
                case Qt::Key_Space:
                    startButton->click();
                    result = true;
                    break;
                default:;
                }
            }
            if (target == templateBrowser) {
                switch (keyEvent->key()){
                case Qt::Key_Enter:
                case Qt::Key_Return:
                case Qt::Key_Space:
                    startButton->click();
                    result = true;
                    break;
                default:;
                }
            }
        }
    }
    return result;
}

void TemplateDialog::pressStartButton()
{    
    QApplication::setOverrideCursor(Qt::WaitCursor);

    m_box->unmarkAllEntries();

    foundRecordsCount = 0;
    m_box->markAllEntries(0);

    //.oO0
    startButton->setDisabled(true);
    //0Oo.

    int c;
    c = templateList->currentRow();

    MyTmpFile::tasks t;
    t = tmpFile->getTask(templateName.at(c));

    int remarkCount = 0;

    // .oO0
    foundRecords->setText("0");
    searchProgress->setMaximum(m_box->markedEntriesCount() * t.count());
    searchProgress->setValue(0);
    // 0Oo.   

    int subset = 0;

    foreach(MyTmpFile::task i, t){
        if (i.addTask) {
            foundRecords->setText(QString::number(m_box->markedEntriesCount(subset)));
            subset++;
            m_box->markAllEntries(subset);
        }

        QString remarkWithAccountOfResemble;
        remarkWithAccountOfResemble = i.remark;
        foreach(QString s, i.remark.split(",")){
            s.remove(SIGN_INV);
            remarkWithAccountOfResemble.replace(s, remarkToResemble(s));
        }

        QStringList lstTemp;
        foreach(QString tmp, i.keyword.split(",")){
            if (tmp.isEmpty()) { continue; }
            lstTemp << wrd->replaceWordToKey(tmp);
        }        
        i.keyword = lstTemp.join(",");        

        Selector s(remarkWithAccountOfResemble, i.category, i.keyword, i.date, i.session, i.print, i.title,
                   m_box->category(), m_box->dateFormat());

        searchProgress->setMaximum(searchProgress->maximum()-(m_box->getCount()-m_box->markedEntriesCount(subset)));

        foreach(MyAbcFile* abcFile, m_box->markedAbcFiles(subset)){
             foreach(Entry* entry, abcFile->markedEntries(subset)){
                 if (!s.match(entry)){
                     abcFile->unmarkEntry(entry, subset);
                 }
                 // .oO0
                 remarkCount++;
                 searchProgress->setValue(remarkCount);
                 // 0Oo.
             }
         }        
    }   

    foundRecordsCount = m_box->markedEntriesCount();

    // .oO0
    foundRecords->setText(QString::number(foundRecordsCount));
    templateList->setFocus();
    // 0Oo.

    QApplication::restoreOverrideCursor();

    if (foundRecordsCount > MAX_NUM_OF_SHOW){
        QMessageBox::information(this,tr("Warning"), tr("Selected list is very big. The output will be truncated..."));
    }

    m_box->makePlayListFile(templateName.at(c));
}

void TemplateDialog::changeTemplate(int c)
{    
    //.oO0
    searchProgress->setValue(0);
    foundRecords->setText("0");
    //0Oo.

    bool ok;
    ok = true;
    templateBrowser->setHtml(tmpFile->viewTask(templateName.at(c),&ok));
    startButton->setEnabled(ok);
}
void TemplateDialog::retranslateThis()
{
    title->setText(tr("Template"));
    m_box->setCategoryLabels();
    int r = templateList->currentRow();
    if (r > -1) {
        changeTemplate(templateList->currentRow());
    }
}

QString TemplateDialog::remarkToResemble(const QString &r)
{
    QString result = r;

    MyAbcFile* abcFile;
    Remark remark(r);

    abcFile = m_box->findChild<MyAbcFile*>(NAME_ABC_FILE + "_" + remark.rema());

    if (abcFile) {
        QString s;
        s = abcFile->getEntryPart(remark.rk(), RESEMBLE);
        if (!s.isEmpty()){
            result = s;
        }
    }

    return result;
}

