#include "verifydialog.h"
#include "remark.h"
#include "global.h"
#include "altqfile.h"

#include <QFile>
#include <QDir>
#include <QCryptographicHash>
#include <QTextStream>

//VerifyDialog::VerifyDialog(AnyBox* b, QWidget* w) : QFrame(w)
VerifyDialog::VerifyDialog(AnyBox* b, QFrame* w) : QFrame(w)
{           
    m_box = b;
    //setupUi(this);
    setupUi(w);

    boxPixmap->setPixmap(m_box->boxPixmap());

    QApplication::setOverrideCursor(Qt::WaitCursor);

    setStatusNames();
    copyResultCode = PROCESS;

    m_base = static_cast<BaseFolder*>(m_box->baseUnit());

    sss = static_cast<MySssFile*>(m_box->unit(NAME_SSS_FILE));
    wrd = static_cast<MyWrdFile*>(m_box->unit(NAME_WRD_FILE));
    ttt = static_cast<MyTmpFile*>(m_box->unit(NAME_TMP_FILE));
    log = static_cast<MyLogFile*>(m_box->unit(NAME_LOG_FILE));    

    //.oO0
    label_F->hide();
    scanProgress->hide();
    mode->hide();
    label_B->hide();
    sssStatus->hide();
    sssStatusDetail->hide();
    label_C->hide();
    wrdStatus->hide();
    wrdStatusDetail->hide();
    label_D->hide();
    abcStatus->hide();
    abcProgress->hide();
    abcStatusDetail->hide();
    label_E->hide();
    fileStatus->hide();
    fileStatusDetail->hide();
    copyGroup->hide();
    backupFolder->setText(m_base->backupDir());
    scanProgress->setMinimum(0);
    scanProgress->setMaximum(totalFilesInBox() + 2);
    scanProgress->setValue(0);
    label_A->hide();
    label_F->show();
    scanProgress->show();    
    //0Oo.    

    QFile tttFile(ttt->completeName());
    filesToCopy.append(tttFile.fileName()); // tmp-файл будет в backup

    // wrd - file
    QFile wrdFile(wrd->completeName());

    QTextStream stw(&wrdFile);
    stw.setCodec("UTF-8");
    stw.setGenerateByteOrderMark(false);

    filesToCopy.append(wrdFile.fileName()); // wrd-файл будет в backup

    if (wrdFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        QRegExp exp(ABC_WRD);
        QString l;
        QStringList w;       
        while (!stw.atEnd()){
            l = stw.readLine();
            w = l.split("~");
            QString k = w.takeFirst();            
            if (exp.exactMatch(k) && (!w.isEmpty())) {
                k = k.trimmed();
                if(words.contains(k)){
                    doubleWords.insert(k);
                } else {
                    words.insert(k);
                }
            }
        }
        words.insert("unknown");
        wrdFile.close();
    }

    //.oO0
    scanProgress->setValue(scanProgress->value() + 1);
    //0Oo.

    // sss - file    
    QFile sssFile(sss->completeName());

    QTextStream sts(&sssFile);
    sts.setCodec("UTF-8");
    sts.setGenerateByteOrderMark(false);

    filesToCopy.append(sssFile.fileName()); // session-файл будет в backup

    if (sssFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        while (!sts.atEnd()){
            QStringList parts;
            parts = sts.readLine().split("~");
            if (parts.count() == 4){
                for (int i = 0; i < 4; i++){
                    parts[i] = parts[i].trimmed();
                }
                uint x = parts[MySssFile::NO].toUInt();
                if (x &&
                    !parts[MySssFile::CREATION_DATE].isEmpty() &&
                    !parts[MySssFile::TYPE].isEmpty()) {
                    if (sssNums.contains(x)) {
                        sssDoubleNums.insert(x);
                    }
                    sssNums.insert(x);
                }
            }
        }
        sssFile.close();
    }

    //.oO0
    scanProgress->setValue(scanProgress->value() + 1);
    //0Oo.

    // abc - files
    QString abcFilter;
    for (int i = ABC_WIDTH; i > 0; i--){ abcFilter += ABC_CHAR; }

    QDir sa(m_box->completeName());
    QFileInfoList a = sa.entryInfoList(QStringList(abcFilter),QDir::Dirs);

    if (!a.isEmpty()) {
        foreach(QFileInfo tmpA, a){
            QDir sb(tmpA.absoluteFilePath());
            QFileInfoList b = sb.entryInfoList(QStringList(abcFilter),QDir::Dirs);
            foreach(QFileInfo tmpB, b) {

                QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);

                QDir sc(tmpB.absoluteFilePath());
                QFileInfoList c = sc.entryInfoList(QDir::Files);

                if (!c.isEmpty()){

                    QFileInfo currentAbcFileInfo(sc,NAME_ABC_FILE);

                    QString abcPath;
                    abcPath = QDir::toNativeSeparators(currentAbcFileInfo.absoluteFilePath());

                    c.takeAt(c.indexOf(currentAbcFileInfo));

                    QFile currentAbcFile(abcPath);

                    QTextStream st(&currentAbcFile);
                    st.setCodec("UTF-8");
                    st.setGenerateByteOrderMark(false);

                    filesToCopy.append(abcPath);    // abc-файлы будут в backup

                    if (currentAbcFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
                        QMap<QString, Entry> e;
                        while (!st.atEnd()){
                            Entry parts(st.readLine());
                            if (parts.Ok()){

                                QString rm;
                                rm = parts.getField(REMARK);
                                uint ss;
                                ss = parts.getField(SESSION).toUInt();

                                if (!sssNums.contains(ss)){
                                    sssAbsent.insert(ss);
                                }

                                if (e.contains(rm)){
                                    doubleRemark.insert(rm);
                                } else {
                                    e[rm] = parts;
                                }
                            }
                            abcEntries[abcPath] = e;
                        }
                        currentAbcFile.close();
                    }

                    //.oO0
                    scanProgress->setValue(scanProgress->value() + 1);
                    //0Oo.

                    foreach (QFileInfo tmpC, c){
                        QString checkSum;
                        QCryptographicHash md5(QCryptographicHash::Md5);
                        QFile file(tmpC.absoluteFilePath());
                        if (file.open(QIODevice::ReadOnly)) {
                            md5.reset();
                            md5.addData(file.readAll());
                            file.close();
                            checkSum = md5.result().toHex();
                        } else {
                            checkSum = "";
                        }

                        QStringList pts;                        
                        pts = QDir::toNativeSeparators(tmpC.absoluteFilePath()).split(QDir::separator());
                        QString remarkFromName;
                        int cnt;
                        cnt = pts.count();
                        if (cnt >= 3) {
                            remarkFromName = pts.at(cnt - 3) +
                                             pts.at(cnt - 2) +
                                             pts.at(cnt - 1).split(".").at(0);
                        }

                        QMap<QString, Entry> tmpEntries;
                        tmpEntries = abcEntries[abcPath];
                        Entry tmpEntry;
                        tmpEntry = tmpEntries.value(remarkFromName);

                        if (m_box->useSubFile() && (tmpC.suffix() == SUB_FILE_EXT)){
                            if (tmpEntries.contains(remarkFromName)){
                                filesToCopy.append(file.fileName());
                            } else {
                                oddFiles[remarkFromName] = tmpC.absoluteFilePath();
                            }
                        } else {
                            if (checkSum != tmpEntry.getField(CHECKSUM)){
                                oddFiles[remarkFromName] = tmpC.absoluteFilePath();
                            } else {
                                files[remarkFromName] = checkSum;
                                filesToCopy.append(file.fileName());    // сами предметные файлы будут в backup
                            }
                        }

                        //.oO0
                        scanProgress->setValue(scanProgress->value() + 1);
                        //0Oo.
                    }
                }
            }
        }
    }

    QString ws;
    bool sssResult;

    //.oO0
    label_F->hide();
    scanProgress->hide();
    mode->show();
    copyGroup->show();
    label_B->show();
    sssStatus->setText(status_pr);
    sssStatusCode = PROCESS;
    sssStatus->show();
    //0Oo.

    sssResult = sssOk(sss->completeName(), &ws);
    if (sssResult){
        sssStatus->setText(status_ok);
        sssStatusCode = OK;
    } else {
        sssStatus->setText(status_er);
        sssStatusCode = ERROR_1;
        //.oO0
        sssStatusDetail->show();
        //0Oo.
    }
    //.oO0
    sssStatusDetail->setHtml(ws);
    //0Oo.

    ws.clear();
    bool wrdResult;

    //.oO0
    label_C->show();
    wrdStatus->setText(status_pr);
    wrdStatusCode = PROCESS;
    wrdStatus->show();
    //0Oo.

    wrdResult = wrdOk(wrd->completeName(), &ws);
    if (wrdResult){
        wrdStatus->setText(status_ok);
        wrdStatusCode = OK;
    } else {
        wrdStatus->setText(status_er);
        wrdStatusCode = ERROR_1;
        //.oO0
        wrdStatusDetail->show();
        //0Oo.
    }
    //.oO0
    wrdStatusDetail->setHtml(ws);
    //0Oo.


    ws.clear();
    bool abcResult = true;

    //.oO0
    label_D->show();
    abcStatus->setText(status_pr);
    abcStatusCode = PROCESS;
    abcStatus->show();
    abcProgress->setMinimum(0);
    abcProgress->setMaximum(abcEntries.count());
    abcProgress->setValue(0);
    abcProgress->show();
    //0Oo.

    foreach(QString tmp, abcEntries.keys()){
        QString v;
        abcResult = abcResult & abcOk(tmp, &v);
        ws.append(v);

        //.oO0
        abcProgress->setValue(abcProgress->value() + 1);
        //0Oo.
    }

    //.oO0
    abcProgress->hide();
    //0Oo.

    if (abcResult){
        abcStatus->setText(status_ok);
        abcStatusCode = OK;
    } else {
        abcStatus->setText(status_er);
        abcStatusCode = ERROR_1;
        //.oO0
        abcStatusDetail->show();
        //0Oo.
    }
    //.oO0
    abcStatusDetail->setHtml(ws);
    //0Oo.

    ws.clear();
    bool fileResult;

    //.oO0
    label_E->show();
    fileStatus->setText(status_pr);
    fileStatusCode = PROCESS;
    fileStatus->show();
    //0Oo.

    fileResult = fileOk(&ws);
    if (fileResult){
        fileStatus->setText(status_ok);
        fileStatusCode = OK;
    } else {
        fileStatus->setText(status_er);
        fileStatusCode = ERROR_1;
        //.oO0
        fileStatusDetail->show();
        //0Oo.
    }
    //.oO0
    fileStatusDetail->setHtml(ws);
    //0Oo.

    //.oO0
    if (sssResult & wrdResult & abcResult & fileResult){
        copyGroup->setEnabled(true);
        backupButton->setFocus();       
    }
    //0Oo.

    connect(backupButton, SIGNAL(clicked()), this, SLOT(pressBackupButton()));

    QApplication::restoreOverrideCursor();
}

bool VerifyDialog::wrdOk(QString filename, QString *message)
{
    bool result = false;

    // Список возможных ошибок:
    // a) неуникальность индексов в wrd-файле
    QString message_keywordDouble = tr("is nonunique") + "...";

    QString stat("%1" + filename + "%2" + "... %3");
    QString erLine("<tr> <td width = 10% align = right>%1: </td> <td> %2 </td> </tr>");

    if (doubleWords.isEmpty()){
        *message = stat.arg("","",status_ok);
        result = true;
    } else {
        *message = stat.arg("<b>","</b>",status_er);
        message->append("<table width = 100%>");
        foreach(QString tmpKey, doubleWords) {
            message->append(erLine.arg(tmpKey,message_keywordDouble));
        }
        message->append("</table>");
    }
    message->prepend("<p>");
    message->append("</p>");

    return result;
}

bool VerifyDialog::sssOk(QString filename,QString *message)
{
    bool result = false;

    // Список возможных ошибок:
    // a) сессия отсутствует в session-файле (хотя присутствует в abc-файлах)
    QString message_sessionAbsent = tr("is absent") + "...";
    // б) номер сессии не уникален
    QString message_sessionDouble = tr("is nonunique") + "...";

    QString stat("%1" + filename + "%2" + "... %3");
    QString erLine("<tr> <td width = 10% align = right>Session %1: </td> <td> %2 </td> </tr>");

    if (sssAbsent.isEmpty() && sssDoubleNums.isEmpty()) {
        *message = stat.arg("","",status_ok);
        result = true;
    } else {
        *message = stat.arg("<b>","</b>",status_er);

        message->append("<table width = 100%>");

        QList<uint> sssAbsentList;
        sssAbsentList = sssAbsent.toList();
        qSort(sssAbsentList.begin(), sssAbsentList.end());

        foreach(uint x, sssAbsentList){
            message->append(erLine.arg(QString::number(x),message_sessionAbsent));
        }

        foreach(uint x, sssDoubleNums) {            
            message->append(erLine.arg(QString::number(x),message_sessionDouble));
        }
        message->append("</table>");
    }
    message->prepend("<p>");
    message->append("</p>");

    return result;
}

bool VerifyDialog::abcOk(QString filename, QString *message)
{
    QApplication::setOverrideCursor(Qt::WaitCursor);

    bool result = true;

    // Список возможных ошибок:
    // а) поле "дата" в записи некорректна
    QString message_dateWrong = tr("date is wrong") + "...";
    // б) использование одинакового имени ремарки для разных записей
    QString message_multipleRemark = tr("multiple use") + "...";
    // в) используемая ремарка не имеет отношения к текущему abc-файлу
    QString message_belongRemark = tr("not belong") + "...";
    // г) предметный файл есть, но его контрольная сумма не такая, какая указана в abc-файле
    QString message_checksumWrong = tr("checksum wrong") + "...";
    // д) для записи в abc-файле отсутствует предметный файл
    QString message_fileAbsent = tr("linked file is absent") + "...";
    // е) нет ремарки, на которую ссылается поле "resemble"
    QString message_resembleWrong = tr("resembling remark is wrong") + "...";
    // ж) запись, на которую ссылается поле "remark", сама на что-то ссылается
    QString message_resembleResemble = tr("resembling remark is not a prime") + "...";
    // з) нет ключевого слова, указанного в записи
    QString message_keywordUnknown = tr("index of keyword '%1' is unknown") + "...";

    QString stat("%1" + filename + "%2" + "... %3");
    QString erLine("<tr> <td width = 10% align = right> %1: </td> <td> %2 </td> </tr>");
    QString erText;

    QStringList pts;    
    pts = filename.split(QDir::separator());
    QString rema;
    if (pts.count() >= 3) {
        rema = pts.at(pts.count() - 3) + pts.at(pts.count() - 2);
    }

    foreach(QString tmp, doubleRemark){
        erText.append(erLine.arg(tmp,message_multipleRemark));
        result = false;
    }

    foreach(Entry entry, abcEntries.value(filename).values()){

        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);

        QString r;
        r = entry.getField(REMARK);
        Remark currentRemark(r);

        if (doubleRemark.contains(r)) {
            continue;
        }
        if (rema != currentRemark.rema()){
            erText.append(erLine.arg(r,message_belongRemark));
            result = false;
            continue;
        }

        QString cs;
        cs = entry.getField(CHECKSUM);

        if (!files.contains(r) && !oddFiles.contains(r)){
            erText.append(erLine.arg(r,message_fileAbsent));
            result = false;
        } else {
            if (files.value(r) != cs){
                erText.append(erLine.arg(r,message_checksumWrong));
                result = false;
            }
        }
        QString d;
        d = entry.getField(DATEOFCREATION);
        if (!QDate::fromString(d, m_box->dateFormat()).isValid()){
            erText.append(erLine.arg(r,message_dateWrong));
            result = false;
        }

        QStringList kw;
        kw = entry.getField(KEYWORD).split(",");

        foreach(QString tmpWord, kw){
            if (!words.contains(tmpWord)){
                erText.append(erLine.arg(r,message_keywordUnknown.arg(tmpWord)));
                result = false;
            }
        }

        QString rs;
        rs = entry.getField(RESEMBLE);
        Remark currentResemble(rs);

        QString rFilename;
        pts[pts.count() - 3] = currentResemble.re();
        pts[pts.count() - 2] = currentResemble.ma();        
        rFilename = pts.join(QDir::separator());

        QMap<QString, Entry> tmpEntries;

        if(abcEntries.contains(rFilename)){
            tmpEntries = abcEntries.value(rFilename);
            Entry tmpEntry;
            if(tmpEntries.contains(rs)) {
                tmpEntry = tmpEntries.value(rs);
                if(tmpEntry.getField(REMARK) != tmpEntry.getField(RESEMBLE)){
                    erText.append(erLine.arg(r,message_resembleResemble));
                    result = false;
                }
            } else {
                erText.append(erLine.arg(r,message_resembleWrong));
                result = false;
            }
        } else {
            erText.append(erLine.arg(r,message_resembleWrong));
            result = false;
        }
    }

    if (result) {
        *message = stat.arg("","",status_ok);
    } else {
        *message = stat.arg("<b>","</b>",status_er);
        message->append("<table width = 100%>");
        message->append(erText);
        message->append("</table>");
    }
    message->prepend("<p>");
    message->append("</p>");

    QApplication::restoreOverrideCursor();

    return result;
}

bool VerifyDialog::fileOk(QString *message)
{
    if (oddFiles.isEmpty()) {
        *message = tr("All files is accurate") + "...";
        message->prepend("<p>");
        message->append("</p>");
        return true;
    }
    // Список возможных ошибок:
    // a) найден файл, не имеющий отношения к MyBox
    QString message_oddFile = tr("is odd") + "...";

    QMap<QString, QStringList> info;

    QString stat("<b>%1</b>... %2");
    QString erLine("<tr> <td width = 20% align = right> File <b>%1</b>: </td> <td> %2 </td> </tr>");

    foreach(QString tmp, oddFiles.values()){
        QFileInfo i(tmp);
        info[i.absolutePath()].append(i.fileName());
    }

    foreach(QString p, info.keys()){
        message->append(stat.arg(p,status_er));
        message->append("<table width = 100%>");
        foreach(QString n, info.value(p)) {
            message->append(erLine.arg(n,message_oddFile));
        }
        message->append("</table>");
    }

    message->prepend("<p>");
    message->append("</p>");

    return false;
}

int VerifyDialog::totalFilesInBox()
{
    QString abcFilter;
    for (int i = ABC_WIDTH; i > 0; i--){ abcFilter += ABC_CHAR; }

    int total = 0;

    QDir sa(m_box->completeName());
    QFileInfoList a = sa.entryInfoList(QStringList(abcFilter),QDir::Dirs) ;
    if (!a.isEmpty()) {
        foreach(QFileInfo tmpA, a){

            QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);

            QDir sb(tmpA.absoluteFilePath());
            QFileInfoList b = sb.entryInfoList(QStringList(abcFilter),QDir::Dirs);

            foreach(QFileInfo tmpB, b) {            
                QDir sc(tmpB.absoluteFilePath());
                QFileInfoList c = sc.entryInfoList(QDir::Files);
                total += c.count();                
            }
        }
    }
    return total;
}

void VerifyDialog::pressBackupButton()
{
    QApplication::setOverrideCursor(Qt::WaitCursor);

    log->openLog(QString("BACKUP"));
    *log << "    Backup box   : " + m_box->objectName();

    //.oO0
    backupButton->setDisabled(true);                // повторное копирование запрещено
    copyProgress->setMaximum(filesToCopy.count());
    //0Oo.

    QString boxFolderName;  // путь к MyBox
    boxFolderName = m_base->completeName();

    QDir cpyFolder(m_base->backupDir()); // каталог для размещения резервной копии

    // если каталога для копии не существует и попытка создания его не удалась,то ...
    if (!cpyFolder.exists() && !cpyFolder.mkdir(cpyFolder.absolutePath())){
        //.oO0
        copyResult->setText(tr("Do not create backup folder") + "..."); // ... сообщить и ...
        copyResultCode = ERROR_1;
        //0Oo.
        *log << "    Backup status: can not update...";
        log->closeLog();
        QApplication::restoreOverrideCursor();
        return;                                                // ... выйти
    }

    QString cpyFolderName;  // путь к каталогу для копии
    cpyFolderName = cpyFolder.absolutePath();   

    //.oO0
    label_GB->setText(tr("Folder") + ":");       // сперва сообщим, что
    copyResult->setText(tr("create") + "...");   // будем создавать каталоги
    //0Oo.

    int countFolder = 0;

    // перебор имен abc-файлов
    // (их количество соответствует количеству подкаталогов)
    foreach(QString abcFile, abcEntries.keys()){

        countFolder++;

        // новое имя abc-файла (в старом имени заменяется путь к MyBox на путь к каталогу для копии)
        QFileInfo copyInfo(abcFile.replace(boxFolderName,cpyFolderName,Qt::CaseInsensitive));

        //.oO0
        curFile->setText(QDir::toNativeSeparators(copyInfo.absolutePath())); // выводим имя каталога
        //0Oo.

        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);

        QDir copyPath(copyInfo.absolutePath()); // создаваемый каталог
        if (!copyPath.exists()) {               // если создаваемого каталога нет, то ...
            QString relName;                    // ... вычисляем имя подкаталога ...
            relName = copyInfo.absolutePath().replace(cpyFolderName,"");
            if (relName.startsWith('/')){
                relName = relName.mid(1);
            }
            if(!cpyFolder.mkpath(relName)){     // ... пытаемся создать этот подкаталог
                                                // если попытка создания неудачна, то ...
                //.oO0
                copyResult->setText(tr("do not create folder") + "...");    // ... сообщить и ...
                copyResultCode = ERROR_2;
                //0Oo.
                *log << "    Backup status: can not update...";
                log->closeLog();
                QApplication::restoreOverrideCursor();
                return;                                             // ... выйти
            }
        }
    }

    //.oO0
    label_GB->setText(tr("File:")); // сообщаем, что будем копировать файлы
    //0Oo.

    // упорядоченный по именам список файлов для копирования
    qSort(filesToCopy.begin(),filesToCopy.end());

    int count = copyProgress->minimum();    

    foreach(QString tmpFile, filesToCopy){  // перебор всех имен копируемых файлов

        tmpFile = QDir::fromNativeSeparators(tmpFile);

        //.oO0
        curFile->setText(QDir::toNativeSeparators(tmpFile));                  // имя текущего файла
        copyResult->setText(tr("copying") + "..."); // состояние процесса копирования
        copyProgress->setValue(++count);
        //0Oo.

        // чтобы не очень часто обновлялся экран
        int z;
        z = filesToCopy.count() / 100;
        if (z && ((count + z - 1) % z)) {
        } else {
            QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
        }

        // информация о файле-оригинале
        QFileInfo originalInfo(tmpFile);

        // информация о файле-копии
        QFileInfo copyInfo(tmpFile.replace(QDir::fromNativeSeparators(boxFolderName),cpyFolderName,Qt::CaseInsensitive));

        AltQFile originalFile(originalInfo.absoluteFilePath());  // файл-оригинал
        QFile copyFile(copyInfo.absoluteFilePath());          // файл-копия

        if (copyFile.exists()) {    // если файл-копия уже существует ...

            // ... проверить дату последнего изменения файла-копии
            if (copyInfo.lastModified() != originalInfo.lastModified()){ // если файл-копия не такой, то ...
                /*
                copyFile.setPermissions(QFile::ReadOwner|QFile::WriteOwner|
                                          QFile::ReadUser|QFile::WriteUser|
                                          QFile::ReadGroup|QFile::WriteGroup|
                                          QFile::ReadOther|QFile::WriteOther);
                */
                copyFile.remove();                                       // ... удалить его и ...
                if (!originalFile.copy(copyInfo.absoluteFilePath())){    // ... попробовать скопировать
                    //.oO0
                    copyResult->setText(tr("copying is not complete") + "...");   // при неудачном копировании
                    copyResultCode = ERROR_3;
                    //0Oo.
                    *log << "    Backup status: can not update...";
                    log->closeLog();
                    QApplication::restoreOverrideCursor();
                    return;                                              // сообщить и выйти
                }
            }
        } else {    // если файл-копия отсутствует ...
            if (!originalFile.copy(copyInfo.absoluteFilePath())){ // попробовать скопировать
                //.oO0
                copyResult->setText(tr("copying is not complete") + "..."); // при неудачной попытке
                copyResultCode = ERROR_3;
                //0Oo.
                *log << "    Backup status: can not update...";
                log->closeLog();
                QApplication::restoreOverrideCursor();
                return;                                            // сообщить и выйти
            }
        }

        //.oO0
        copyResult->setText(tr("copying is successful complete") + "...");  // сообщить об удачном завершении
        //0Oo.        
    }
    curFile->setText(tr("all"));
    copyResultCode = OK;

    *log << "    Backup status: actual";
    log->closeLog();    

    QApplication::restoreOverrideCursor();
}

void VerifyDialog::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange){
        retranslateUi(this);
        retranslateThis();
    }
    QFrame::changeEvent(event);
}

void VerifyDialog::setStatusNames(){
    status_ok = tr("ok");
    status_er = tr("caution");
    status_pr = "...";
}

void VerifyDialog::setStatusLabel(QLabel* label, int code)
{
    switch (code){
    case PROCESS:
        label->setText(status_pr);
        break;
    case OK:
        label->setText(status_ok);
        break;
    case ERROR_1:
        label->setText(status_er);
        break;
    default:;
    }
}

void VerifyDialog::setCopyStatusLabel()
{
    switch(copyResultCode){
    case PROCESS:
        label_GB->setText(tr("Folder") + ":");
        copyResult->setText("...");
        break;
    case OK:
        label_GB->setText(tr("File:"));
        curFile->setText(tr("all"));
        copyResult->setText(tr("copying is successful complete") + "...");
        break;
    case ERROR_1:
        label_GB->setText(tr("Folder") + ":");
        copyResult->setText(tr("Do not create backup folder") + "...");
        break;
    case ERROR_2:
        label_GB->setText(tr("Folder") + ":");
        copyResult->setText(tr("do not create folder") + "...");
        break;
    case ERROR_3:
        label_GB->setText(tr("File:"));
        copyResult->setText(tr("copying is not complete") + "...");
        break;
    default:;
    }
}

void VerifyDialog::retranslateThis()
{
    setStatusNames();
    setStatusLabel(sssStatus, sssStatusCode);
    setStatusLabel(wrdStatus, wrdStatusCode);
    setStatusLabel(abcStatus, abcStatusCode);
    setStatusLabel(fileStatus, fileStatusCode);
    setCopyStatusLabel();
}
