#ifndef VERIFYDIALOG_H
#define VERIFYDIALOG_H

#include <QFrame>
#include <QMap>

#include "ui_verifydialog.h"
#include "anybox.h"
#include "mysssfile.h"
#include "mytmpfile.h"
#include "myabcfile.h"
#include "mywrdfile.h"
#include "mylogfile.h"
#include "entry.h"
#include "basefolder.h"

/*
  Проверка mybox и создание резервной копии (widget)
*/

class VerifyDialog: public QFrame, public Ui::VerifyDialogGUI
{
    Q_OBJECT

    AnyBox* m_box;
    BaseFolder* m_base;

    MySssFile *sss;
    MyWrdFile *wrd;
    MyTmpFile *ttt;
    MyLogFile *log;

    QSet<uint> sssNums;             // список номеров сессий
    QSet<uint> sssDoubleNums;       // список сессий с одинаковыми номерами
    QSet<uint> sssAbsent;           // список сессий, которых нет в session-файле, но они есть в abc-файлах

    QSet<QString> words;            // список ключевых слов (индексы)
    QSet<QString> doubleWords;      // список одинаковых ключевых слов (одинаковые индексы)

    QSet<QString> doubleRemark;     // список ремарок, которые использованы многократно
    QMap<QString,QString> oddFiles; // список файлов, не имеющих отношения к MyBox, но находящиеся там
                                    // (ключ - ремарка, значение - полное имя файла)
    QString status_ok;
    QString status_er;
    QString status_pr;

    QMap<QString, QMap<QString,Entry> > abcEntries; // все abc-файлы со своим содержанием
                                                    // (ключ - полное имя abc-файла, значение - содержание)

    QMap<QString,QString> files; // список предметных файлов (ключ - ремарка, значение - контр. сумма)
    QList<QString> filesToCopy;  // список файлов для backup

    int totalFilesInBox();       // общее количество файлов для backup (без tmp, sss, wrd - файлов)

    bool sssOk(QString, QString*);  // проверка session-файла
    bool wrdOk(QString, QString*);  // проверка wrd-файла
    bool abcOk(QString, QString*);  // проверка abc-файла
    bool fileOk(QString*);          // проверка предметных файлов


    enum {PROCESS, OK, ERROR_1, ERROR_2, ERROR_3};
    int sssStatusCode, wrdStatusCode, abcStatusCode, fileStatusCode;
    void setStatusNames();
    void setStatusLabel(QLabel* label, int code);

    int copyResultCode;
    void setCopyStatusLabel();

    void changeEvent(QEvent *event);
    void retranslateThis();

public:    
    VerifyDialog(AnyBox*, QFrame*);

private slots:    
    void pressBackupButton();

};

#endif // VERIFYDIALOG_H
