#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QProcess>
#include <QMap>
#include <QFile>

#ifdef Q_OS_UNIX
#include <QSocketNotifier>
#endif

namespace Ui {    
    class MyMPlayer;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

    Ui::MyMPlayer *ui;

#ifdef Q_OS_UNIX
    static int sigtermFd[2];
    QSocketNotifier *snTerm;
#endif

    int pX, pY;

    QMap<QString,QString> m_sets;    // параметры командной строки (параметр - значение)

    QFile m_playlistFile;            // файл со списком воспроизведения
    QStringList m_playlist;          // список файлов для воспроизведения

    QStringList m_subTitles;         // данные из sub-файла
    QStringList m_subAuthors;
    QStringList m_subYears;

    QFile m_appPlay;                 // приложение для воспроизведения (mplayer)
    QString m_cmd;                   // строка запуска приложения для воспроизведения

    QList<QByteArray> codecNames;    // список названий текстовых кодеков
    QList<QTextCodec*> codecs;       // список текстовых кодеков
    QTextCodec *currentCodec;        // текущий кодек
    int currentCodecId;              // номер текущего текстового кодека

    bool play;
    int currentMelody;
    int currentVolume;
    int stepVolume;

    QMap<QString,QString> info;
    QVector<QString> infoTitle;

    int infoCurrent;

    QString unknown;

    QProcess *process;

    int timerVolumeDownId;
    int timerTimePosId;
    int timerChangeInfoId;
    int timerChkExitId;

    QString playlistFilename;

    float time;
    float timeA;
    float timeB;

    virtual void timerEvent(QTimerEvent*);
    void volumeDownSoft();
    void changeCurrentMelody(bool);

    void closeEvent(QCloseEvent *);
    void keyPressEvent(QKeyEvent*);
    void mousePressEvent(QMouseEvent*);
    void mouseReleaseEvent(QMouseEvent*);
    void mouseMoveEvent(QMouseEvent*);

    bool subData(const QString &, QString &, QString &, QString &);
    bool fileExists(const QString &);   // проверка существования файла в текущем каталоге и в каталогах PATH

public:
    MainWindow(const QString &args, QWidget *parent = 0);
    ~MainWindow();

#ifdef Q_OS_UNIX
    static void termSignalHandler(int unused);
#endif

public slots:
//#ifdef Q_OS_UNIX
    void handleSigTerm();
//#endif

private slots:
    void reqInfo();
    void getInfo();
    void pressNext();
    void pressPrev();    
    void pressPlay();
    void pressHome();
    void pressEnd();
    void pressOff();

    void playMelody();

};

#endif // MAINWINDOW_H
